/*
 * Copyright (C) 2015 Freescale Semiconductor, Inc. All Rights Reserved.
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/ctype.h>
#include <linux/types.h>
#include <linux/delay.h>
#include <linux/clk.h>
#include <linux/of_device.h>
#include <linux/i2c.h>
#include <linux/of_gpio.h>
#include <linux/pinctrl/consumer.h>
#include <linux/regulator/consumer.h>
#include <linux/fsl_devices.h>
#include <linux/mipi_csi2.h>
#include <media/v4l2-chip-ident.h>
#include "v4l2-int-device.h"
#include "mxc_v4l2_capture.h"

#define MAX_SENSOR_NUM	4

#define ADDR_MAX9286		0x6A
#define ADDR_MAX9271		0x40
#define ADDR_MAX9271_ALL	(ADDR_MAX9271 + MAX_SENSOR_NUM + 1)  //Broadcast address
#define ADDR_OV_SENSOR	0x30
#define ADDR_AP_SENSOR	0x5D

#ifdef CONFIG_SENSOR_OV10635
unsigned int g_max9286_width = 1280;
unsigned int g_max9286_height = 800;
#else
unsigned int g_max9286_width = 1280;
unsigned int g_max9286_height = 720;
#endif
unsigned int g_sensor_num = 0;
unsigned char g_sensor_is_there = 0;  //Bit 0~3 for 4 cameras, 0b1= is there; 0b0 = is not there.

/*!
 * Maintains the information on the current state of the sesor.
 */
static struct sensor_data max9286_data[MAX_SENSOR_NUM];

static int max9286_probe(struct i2c_client *adapter,
				const struct i2c_device_id *device_id);
static int max9286_remove(struct i2c_client *client);

static int ioctl_dev_init(struct v4l2_int_device *s);

static const struct i2c_device_id max9286_id[] = {
	{"max9286_mipi", 0},
	{},
};

MODULE_DEVICE_TABLE(i2c, max9286_id);

static struct i2c_driver max9286_i2c_driver = {
	.driver = {
		  .owner = THIS_MODULE,
		  .name  = "max9286_mipi",
		  },
	.probe  = max9286_probe,
	.remove = max9286_remove,
	.id_table = max9286_id,
};

#ifdef CONFIG_SENSOR_AP0101
typedef struct
{
    unsigned short reg;
    unsigned short val;
} reg_item_u16_t;

typedef struct
{
    unsigned short addr;
    unsigned short data_bits;
    unsigned int value;
} reg_item2_t;

enum SequencerID
{
    PAL_OUTPUT = 0 ,
    NTSC_OUTPUT,
    DEINTERLACE_PAL_480,
    DEINTERLACE_PAL_400,
    DEINTERLACE_NTSC_480,
    DEINTERLACE_NTSC_400,
    LSC_3P6mm,
    AETARGET_HIGH,
    AETARGET_NORMAIL,
    AETARGET_LOW,
};

//
//Host Command Code
//
#define HCC_SYSMGR_SET_STATE			0x8100
#define HCC_SYSMGR_GET_STATE			0x8101

//Overlay
#define HCC_OVERLAY_ENABLE_OVERLAY		0x8200
#define HCC_OVRL_SET_CALIBRATION		0x8202
#define HCC_OVRL_SET_BITMAP_PROP		0x8203
#define HCC_OVRL_GET_BITMAP_PROP		0x8204
#define HCC_OVERLAY_LOAD_BUFFER			0x8206

//Sequencer commands
#define HCC_SEQ_REFRESH				0x8606
#define HCC_SEQ_REFRESH_STATUS			0x8607

//GPIO Manager commands
#define HCC_GPIO_SET_PROP			0x8400
#define HCC_GPIO_GET_PROP			0x8401
#define HCC_GPIO_SET_STATE			0x8402
#define HCC_GPIO_GET_STATE			0x8403

//Sensor command
#define HCC_SENSOR_MGR_DISCOVER_SENSOR		0x8E00
#define HCC_SENSOR_MGR_INITIALIZE_SENSOR	0x8E01

//CCIM command
#define HCC_CCIMGR_GET_LOCK			0x8D00
#define HCC_CCIMGR_LOCK_STATUS			0x8D01
#define HCC_CCIMGR_RELEASE_LOCK			0x8D02
#define HCC_CCIMGR_WRITE			0x8D06
#define HCC_CCIMGR_READ				0x8D05
#define HCC_CCIMGR_WRITE_BITFIELD		0x8D07
#define HCC_CCIMGR_STATUS			0x8D08

//STE Manager commands
#define HCC_STEMGR_CONFIG			0x8310
#define HCC_STEMGR_WRITE_CONFIG			0x8313

//Patch commands
#define HCC_PATCH_LOAD_PATCH			0x8700
#define HCC_PATCH_STATUS			0x8701
#define HCC_PATCH_APPLY_PATCH			0x8702
#define HCC_PATCH_RESERVE_RAM			0x8706

//Miscellaneous commands
#define HCC_MISC_INVOKE_COMMAND_SEQ		0x8900

//
// AP0101 Registers
//
#define REG_CHIP_VERSION			0x0000
#define REG_COMMAND_REGISTER			0x0040
#define REG_CMD_HANDLER_PARAMS_POOL_0		0xFC00
#define REG_CMD_HANDLER_PARAMS_POOL_1		0xFC02
#define REG_CMD_HANDLER_PARAMS_POOL_2		0xFC04
#define REG_CMD_HANDLER_PARAMS_POOL_3		0xFC06
#define REG_CMD_HANDLER_PARAMS_POOL_4		0xFC08
#define REG_CMD_HANDLER_PARAMS_POOL_5		0xFC0A
#define REG_CMD_HANDLER_PARAMS_POOL_6		0xFC0C
#define REG_CMD_HANDLER_PARAMS_POOL_7		0xFC0E

#define REG_ACCESS_CTL_STAT			0x0982
#define REG_PHYSICAL_ADDRESS_ACCESS		0x098A
#define REG_LOGICAL_ADDRESS_ACCESS		0x098E
#define REG_PATCHLDR_APPLY_STATUS		0xE00C

#define REG_CAM_FRAME_SCAN_CONTROL		0xC8A4

#define REG_SYSMGR_CONFIG_MODE			0xDC07

//
//System State
//
#define SYS_STATE_OTPM_DEFAULTS			0x10
#define SYS_STATE_OTPM_CONFIG			0x11
#define SYS_STATE_OC_LOAD_DEFAULTS		0x12
#define SYS_STATE_OC_LOAD_PATCHES		0x13
#define SYS_STATE_OC_LOAD_CALIB			0x14
#define SYS_STATE_OC_CONFIG_TASKS		0x15
#define SYS_STATE_OC_COMPLETE			0x16
#define SYS_STATE_AUTO_CONFIG			0x17
#define SYS_STATE_HOST_CONFIG			0x18
#define SYS_STATE_FLASH_CONFIG			0x19
#define SYS_STATE_FC_LOAD_DEFAULTS		0x1A
#define SYS_STATE_FC_LOAD_PATCHES		0x1B
#define SYS_STATE_FC_LOAD_CALIB			0x1C
#define SYS_STATE_FC_CONFIG_TASKS		0x1D
#define SYS_STATE_FC_COMPLETE			0x1E
#define SYS_STATE_CONFIG_COMPLETE		0x1F
#define SYS_STATE_IDLE				0x20
#define SYS_STATE_SET_FEATURES			0x21
#define SYS_STATE_ENTER_CONFIG_CHANGE		0x28
#define SYS_STATE_DO_CONFIG_CHANGE		0x29
#define SYS_STATE_VALIDATE_CONFIG		0x2A
#define SYS_STATE_HARD_STANDBY_DURING_CONFIG	0x2B
#define SYS_STATE_CONFIG_POWER_ON		0x2C
#define SYS_STATE_CONFIG_POWER_OFF		0x2D
#define SYS_STATE_STREAMING			0x31
#define SYS_STATE_STOP_STREAMING		0x33
#define SYS_STATE_ENTER_STREAMING		0x34
#define SYS_STATE_ENTER_SUSPEND			0x40
#define SYS_STATE_SUSPENDED			0x41
#define SYS_STATE_DO_SUSPEND			0x42
#define SYS_STATE_ENTER_SOFT_STANDBY		0x50
#define SYS_STATE_DO_SOFT_STANDBY		0x51
#define SYS_STATE_DRIVER_SOFT_STANDBY		0x52
#define SYS_STATE_SOFT_STANDBY			0x53
#define SYS_STATE_DRIVER_LEAVE_SOFT_STANDBY	0x54
#define SYS_STATE_LEAVE_SOFT_STANDBY		0x55
#define SYS_STATE_ENTER_HARD_STANDBY		0x58
#define SYS_STATE_DO_HARD_STANDBY		0x59
#define SYS_STATE_DRIVER_HARD_STANDBY		0x5A
#define SYS_STATE_HARD_STANDBY			0x5B
#define SYS_STATE_DRIVER_LEAVE_HARD_STANDBY	0x5C
#define SYS_STATE_LEAVE_HARD_STANDBY		0x5D
#define SYS_STATE_ENTER_RESYNC			0x60
#define SYS_STATE_DO_RESYNC			0x61
#define SYS_STATE_UNKNOWN			0xFF

//
// Result Status
//
#define RESULT_ENOERR		0
#define RESULT_ENOENT		1
#define RESULT_EINTR		2
#define RESULT_EIO		3
#define RESULT_E2BIG		4
#define RESULT_EBADF		5
#define RESULT_EAGAIN		6
#define RESULT_ENOMEM		7
#define RESULT_EACCES		8
#define RESULT_EBUSY		9
#define RESULT_EEXIST		10
#define RESULT_ENODEV		11
#define RESULT_EINVAL		12
#define RESULT_ENOSPC		13
#define RESULT_ERANGE		14
#define RESULT_ENOSYS		15
#define RESULT_EALREADY		16
#define RESULT_EUNKNOW		255


static inline int ap0101_read_reg(int index, unsigned short reg, int len, void * pdata)
{
	unsigned char u8_buf[2] = {0};
	unsigned int value = 0;
	int retry, timeout = 20;

	if ((pdata == NULL) || (len > 4))
		return RESULT_EUNKNOW;

	u8_buf[0] = reg >> 8;
	u8_buf[1] = reg & 0xff;

	max9286_data[0].i2c_client->addr = ADDR_AP_SENSOR + index;
	for (retry = 0; retry < timeout; retry ++) {
		if (2 != i2c_master_send(max9286_data[0].i2c_client, u8_buf, 2)) {
			dev_dbg(&max9286_data[0].i2c_client->dev,
				"%s:write reg error: reg=%x, retry = %d.\n", __func__, reg, retry);
			msleep(10);
			continue;
		}

		if (len != i2c_master_recv(max9286_data[0].i2c_client, (char *)(&value), len)) {
			dev_dbg(&max9286_data[0].i2c_client->dev,
				"%s:read reg error: reg=%x,len=%d, retry = %d.\n", __func__, reg, len, retry);
			msleep(10);
			continue;
		}
		break;
	}

	if (retry >= timeout) {
		dev_err(&max9286_data[0].i2c_client->dev,
			"%s:read reg error: reg=%x,len=%d.\n", __func__, reg, len);
		return RESULT_EUNKNOW;
	}

	if (len == 1) {
		*(unsigned int *)pdata = value & 0xff;
	} else if (len == 2) {
		*(unsigned int *)pdata = ((value & 0xff) << 8) + ((value >> 8) & 0xff);
	} else if (len == 3) {
		*(unsigned int *)pdata = ((value & 0xff) << 16) + (value & 0xff00) + ((value >> 16) & 0xff);
	} else if (len == 4){
		*(unsigned int *)pdata = ((value & 0xff) << 24) + (((value >> 8) & 0xff) << 16) + (((value >> 16) & 0xff) << 8) + ((value >> 24) & 0xff);
	}

	return RESULT_ENOERR;
}

static inline int ap0101_write_reg(int index, unsigned short reg, int len, unsigned int val)
{
	unsigned char u8_buf[6] = {0};
	unsigned int buf_len = 2;
	int retry, timeout = 20;

	u8_buf[0] = reg >> 8;
	u8_buf[1] = reg & 0xff;

	switch (len) {
		case 1:
			u8_buf[2] = val & 0xff;
			break;
		case 2:
			u8_buf[2] = (val >> 8) & 0xff;
			u8_buf[3] = val & 0xff;
			break;
		default:
			u8_buf[2] = val >> 24;
			u8_buf[3] = (val >> 16) & 0xff;
			u8_buf[4] = (val >> 8) & 0xff;
			u8_buf[5] = val & 0xff;
			break;
	}
	buf_len += len;

	max9286_data[0].i2c_client->addr = ADDR_AP_SENSOR + index;
	for (retry = 0; retry < timeout; retry ++) {
		if (i2c_master_send(max9286_data[0].i2c_client, u8_buf, buf_len) < 0) {
			dev_dbg(&max9286_data[0].i2c_client->dev,
				"%s:write reg error: reg=0x%x, val=0x%x, retry = %d.\n", __func__, reg, val, retry);
			msleep(10);
			continue;
		}
		break;
	}

	if (retry >= timeout) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s:write reg error: reg=0x%x, val=0x%x.\n", __func__, reg, val);
		return RESULT_EUNKNOW;
	}

	return RESULT_ENOERR;
}

static int ap0101_CheckDoorBell(int index)
{
	unsigned short reg_data = 0;
	int result;

	result = ap0101_read_reg(index, REG_COMMAND_REGISTER, sizeof(reg_data), &reg_data);
	if (result == RESULT_ENOERR) {
		reg_data = reg_data & (1 << 15);
		if (reg_data != 0) {
			//AP0101 is busy;
			dev_dbg(&max9286_data[0].i2c_client->dev, "%s: AP0101 is busy.\n", __func__);
			return RESULT_EBUSY;
		} else
			return RESULT_ENOERR;
	} else {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s: read REG_COMMAND_REGISTER fail.\n", __func__);
	}

	return RESULT_EUNKNOW;
}

static int ap0101_GetResultStatus(int index)
{
	unsigned short reg_data = 0;
	int result;

	result = ap0101_read_reg(index, REG_COMMAND_REGISTER, sizeof(reg_data), &reg_data);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s: read REG_COMMAND_REGISTER fail.\n", __func__);
		return RESULT_EUNKNOW;
	}

	return reg_data & 0x7FFF;
}

static int ap0101_IssueHostCommand(int index, unsigned short command)
{
	int retry = 0, timeout = 20;
	int result;

	while (retry < timeout) {
		result = ap0101_CheckDoorBell(index);
		if (result == RESULT_ENOERR)
			break;

		retry += 1;
		dev_dbg(&max9286_data[0].i2c_client->dev, "%s: CheckDoorBell busy, retry = %d.\n", __func__, retry);
		msleep(50);
	}

	if (retry >= timeout) {
		dev_info(&max9286_data[0].i2c_client->dev, "%s: CheckDoorBell timeout.\n", __func__);
		return RESULT_EBUSY;
	}

	dev_dbg(&max9286_data[0].i2c_client->dev, "%s: CMD = 0x%x.\r\n", __func__, command);
	ap0101_write_reg(index, REG_COMMAND_REGISTER, 2, command);

	retry = 0;
	while (retry < timeout) {
		result = ap0101_CheckDoorBell(index);
		if (result == RESULT_ENOERR)
			return ap0101_GetResultStatus(index);

		retry += 1;
		dev_dbg(&max9286_data[0].i2c_client->dev, "%s: CheckDoorBell cmd busy, retry = %d.\n", __func__, retry);
		msleep(50);
	}

	dev_info(&max9286_data[0].i2c_client->dev, "%s: timeout.\n", __func__);

	return ap0101_GetResultStatus(index);
}

static int ap0101_GetLock(int index)
{
	int result = RESULT_ENOERR;

	result = ap0101_IssueHostCommand(index, HCC_CCIMGR_GET_LOCK);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, 
			"%s: HCC_CCIMGR_GET_LOCK error,  result = %d.\n", __func__, result);
		return result;
	}

	result = ap0101_IssueHostCommand(index, HCC_CCIMGR_LOCK_STATUS);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, 
			"%s: HCC_CCIMGR_LOCK_STATUS error, result = %d.\n", __func__, result);
	}

	return result;
}

static int ap0101_ReleaseLock(int index)
{
	int result = RESULT_ENOERR;

	result = ap0101_IssueHostCommand(index, HCC_CCIMGR_RELEASE_LOCK);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, 
			"%s: HCC_CCIMGR_RELEASE_LOCK error, result = %d.\n", __func__, result);
	}
	return result;
}

static int ap0101_WriteCmdParams(int index, 
		unsigned char param0, unsigned char param1, unsigned char param2, unsigned char param3,
		unsigned char param4, unsigned char param5, unsigned char param6, unsigned char param7,
		unsigned char param8, unsigned char param9, unsigned char param10, unsigned char param11,
		unsigned char param12, unsigned char param13, unsigned char param14, unsigned char param15)
{
	unsigned short value;
	int result = RESULT_ENOERR;

	value = (param0 << 8) | param1;
	result = ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_0, 2, value);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, 
			"%s: REG_CMD_HANDLER_PARAMS_POOL_0 error, result = %d.\n", __func__, result);
		return result;
	}

	value = (param2 << 8) | param3;
	ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_1, 2, value);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, 
			"%s: REG_CMD_HANDLER_PARAMS_POOL_1 error, result = %d.\n", __func__, result);
		return result;
	}

	value = (param4 << 8) | param5;
	ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_2, 2, value);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, 
			"%s: REG_CMD_HANDLER_PARAMS_POOL_2 error, result = %d.\n", __func__, result);
		return result;
	}

	value = (param6 << 8) | param7;
	ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_3, 2, value);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, 
			"%s: REG_CMD_HANDLER_PARAMS_POOL_3 error, result = %d.\n", __func__, result);
		return result;
	}

	value = (param8 << 8) | param9;
	ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_4, 2, value);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, 
			"%s: REG_CMD_HANDLER_PARAMS_POOL_4 error, result = %d.\n", __func__, result);
		return result;
	}

	value = (param10 << 8) | param11;
	ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_5, 2, value);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, 
			"%s: REG_CMD_HANDLER_PARAMS_POOL_5 error, result = %d.\n", __func__, result);
		return result;
	}

	value = (param12 << 8) | param13;
	ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_6, 2, value);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, 
			"%s: REG_CMD_HANDLER_PARAMS_POOL_6 error, result = %d.\n", __func__, result);
		return result;
	}

	value = (param14 << 8) | param15;
	ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_7, 2, value);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, 
			"%s: REG_CMD_HANDLER_PARAMS_POOL_7 error, result = %d.\n", __func__, result);
	}
	return result;
}

static int ap0101_WriteSensorSequencer(int index, unsigned short reg, unsigned short* sequencer, unsigned int length)
{
	unsigned int pools_num = length/6, reminder = length%6;
	unsigned short last_pool[]= {0, 0, 0, 0, 0, 0, 0, 0};
	unsigned short value;
	unsigned int i, timeout;
	int result = RESULT_ENOERR;

	ap0101_GetLock(index);
	
	for (i = 0; i<pools_num; i++) {
		value = reg; //entrance
		ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_0, 2, value);
		value = 0x0C00 | (sequencer[i*6] >> 8);
		ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_1, 2, value);
		value = ((sequencer[i*6] & 0xFF) << 8) | (sequencer[i*6+1] >> 8);
		ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_2, 2, value);
		value = ((sequencer[i*6+1] & 0xFF) << 8) | (sequencer[i*6+2] >> 8);
		ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_3, 2, value);
		value = ((sequencer[i*6+2] & 0xFF) << 8) | (sequencer[i*6+3] >> 8);
		ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_4, 2, value);
		value = ((sequencer[i*6+3] & 0xFF) << 8) | (sequencer[i*6+4] >> 8);
		ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_5, 2, value);
		value = ((sequencer[i*6+4] & 0xFF) << 8) | (sequencer[i*6+5] >> 8);
		ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_6, 2, value);
		value = (sequencer[i*6+5] & 0xFF) << 8;
		ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_7, 2, value);

		result = ap0101_IssueHostCommand(index, HCC_CCIMGR_WRITE);
		if (result != RESULT_ENOERR) {
			dev_info(&max9286_data[0].i2c_client->dev, 
				"%s: HCC_HCC_CCIMGR_WRITE result = %d.\n", __func__, result);
			goto EXIT;
		}

		timeout = 100;
		result = ap0101_IssueHostCommand(index, HCC_CCIMGR_STATUS);
		while (result != RESULT_ENOERR) {
			result = ap0101_IssueHostCommand(index, HCC_CCIMGR_STATUS);
			if(--timeout <= 0) {
				dev_info(&max9286_data[0].i2c_client->dev, 
					"%s: Time out error in WriteSensorSequencer.\n", __func__);
				goto EXIT;
			}
		}
	}

	if (reminder > 0) {
		for (i = length - reminder; i < length; i++) {
			last_pool[i - (length - reminder)] = sequencer[i];
		}

		value = reg;  //entrance
		ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_0, 2, value);
		value = ((reminder * 2) << 8) | (last_pool[0] >> 8);
		ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_1, 2, value);
		value = ((last_pool[0] & 0xFF) << 8) | (last_pool[1] >> 8);
		ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_2, 2, value);
		value = ((last_pool[1] & 0xFF) << 8) | (last_pool[2] >> 8);
		ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_3, 2, value);
		value = ((last_pool[2] & 0xFF) << 8) | (last_pool[3] >> 8);
		ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_4, 2, value);
		value = ((last_pool[3] & 0xFF) << 8) | (last_pool[4] >> 8);
		ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_5, 2, value);
		value = ((last_pool[4] & 0xFF) << 8) | (last_pool[5] >> 8);
		ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_6, 2, value);
		value = (last_pool[5] & 0xFF) << 8;
		ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_7, 2, value);

		result = ap0101_IssueHostCommand(index, HCC_CCIMGR_WRITE);
		if (result != RESULT_ENOERR) {
			dev_info(&max9286_data[0].i2c_client->dev, 
				"%s: HCC_HCC_CCIMGR_WRITE for remained, result = %d.\n", __func__, result);
			goto EXIT;
		}

		timeout = 100;
		result = ap0101_IssueHostCommand(index, HCC_CCIMGR_STATUS);
		while (result != RESULT_ENOERR) {
			result = ap0101_IssueHostCommand(index, HCC_CCIMGR_STATUS);
			if(--timeout <= 0) {
				dev_info(&max9286_data[0].i2c_client->dev, 
					"%s: Time out error in WriteSensorSequencer for remained.\n", __func__);
				goto EXIT;
			}
		}
	}

EXIT:
	ap0101_ReleaseLock(index);
	return result;
}

static int ap0101_WriteSensorRegList(int index, reg_item_u16_t *regs, int size)
{
	int i, timeout;
	unsigned char param0, param1, param2, param3, param4;
	int result = RESULT_ENOERR;

	ap0101_GetLock(index);

	for (i=0; i<size; i++) {
		param0 = regs[i].reg >> 8;
		param1 = regs[i].reg & 0xFF;
		param2 = 2;
		param3 = regs[i].val >> 8;
		param4 = regs[i].val & 0xFF;
		ap0101_WriteCmdParams(index, param0, param1, param2, param3, param4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);

		result = ap0101_IssueHostCommand(index, HCC_CCIMGR_WRITE);
		if (result != RESULT_ENOERR) {
			dev_info(&max9286_data[0].i2c_client->dev, 
				"%s: HCC_HCC_CCIMGR_WRITE error, i = %d, result = %d.\n", __func__, i, result);
			break;
		}

		timeout = 100;
		result = ap0101_IssueHostCommand(index, HCC_CCIMGR_STATUS);
		while (result != RESULT_ENOERR) {
			result = ap0101_IssueHostCommand(index, HCC_CCIMGR_STATUS);
			if(--timeout <= 0) {
				dev_info(&max9286_data[0].i2c_client->dev, 
					"%s: Time out error, i = %d.\n", __func__, i);
				goto EXIT;
			}
		}
	}

EXIT:
	ap0101_ReleaseLock(index);
	return result;
}

static int ap0101_WriteSensorReg(int index, unsigned short reg, unsigned short value)
{
	int timeout;
	unsigned char param0, param1, param2, param3, param4;
	int result = RESULT_ENOERR;

	ap0101_GetLock(index);

	param0 = reg >> 8;
	param1 = reg & 0xFF;
	param2 = 2;
	param3 = reg >> 8;
	param4 = reg & 0xFF;
	ap0101_WriteCmdParams(index, param0, param1, param2, param3, param4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);

	result = ap0101_IssueHostCommand(index, HCC_CCIMGR_WRITE);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, 
			"%s: HCC_HCC_CCIMGR_WRITE error, result = %d.\n", __func__, result);
		goto EXIT;
	}

	timeout = 100;
	result = ap0101_IssueHostCommand(index, HCC_CCIMGR_STATUS);
	while (result != RESULT_ENOERR) {
		result = ap0101_IssueHostCommand(index, HCC_CCIMGR_STATUS);
		if(--timeout <= 0) {
			dev_info(&max9286_data[0].i2c_client->dev, "%s: Time out error.\n", __func__);
			break;
		}
	}

EXIT:
	ap0101_ReleaseLock(index);
	return result;
}

static int ap0101_ReadSensorReg(int index, unsigned short reg, unsigned short * p_value)
{
	int timeout;
	unsigned short value = 0;
	unsigned char param0, param1, param2;
	int result = RESULT_ENOERR;

	ap0101_GetLock(index);

	param0 = reg >> 8;
	param1 = reg & 0xFF;
	param2 = 2;
	ap0101_WriteCmdParams(index, param0, param1, param2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);

	result = ap0101_IssueHostCommand(index, HCC_CCIMGR_READ);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, 
			"%s: HCC_HCC_CCIMGR_READ error, result = %d.\n", __func__, result);
		goto EXIT;
	}

	timeout = 100;
	result = ap0101_IssueHostCommand(index, HCC_CCIMGR_STATUS);
	while (result != RESULT_ENOERR) {
		result = ap0101_IssueHostCommand(index, HCC_CCIMGR_STATUS);
		if (--timeout <= 0) {
			dev_info(&max9286_data[0].i2c_client->dev, "%s: Time out error.\n", __func__);
			goto EXIT;
		}
	}

	result = ap0101_read_reg(index, REG_CMD_HANDLER_PARAMS_POOL_0, sizeof(value), &value);
	* p_value = value;

EXIT:
	ap0101_ReleaseLock(index);
	return result;
}

static int ap0101_SysmgrSetState(int index, unsigned short state)
{
	ap0101_WriteCmdParams(index, state, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
	return ap0101_IssueHostCommand(index, HCC_SYSMGR_SET_STATE);
}

static int ap0101_SysmgrGetState(int index, unsigned short * p_state)
{
	int result;
	unsigned short current_state = SYS_STATE_UNKNOWN;

	result = ap0101_IssueHostCommand(index, HCC_SYSMGR_GET_STATE);
	if (result == RESULT_ENOERR) {
		ap0101_read_reg(index, REG_CMD_HANDLER_PARAMS_POOL_0, sizeof(current_state), &current_state);
		current_state = current_state >> 8;
	} else {
		dev_info(&max9286_data[0].i2c_client->dev, "%s: failed, result = %d.\n", __func__, result);
	}

	*p_state = current_state;
	return result;
}

static int ap0101_SensorInitialization(int index)
{
	int result;

	result = ap0101_IssueHostCommand(index, HCC_SENSOR_MGR_INITIALIZE_SENSOR);
	switch (result) {
		case RESULT_ENOERR:
			break;
		case RESULT_EIO:
			dev_info(&max9286_data[0].i2c_client->dev, "%s: IO error occurred.\n", __func__);
			break;
		case RESULT_ENOENT:
			dev_info(&max9286_data[0].i2c_client->dev, "%s: No sensor attached.\n", __func__);
			break;
		case RESULT_EBADF:
			dev_info(&max9286_data[0].i2c_client->dev, "%s: Attached sensor is unsupported.\n", __func__);
			break;
		case RESULT_EBUSY:
			dev_info(&max9286_data[0].i2c_client->dev, "%s: Failed to obtain CCIM lock - retry.\n", __func__);
			break;
		case RESULT_EACCES:
			dev_info(&max9286_data[0].i2c_client->dev, "%s: Cannot perform sensor initialization at this time - retry.\n", __func__);
			break;
		default:
			dev_info(&max9286_data[0].i2c_client->dev, "%s: Unknown error occurred.\n", __func__);
			break;
	}

	return result;
}

static int ap0101_SysmgrDoSystemStateChange(int index, unsigned int next_state, unsigned int expected_state)
{
	int result;
	unsigned int timeout = 10;
	unsigned short current_state;

	//request the state change
	result = ap0101_SysmgrSetState(index, next_state);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, "%s: SysmgrSetState failed, result = %d.\n", __func__, result);
		return result;
	}

	//and wait for the change to complete
	// - if device returns EBUSY we continue to wait
	while (timeout != 0) {
		result = ap0101_SysmgrGetState(index, &current_state);
		if (result == RESULT_ENOERR)
			break;

		if (result != RESULT_EBUSY) {
			dev_info(&max9286_data[0].i2c_client->dev, "%s: SysmgrGetState failed, result = %d.\n", __func__, result);
			return result;
		}
		timeout -= 1;
	}

	//device failed to respond in time
	if (timeout == 0) {
		dev_info(&max9286_data[0].i2c_client->dev, "%s: Busy, result = %d.\n", __func__, result);
		return result;
	}

	//is device in the expected state?
	if (current_state != expected_state) {
		dev_info(&max9286_data[0].i2c_client->dev, "%s: Wrong state, result = %d.\n", __func__, result);
	}

	return result;
}

static int ap0101_ChangeConfig(int index)
{
	int result;

	result = ap0101_SysmgrDoSystemStateChange(index, SYS_STATE_ENTER_CONFIG_CHANGE, SYS_STATE_STREAMING);

	return result;
}

static int ap0101_SensorDiscovery(int index)
{
	int result;

	result = ap0101_IssueHostCommand(index, HCC_SENSOR_MGR_DISCOVER_SENSOR);
	switch (result) {
		case RESULT_ENOERR:
			break;
		case RESULT_EIO:
			dev_info(&max9286_data[0].i2c_client->dev, "%s: IO error occurred.\n", __func__);
			break;
		case RESULT_ENOENT:
			dev_info(&max9286_data[0].i2c_client->dev, "%s: Failed to detect any sensor.\n", __func__);
			break;
		case RESULT_EBUSY:
			dev_info(&max9286_data[0].i2c_client->dev, "%s: Failed to obtain CCIM lock - retry.\n", __func__);
			break;
		case RESULT_EACCES:
			dev_info(&max9286_data[0].i2c_client->dev, "%s: Cannot perform sensor discovery at this time - retry.\n", __func__);
			break;
		case RESULT_EALREADY:
			dev_info(&max9286_data[0].i2c_client->dev, "%s: The sensor has already been discovered. It can only be performed once.\n", __func__);
			break;
		default:
			dev_info(&max9286_data[0].i2c_client->dev, "%s: Unknown error occurred.\n", __func__);
			break;
	}

	return result;
}

static int ap0101_ReserveRAM(int index, unsigned short start, unsigned short size)
{
	int result;

	ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_0, 2, start);
	ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_1, 2, size);

	result = ap0101_IssueHostCommand(index, HCC_PATCH_RESERVE_RAM);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, "%s: PATCH_RESERVE_RAM error.\n", __func__);
	}

	return result;
}

static int ap0101_ApplyPatch(int index, unsigned short loader_addr, unsigned short patch_id, 
		unsigned short firmware_id_msb, unsigned short firmwareid_id_lsb, unsigned short reserve)
{
	int result;

	ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_0, 2, loader_addr);
	ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_1, 2, patch_id);
	ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_2, 2, firmware_id_msb);
	ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_3, 2, firmwareid_id_lsb);
	ap0101_write_reg(index, REG_CMD_HANDLER_PARAMS_POOL_4, 2, reserve);

	result = ap0101_IssueHostCommand(index, HCC_PATCH_APPLY_PATCH);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, "%s: PATCH_APPLY_PATCH error.\n", __func__);
		return result;
	}

	result = ap0101_IssueHostCommand(index, HCC_PATCH_STATUS);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, "%s: PATCH_STATUS error.\n", __func__);
	}

	return result;
}

static int ap0101_LastPatchStatus(int index)
{
	unsigned int reg_data = RESULT_EBADF;

	ap0101_read_reg(index, REG_PATCHLDR_APPLY_STATUS, 1, &reg_data);
	if (reg_data != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev, 
			"%s: Failed to apply patch: ERROR Code = 0x%x.\n", __func__, reg_data);
	}

	return reg_data;
}

static int ap0101_LoadPatch(int index, unsigned short start, unsigned short size, unsigned short phy_addr, 
		reg_item_u16_t *contents, unsigned short loader_addr, unsigned short patch_id, 
		unsigned short firmware_id_msb, unsigned short firmware_id_lsb, unsigned short reserve)
{
	int i;
	int result;

	result = ap0101_ReserveRAM(index, start, size);
	if (result != RESULT_ENOERR)
		return result;

	ap0101_write_reg(index, REG_ACCESS_CTL_STAT, 2, 0x0001);
	ap0101_write_reg(index, REG_PHYSICAL_ADDRESS_ACCESS, 2, phy_addr);

	for(i=0; i<(size/2); i++) {
		ap0101_write_reg(index, contents[i].reg, 2, contents[i].val);
	}
	ap0101_write_reg(index, REG_LOGICAL_ADDRESS_ACCESS, 2, 0x0000);

	result = ap0101_ApplyPatch(index, loader_addr, patch_id, firmware_id_msb, firmware_id_lsb, reserve);
	if (result != RESULT_ENOERR)
		return result;

	result = ap0101_LastPatchStatus(index);

	return result;
}

static void ap0101_WriteAP0101Regsiters(int index, reg_item2_t *settings, unsigned int length)
{
	int i;

	for (i=0; i<length; i++) {
		if (settings[i].addr == 0xFFFF)
			msleep(settings[i].value * 2);  //The wait function is not precise as expection, so double the time.
		else
			ap0101_write_reg(index, settings[i].addr, (settings[i].data_bits / 8), settings[i].value);
	}
}

reg_item2_t ap0101_settings[] = {
	//Step2-PLL_Timing - YUV8 720p30
	{0xCA84, 8, 0x05},	// CAM_SYSCTL_PLL_CONTROL
	{0xCA88, 16, 0x0462},	// CAM_SYSCTL_PLL_DIVIDER_M_N_1_CLK
	{0xCA8A, 16, 0x0462},	// CAM_SYSCTL_PLL_DIVIDER_M_N_2_CLK
	{0xCA8C, 16, 0x00FF},	// CAM_SYSCTL_PLL_DIVIDER_P_1_CLK
	{0xCA8E, 16, 0x007F},	// CAM_SYSCTL_PLL_DIVIDER_P_2_CLK
	{0xCA9C, 16, 0x0405},	// CAM_PORT_PARALLEL_CONTROL, YUV8
	{0xC838, 32, 0x080210C5},	// CAM_SENSOR_CONTROL_EXTERNAL_PLL
	{0xC804, 16, 0x000C},	// CAM_SENSOR_CFG_Y_ADDR_START
	{0xC806, 16, 0x0020},	// CAM_SENSOR_CFG_X_ADDR_START
	{0xC808, 16, 0x02DF},	// CAM_SENSOR_CFG_Y_ADDR_END
	{0xC80A, 16, 0x051F},	// CAM_SENSOR_CFG_X_ADDR_END
	{0xC80C, 32, 0x0230C2B0},	// CAM_SENSOR_CFG_PIXCLK
	{0xC810, 16, 0x02BC},	// CAM_SENSOR_CFG_FINE_INTEG_TIME_MIN
	{0xC812, 16, 0x0672},	// CAM_SENSOR_CFG_FINE_INTEG_TIME_MAX
	{0xC814, 16, 0x02EE},	// CAM_SENSOR_CFG_FRAME_LENGTH_LINES
	{0xC816, 16, 0x0672},	// CAM_SENSOR_CFG_LINE_LENGTH_PCK
	{0xC818, 16, 0x0000},	// CAM_SENSOR_CFG_FINE_CORRECTION
	{0xC844, 16, 0x0800},	// CAM_SENSOR_CONTROL_OPERATION_MODE
	{0xC846, 16, 0x0000},	// CAM_SENSOR_CONTROL_READ_MODE
	{0xC89C, 16, 0x0000},	// CAM_CROP_WINDOW_XOFFSET
	{0xC89E, 16, 0x0000},	// CAM_CROP_WINDOW_YOFFSET
	{0xC8A0, 16, 0x0500},	// CAM_CROP_WINDOW_WIDTH
	{0xC8A2, 16, 0x02D4},	// CAM_CROP_WINDOW_HEIGHT
	{0xC8A8, 8, 0x00},	// CAM_FOV_CALIB_X_OFFSET
	{0xC8A9, 8, 0x00},	// CAM_FOV_CALIB_Y_OFFSET
	{0xCA90, 16, 0x0500},	// CAM_OUTPUT_WIDTH
	{0xCA92, 16, 0x02D4},	// CAM_OUTPUT_HEIGHT
	{0xC8BC, 8, 0x00},	// CAM_AET_AEMODE
	{0xC8D4, 16, 0x0000},	// CAM_AET_FRAME_RATE_0
	{0xC8D6, 16, 0x0000},	// CAM_AET_FRAME_RATE_1
	{0xC8D8, 16, 0x0000},	// CAM_AET_FRAME_RATE_2

	//CCM
	{0xC924, 16, 0x0A8C},	// CAM_AWB_COLOR_TEMPERATURE_MIN
	{0xC926, 16, 0x1964},	// CAM_AWB_COLOR_TEMPERATURE_MAX
	{0xC982, 8, 0x82},	// CAM_AWB_K_R_L
	{0xC983, 8, 0x80},	// CAM_AWB_K_G_L
	{0xC984, 8, 0x86},	// CAM_AWB_K_B_L
	{0xC985, 8, 0x84},	// CAM_AWB_K_R_R
	{0xC986, 8, 0x82},	// CAM_AWB_K_G_R
	{0xC987, 8, 0x80},	// CAM_AWB_K_B_R
	{0xC980, 16, 0x1450},	// CAM_AWB_TINTS_CTEMP_THRESHOLD
	{0xC91A, 16, 0x00AC},	// CAM_AWB_CCM_R_RG_GAIN
	{0xC91C, 16, 0x00B6},	// CAM_AWB_CCM_R_BG_GAIN
	{0xC916, 16, 0x00AC},	// CAM_AWB_CCM_M_RG_GAIN
	{0xC918, 16, 0x0110},	// CAM_AWB_CCM_M_BG_GAIN
	{0xC912, 16, 0x006A},	// CAM_AWB_CCM_L_RG_GAIN
	{0xC914, 16, 0x0130},	// CAM_AWB_CCM_L_BG_GAIN
	{0xC8DC, 16, 0x0099},	// CAM_AWB_CCM_L_0
	{0xC8DE, 16, 0x0033},	// CAM_AWB_CCM_L_1
	{0xC8E0, 16, 0x0034},	// CAM_AWB_CCM_L_2
	{0xC8E2, 16, 0xFFB0},	// CAM_AWB_CCM_L_3
	{0xC8E4, 16, 0x00FB},	// CAM_AWB_CCM_L_4
	{0xC8E6, 16, 0x0055},	// CAM_AWB_CCM_L_5
	{0xC8E8, 16, 0xFFC7},	// CAM_AWB_CCM_L_6
	{0xC8EA, 16, 0xFF98},	// CAM_AWB_CCM_L_7
	{0xC8EC, 16, 0x01A0},	// CAM_AWB_CCM_L_8
	{0xC8EE, 16, 0x00FD},	// CAM_AWB_CCM_M_0
	{0xC8F0, 16, 0xFFF8},	// CAM_AWB_CCM_M_1
	{0xC8F2, 16, 0x000B},	// CAM_AWB_CCM_M_2
	{0xC8F4, 16, 0xFFE5},	// CAM_AWB_CCM_M_3
	{0xC8F6, 16, 0x00E7},	// CAM_AWB_CCM_M_4
	{0xC8F8, 16, 0x0034},	// CAM_AWB_CCM_M_5
	{0xC8FA, 16, 0x0003},	// CAM_AWB_CCM_M_6
	{0xC8FC, 16, 0xFFF8},	// CAM_AWB_CCM_M_7
	{0xC8FE, 16, 0x0106},	// CAM_AWB_CCM_M_8
	{0xC900, 16, 0x0096},	// CAM_AWB_CCM_R_0
	{0xC902, 16, 0x0051},	// CAM_AWB_CCM_R_1
	{0xC904, 16, 0x001D},	// CAM_AWB_CCM_R_2
	{0xC906, 16, 0xFFE0},	// CAM_AWB_CCM_R_3
	{0xC908, 16, 0x00EB},	// CAM_AWB_CCM_R_4
	{0xC90A, 16, 0x0023},	// CAM_AWB_CCM_R_5
	{0xC90C, 16, 0x0000},	// CAM_AWB_CCM_R_6
	{0xC90E, 16, 0xFFFF},	// CAM_AWB_CCM_R_7
	{0xC910, 16, 0x0101},	// CAM_AWB_CCM_R_8
	{0xC91E, 16, 0x0A8C},	// CAM_AWB_CCM_L_CTEMP
	{0xC920, 16, 0x0FA0},	// CAM_AWB_CCM_M_CTEMP
	{0xC922, 16, 0x1964},	// CAM_AWB_CCM_R_CTEMP

	//AWB
	{0xC92A, 16, 0x0020},	// CAM_AWB_X_SHIFT
	{0xC92C, 16, 0x0014},	// CAM_AWB_Y_SHIFT
	{0xC92E, 16, 0x0075},	// CAM_AWB_RECIP_X_SCALE
	{0xC930, 16, 0x0055},	// CAM_AWB_RECIP_Y_SCALE
	{0xC932, 16, 0xFFF4},	// CAM_AWB_ROT_CENTER_X
	{0xC934, 16, 0xFFE2},	// CAM_AWB_ROT_CENTER_Y
	{0xC936, 8, 0x32},	// CAM_AWB_ROT_SIN
	{0xC937, 8, 0x28},	// CAM_AWB_ROT_COS
	{0xC938, 16, 0x0000},	// CAM_AWB_WEIGHT_TABLE_0
	{0xC93A, 16, 0x0110},	// CAM_AWB_WEIGHT_TABLE_1
	{0xC93C, 16, 0x0000},	// CAM_AWB_WEIGHT_TABLE_2
	{0xC93E, 16, 0x0000},	// CAM_AWB_WEIGHT_TABLE_3
	{0xC940, 16, 0x0000},	// CAM_AWB_WEIGHT_TABLE_4
	{0xC942, 16, 0x1707},	// CAM_AWB_WEIGHT_TABLE_5
	{0xC944, 16, 0x7300},	// CAM_AWB_WEIGHT_TABLE_6
	{0xC946, 16, 0x0000},	// CAM_AWB_WEIGHT_TABLE_7
	{0xC948, 16, 0x0001},	// CAM_AWB_WEIGHT_TABLE_8
	{0xC94A, 16, 0x0007},	// CAM_AWB_WEIGHT_TABLE_9
	{0xC94C, 16, 0x7700},	// CAM_AWB_WEIGHT_TABLE_10
	{0xC94E, 16, 0x0000},	// CAM_AWB_WEIGHT_TABLE_11
	{0xC950, 16, 0x0120},	// CAM_AWB_WEIGHT_TABLE_12
	{0xC952, 16, 0x0004},	// CAM_AWB_WEIGHT_TABLE_13
	{0xC954, 16, 0x3470},	// CAM_AWB_WEIGHT_TABLE_14
	{0xC956, 16, 0x0200},	// CAM_AWB_WEIGHT_TABLE_15
	{0xC958, 16, 0x1000},	// CAM_AWB_WEIGHT_TABLE_16
	{0xC95A, 16, 0x0007},	// CAM_AWB_WEIGHT_TABLE_17
	{0xC95C, 16, 0x0007},	// CAM_AWB_WEIGHT_TABLE_18
	{0xC95E, 16, 0x1710},	// CAM_AWB_WEIGHT_TABLE_19
	{0xC960, 16, 0x0077},	// CAM_AWB_WEIGHT_TABLE_20
	{0xC962, 16, 0x7770},	// CAM_AWB_WEIGHT_TABLE_21
	{0xC964, 16, 0x0007},	// CAM_AWB_WEIGHT_TABLE_22
	{0xC966, 16, 0x7700},	// CAM_AWB_WEIGHT_TABLE_23
	{0xC968, 16, 0x0011},	// CAM_AWB_WEIGHT_TABLE_24
	{0xC96A, 16, 0x1200},	// CAM_AWB_WEIGHT_TABLE_25
	{0xC96C, 16, 0x0011},	// CAM_AWB_WEIGHT_TABLE_26
	{0xC96E, 16, 0x1100},	// CAM_AWB_WEIGHT_TABLE_27
	{0xC970, 16, 0x0000},	// CAM_AWB_WEIGHT_TABLE_28
	{0xC972, 16, 0x0000},	// CAM_AWB_WEIGHT_TABLE_29
	{0xC974, 16, 0x0000},	// CAM_AWB_WEIGHT_TABLE_30
	{0xC976, 16, 0x0000},	// CAM_AWB_WEIGHT_TABLE_31

	//Step6-CPIPE_Calibration
	{0xC888, 16, 0x0080},	// CAM_CPIPE_CONTROL_FIRST_BLACK_LEVEL
	{0xC97D, 8, 0x10},	// CAM_AWB_MODE
	{0xBC02, 16, 0x01C5},	// LL_MODE
	{0xC81C, 16, 0x0400},	// RESERVED_CAM_1C
	{0xC828, 16, 0x0054},	// RESERVED_CAM_28
	{0xC8CE, 16, 0x0035},	// RESERVED_CAM_CE
	{0xC8CA, 16, 0x002F},	// CAM_AET_AE_MIN_VIRT_AGAIN
	{0xC8CC, 16, 0x0180},	// CAM_AET_AE_MAX_VIRT_AGAIN
	{0xC8C6, 16, 0x008C},	// CAM_AET_AE_MIN_VIRT_DGAIN
	{0xC8C8, 16, 0x03FF},	// CAM_AET_AE_MAX_VIRT_DGAIN
	{0xC99E, 16, 0x0001},	// CAM_ALTM_K1_DAMPING_SPEED
	{0xC848, 16, 0x000B},	// CAM_HDR_MC_CTRL_MODE
	{0xC84C, 16, 0x0E74},	// CAM_HDR_MC_CTRL_S2_THRESHOLD
	{0xC858, 16, 0x0E74},	// CAM_HDR_DLO_CTRL_T2_BARRIER
	{0x2414, 16, 0x0BA0},	// RESERVED_CPIPE_ALTM_REGS_2414
	{0x2416, 16, 0x0FA0},	// RESERVED_CPIPE_ALTM_REGS_2416
	{0xC404, 16, 0x783C},	// RESERVED_EXT_04
	{0xC406, 16, 0x787C},	// RESERVED_EXT_06
	{0xC9E6, 8, 0x32},	// CAM_LL_DEMOSAIC_HIGH
	{0xC9E7, 8, 0x05},	// CAM_LL_DEMOSAIC_LOW
	{0xC9EA, 8, 0x19},	// CAM_LL_AP_THRESH_HIGH
	{0xC9EB, 8, 0x0A},	// CAM_LL_AP_THRESH_LOW
	{0x3222, 16, 0x0912},	// GRB_POS_THRESHOLDS
	{0x3224, 16, 0x0612},	// GRB_NEG_THRESHOLDS
	{0xCA34, 16, 0x0030},	// CAM_LL_NR_LUT_0_GAIN
	{0xCA38, 16, 0x0073},	// CAM_LL_NR_LUT_0_K0
	{0xCACA, 16, 0x0073},	// CAM_LL2_NR_LUT_T2_0_K0
	{0xCA3C, 16, 0x0093},	// CAM_LL_NR_LUT_1_GAIN
	{0xCA40, 16, 0x0073},	// CAM_LL_NR_LUT_1_K0
	{0xCACE, 16, 0x0073},	// CAM_LL2_NR_LUT_T2_1_K0
	{0xCA44, 16, 0x0200},	// CAM_LL_NR_LUT_2_GAIN
	{0xCA48, 16, 0x0073},	// CAM_LL_NR_LUT_2_K0
	{0xCAD2, 16, 0x0073},	// CAM_LL2_NR_LUT_T2_2_K0
	{0xCA4C, 16, 0x0400},	// CAM_LL_NR_LUT_3_GAIN
	{0xCA50, 16, 0x0073},	// CAM_LL_NR_LUT_3_K0
	{0xCAD6, 16, 0x0073},	// CAM_LL2_NR_LUT_T2_3_K0
	{0xCA36, 16, 0x004F},	// CAM_LL_NR_LUT_0_SIGMA
	{0xCA3E, 16, 0x005D},	// CAM_LL_NR_LUT_1_SIGMA
	{0xCA46, 16, 0x00B0},	// CAM_LL_NR_LUT_2_SIGMA
	{0xCA4E, 16, 0x0159},	// CAM_LL_NR_LUT_3_SIGMA
	{0xCAC8, 16, 0x004F},	// CAM_LL2_NR_LUT_T2_0_SIGMA
	{0xCACC, 16, 0x005D},	// CAM_LL2_NR_LUT_T2_1_SIGMA
	{0xCAD0, 16, 0x00B0},	// CAM_LL2_NR_LUT_T2_2_SIGMA
	{0xCAD4, 16, 0x0159},	// CAM_LL2_NR_LUT_T2_3_SIGMA
	{0xCAE8, 32, 0x000007D0},	// CAM_LL2_NR_TRANS_PT_S1
	{0xCAEC, 32, 0x00000DAC},	// CAM_LL2_NR_TRANS_PT_S2
	{0xC9E0, 16, 0x0C80},	// CAM_LL_HIGH_GM
	{0xC9E2, 16, 0x002D},	// CAM_LL_LOW_GM
	{0xC9DE, 16, 0x0BB8},	// CAM_LL_BRIGHT_BM
	{0xC9E0, 16, 0x10E3},	// CAM_LL_HIGH_GM
	{0xA814, 8, 0x08},	// AE_TRACK_ADAPT_THRESH
	{0xA81E, 8, 0x8C},	// AE_TRACK_MIN_GAIN_GATE
	{0x3406, 16, 0x2A00},	// HUE4_Q1Q2
	{0x3404, 16, 0x2A00},	// HUE3_Q1Q2
	{0x3408, 16, 0x3700},	// HUE5_Q1Q2
	{0x3418, 16, 0x0008},	// HUE13_Q3Q4
	{0x341E, 16, 0x000C},	// HUE16_Q3Q4
	{0x340C, 16, 0x2A00},	// HUE7_Q1Q2
	{0x3210, 16, 0x0CB0},	// COLOR_PIPELINE_CONTROL
	{0xBC02, 16, 0x01C5},	// LL_MODE
	{0xCAB4, 16, 0x0003},	// CAM_FLICKER_DETECT_FD_MODE
	
	//TX_SS
	{0x3C84, 16, 0xFFFF}, 	// TX_FRONTPORCH_BACKPORCH

	//Step7-CPIPE_Preference
	{0xCA2C, 16, 0x0006},	// CAM_LL_ADACD_GR_WEIGHTS_STRENGTH_LOW
	{0xCA2E, 16, 0x0003},	// CAM_LL_ADACD_GR_WEIGHTS_STRENGTH_HIGH
	{0xCA30, 16, 0x03E8},	// CAM_LL_ADACD_GR_WEIGHTS_LOW_SNR
	{0xCA32, 16, 0x0D00},	// CAM_LL_ADACD_GR_WEIGHTS_HIGH_SNR
	{0xC9CC, 16, 0xF8C0},	// CAM_LL_BM_OFFSET
	{0xA82C, 16, 0x078D},	// AE_TRACK_LOG_Y_TARGET_SDR_0
	{0xA82E, 16, 0x0833},	// AE_TRACK_LOG_Y_TARGET_SDR_1
	{0xA830, 16, 0x08B3},	// AE_TRACK_LOG_Y_TARGET_SDR_2
	{0xA832, 16, 0x08F3},	// AE_TRACK_LOG_Y_TARGET_SDR_3
	{0xA834, 16, 0x0926},	// AE_TRACK_LOG_Y_TARGET_SDR_4
	{0xA836, 16, 0x096E},	// AE_TRACK_LOG_Y_TARGET_SDR_5
	{0xA838, 16, 0x099F},	// AE_TRACK_LOG_Y_TARGET_SDR_6
	{0xA83A, 16, 0x09D0},	// AE_TRACK_LOG_Y_TARGET_SDR_7
	{0xA83C, 16, 0x07F6},	// AE_TRACK_LOG_Y_TARGET_HDR_0
	{0xA83E, 16, 0x084D},	// AE_TRACK_LOG_Y_TARGET_HDR_1
	{0xA840, 16, 0x08B3},	// AE_TRACK_LOG_Y_TARGET_HDR_2
	{0xA842, 16, 0x08F3},	// AE_TRACK_LOG_Y_TARGET_HDR_3
	{0xA844, 16, 0x0936},	// AE_TRACK_LOG_Y_TARGET_HDR_4
	{0xA846, 16, 0x099A},	// AE_TRACK_LOG_Y_TARGET_HDR_5
	{0xA848, 16, 0x09CC},	// AE_TRACK_LOG_Y_TARGET_HDR_6
	{0xA84A, 16, 0x09F3},	// AE_TRACK_LOG_Y_TARGET_HDR_7
	{0xA84C, 16, 0x03FE},	// RESERVED_AE_TRACK_4C
	{0xA84E, 16, 0x0300},	// RESERVED_AE_TRACK_4E
	{0xA850, 16, 0x0200},	// RESERVED_AE_TRACK_50
	{0xC9FF, 8, 0x16},	// CAM_LL_SDC_DP_STRENGTH_BRIGHT
	{0xC9FE, 8, 0x02},	// CAM_LL_SDC_DP_STRENGTH_DARK
	{0xCA05, 8, 0x16},	// CAM_LL_SDC_HP_STRENGTH_BRIGHT
	{0xCA04, 8, 0x02},	// CAM_LL_SDC_HP_STRENGTH_DARK
	{0xCA0B, 8, 0x04},	// CAM_LL_SDC_CROSSFACTOR_STRENGTH_BRIGHT
	{0xCA0A, 8, 0x14},	// CAM_LL_SDC_CROSSFACTOR_STRENGTH_DARK
	{0xCA11, 8, 0x01},	// CAM_LL_SDC_MAXFACTOR_STRENGTH_BRIGHT
	{0xCA10, 8, 0x01},	// CAM_LL_SDC_MAXFACTOR_STRENGTH_DARK
	{0xCA1B, 8, 0x16},	// CAM_LL_CDC_DP_STRENGTH_BRIGHT
	{0xCA1A, 8, 0x01},	// CAM_LL_CDC_DP_STRENGTH_DARK
	{0xCA21, 8, 0x16},	// CAM_LL_CDC_HP_STRENGTH_BRIGHT
	{0xCA20, 8, 0x01},	// CAM_LL_CDC_HP_STRENGTH_DARK
	{0xCA27, 8, 0x05},	// CAM_LL_CDC_CROSSFACTOR_STRENGTH_BRIGHT
	{0xCA26, 8, 0x1E},	// CAM_LL_CDC_CROSSFACTOR_STRENGTH_DARK
	{0xC9FC, 16, 0x0B54},	// CAM_LL_SDC_DP_BRIGHT_BM
	{0xC9FA, 16, 0x00C8},	// CAM_LL_SDC_DP_DARK_BM
	{0xCA02, 16, 0x0B54},	// CAM_LL_SDC_HP_BRIGHT_BM
	{0xCA00, 16, 0x00C8},	// CAM_LL_SDC_HP_DARK_BM
	{0xCA08, 16, 0x0B54},	// CAM_LL_SDC_CROSSFACTOR_BRIGHT_BM
	{0xCA06, 16, 0x00C8},	// CAM_LL_SDC_CROSSFACTOR_DARK_BM
	{0xCA0E, 16, 0x0B54},	// CAM_LL_SDC_MAXFACTOR_BRIGHT_BM
	{0xCA0C, 16, 0x00C8},	// CAM_LL_SDC_MAXFACTOR_DARK_BM
	{0xCA18, 16, 0x0B54},	// CAM_LL_CDC_DP_BRIGHT_BM
	{0xCA16, 16, 0x00C8},	// CAM_LL_CDC_DP_DARK_BM
	{0xCA1E, 16, 0x0B54},	// CAM_LL_CDC_HP_BRIGHT_BM
	{0xCA1C, 16, 0x00C8},	// CAM_LL_CDC_HP_DARK_BM
	{0xCA24, 16, 0x0B54},	// CAM_LL_CDC_CROSSFACTOR_BRIGHT_BM
	{0xCA22, 16, 0x00C8},	// CAM_LL_CDC_CROSSFACTOR_DARK_BM
	{0xC990, 16, 0x0012},	// CAM_ALTM_LO_GAMMA
	{0xC992, 16, 0x0020},	// CAM_ALTM_HI_GAMMA
	{0xBC07, 8, 0x00},	// LL_GAMMA_SELECT
	{0xC9F0, 16, 0x0064},	// CAM_LL_GAMMA
	{0xC8BE, 16, 0x0000},	// CAM_AET_BLACK_CLIPPING_TARGET
	{0xC9E2, 16, 0x0030},	// CAM_LL_LOW_GM
	{0xC9E0, 16, 0x0F00},	// CAM_LL_HIGH_GM
	{0xC9EC, 16, 0x0380},	// CAM_LL_CONTRAST_BRIGHT_BM
	{0xC9EE, 16, 0x0000},	// CAM_LL_CONTRAST_DARK_BM
	{0xC9F2, 8, 0x28},	// CAM_LL_CONTRAST_GRADIENT_BRIGHT
	{0xC9F3, 8, 0x2D},	// CAM_LL_CONTRAST_GRADIENT_DARK
	{0xC9F4, 8, 0x80},	// CAM_LL_CONTRAST_INTERCEPT_POINT_BRIGHT
	{0xC9F5, 8, 0xFF},	// CAM_LL_CONTRAST_INTERCEPT_POINT_DARK
	{0xC9F0, 16, 0x0064},	// CAM_LL_GAMMA
	{0xC99A, 16, 0x0600},	// CAM_ALTM_DARK_BM
	{0xC99C, 16, 0x0B00},	// CAM_ALTM_BRIGHT_BM
	{0xC98A, 16, 0x0080},	// CAM_ALTM_KEY_K0
	{0xC9A0, 16, 0x00C8},	// CAM_ALTM_SHARPNESS_DARK_BM
	{0xC9A2, 16, 0x0B54},	// CAM_ALTM_SHARPNESS_BRIGHT_BM
	{0xC9A4, 16, 0x0000},	// CAM_ALTM_SHARPNESS_STRENGTH_DARK
	{0xC988, 16, 0x0017},	// CAM_ALTM_MODE
	{0xC994, 16, 0x00AF},	// CAM_ALTM_K1_SLOPE
	{0xC996, 16, 0x1000},	// CAM_ALTM_K1_MIN
	{0xBCB8, 16, 0x0004},	// LL_ALTM_DAMPING_SLOW
	{0x2402, 16, 0x0004},	// RESERVED_CPIPE_ALTM_REGS_2402
	{0x2418, 16, 0x0090},	// RESERVED_CPIPE_ALTM_REGS_2418
	{0x241A, 16, 0x0061},	// RESERVED_CPIPE_ALTM_REGS_241A
	{0x241C, 16, 0x0041},	// RESERVED_CPIPE_ALTM_REGS_241C
	{0x244A, 16, 0x0007},	// RESERVED_CPIPE_ALTM_REGS_244A
	{0xC454, 16, 0x0000},
	{0xC456, 16, 0x003B},
	{0xC450, 16, 0xFFFF},
	{0xC452, 16, 0x00E6},
	{0xC44C, 16, 0x0010},
	{0xC44E, 16, 0x0014},
	{0xCAF8, 16, 0xFD00},	// CAM_ALTM2_LOWLIGHT_DARK_BM
	{0xCAFA, 16, 0x0100},	// CAM_ALTM2_LOWLIGHT_BRIGHT_BM
	{0xC9E8, 8, 0x01},	// CAM_LL_AP_GAIN_DARK
	{0xC9E9, 8, 0x03},	// CAM_LL_AP_GAIN_BRIGHT
	{0xCA58, 16, 0x0B00},	// CAM_LL_CK_0_SNR
	{0xCA64, 16, 0x0880},	// CAM_LL_CK_1_SNR
	{0xCA70, 16, 0x0066},	// CAM_LL_CK_2_SNR
	{0xCA6C, 16, 0x01CD},	// CAM_LL_CK_1_CHROMA_GAIN_HIGH
	{0xCA60, 16, 0x0200},	// CAM_LL_CK_0_CHROMA_GAIN_HIGH
	{0xCA6A, 16, 0x0022},	// RESERVED_CAM_26A

	//Step8-Features
//	{0xC88C, 8, 0x0},	// CAM_MODE_SELECT
	{0xC88C, 8, 0x3},	// CAM_MODE_SELECT
	{0xC88D, 8, 0x3},       // CAM_MODE_SYNC_TYPE
	{0xC88E, 8, 0x0},	// CAM_MODE_SYNC_TRIGGER_MODE
};

static void ap0101_InitAp0101Pipeline(int index)
{
	ap0101_WriteAP0101Regsiters(index, ap0101_settings, sizeof(ap0101_settings) / sizeof(ap0101_settings[0]));
}

static void ap0101_LoadSensorSequencer(int index)
{
	unsigned short sequencer[] = {
		0x4558,0x6E9B,0x4A31,0x4342,0x8E03,0x2714,0x4578,0x7B3D,
		0xFF3D,0xFF3D,0xEA27,0x043D,0x1027,0x0527,0x1535,0x2705,
		0x3D10,0x4558,0x2704,0x2714,0x3DFF,0x3DFF,0x3DEA,0x2704,
		0x6227,0x288E,0x0036,0x2708,0x3D64,0x7A3D,0x0444,0x2C4B,
		0x8F00,0x4373,0x719F,0x6343,0x166F,0x9F92,0x1244,0x1663,
		0x4316,0x9326,0x0426,0x848E,0x0327,0xFC5C,0x0D57,0x5417,
		0x0955,0x5649,0x5F53,0x0553,0x0728,0x6C4C,0x0928,0x2C72,
		0xAD7C,0xA928,0xA879,0x6026,0x9C5C,0x1B45,0x4845,0x0845,
		0x8826,0xBE8E,0x0127,0xF817,0x0227,0xFA17,0x095C,0x0B17,
		0x1026,0xBA5C,0x0317,0x1026,0xB217,0x065F,0x2888,0x9060,
		0x27F2,0x1710,0x26A2,0x26A3,0x5F4D,0x2808,0x1927,0xFA84,
		0x69A0,0x785D,0x2888,0x8710,0x8C82,0x8926,0xB217,0x036B,
		0x9C60,0x9417,0x2926,0x8345,0xA817,0x0727,0xFB17,0x2945,
		0x881F,0x1708,0x27FA,0x5D87,0x108C,0x8289,0x170E,0x4826,
		0x9A28,0x884C,0x0B79,0x1730,0x2692,0x1709,0x9160,0x27F2,
		0x1710,0x2682,0x2683,0x5F4D,0x2808,0x1927,0xFA84,0x69A1,
		0x785D,0x2888,0x8710,0x8C80,0x8A26,0x9217,0x036B,0x9D95,
		0x2603,0x5C01,0x4558,0x8E00,0x2798,0x170A,0x4A65,0x4316,
		0x6643,0x165B,0x4316,0x5943,0x168E,0x0327,0x9C45,0x7817,
		0x0727,0x9D17,0x225D,0x8710,0x2808,0x530D,0x8C80,0x8A45,
		0x5823,0x1708,0x8E01,0x2798,0x8E00,0x2644,0x5C05,0x1244,
		0x4B71,0x759E,0x8B85,0x0143,0x7371,0xA346,0x4316,0x6FA3,
		0x9612,0x4416,0x4643,0x1697,0x2604,0x2684,0x8E03,0x27FC,
		0x5C0D,0x5754,0x1709,0x5556,0x495F,0x5305,0x5307,0x286C,
		0x4C09,0x282C,0x72AE,0x7CAA,0x28A8,0x7960,0x269C,0x5C1B,
		0x4548,0x4508,0x4588,0x26BE,0x8E01,0x27F8,0x1702,0x27FA,
		0x1709,0x5C0B,0x1710,0x26BA,0x5C03,0x1710,0x26B2,0x1706,
		0x5F28,0x8898,0x6027,0xF217,0x1026,0xA226,0xA35F,0x4D28,
		0x081A,0x27FA,0x8469,0xA578,0x5D28,0x8887,0x108C,0x8289,
		0x26B2,0x1703,0x6BA4,0x6099,0x1729,0x2683,0x45A8,0x1707,
		0x27FB,0x1729,0x4588,0x2017,0x0827,0xFA5D,0x8710,0x8C82,
		0x8917,0x0E48,0x269A,0x2888,0x4C0B,0x7917,0x3026,0x9217,
		0x099A,0x6027,0xF217,0x1026,0x8226,0x835F,0x4D28,0x081A,
		0x27FA,0x8469,0xAB78,0x5D28,0x8887,0x108C,0x808A,0x2692,
		0x1703,0x6BA6,0xA726,0x035C,0x0145,0x588E,0x0027,0x9817,
		0x0A4A,0x0A43,0x160B,0x438E,0x0327,0x9C45,0x7817,0x0727,
		0x9D17,0x225D,0x8710,0x2808,0x530D,0x8C80,0x8A45,0x5817,
		0x088E,0x0127,0x988E,0x0076,0xAC77,0xAC46,0x4416,0x16A8,
		0x7A26,0x445C,0x0512,0x444B,0x7175,0xA24A,0x0343,0x1604,
		0x4316,0x5843,0x165A,0x4316,0x0643,0x1607,0x4316,0x8E03,
		0x279C,0x4578,0x7B17,0x078B,0x8627,0x9D17,0x2345,0x5822,
		0x1708,0x8E01,0x2798,0x8E00,0x2644,0x5C05,0x1244,0x4B8D,
		0x602C,0x2C2C
	};

	// Inhibit load of the default sensor sequencer
	//ap0101_write_reg(index, 0xCC02, 2, 0x003F);

	ap0101_WriteSensorReg(index, 0x3088, 0x8000);

	ap0101_WriteSensorSequencer(index, 0x3086, sequencer, sizeof(sequencer) / sizeof(sequencer[0]));

	ap0101_WriteSensorReg(index, 0x309E, 0x0186);
}

static int ap0101_InitSensor(int index)
{
	reg_item_u16_t sensor_settings[] = {
		//Setting mini array load to zero
		{0x3EF2, 0x000F},
		//ADACD tune
		{0x3200, 0x0002},
		{0x3202, 0x0160},
		{0x3206, 0x0604},
		{0x3208, 0x0F0A},
		{0x320A, 0x0080},
		//vrstlo lcg to 12
		{0x3EDA, 0x0C88},
		//blue_halo disable
		{0x318C, 0xC000},
		//DC setting
		{0x31E0, 0x0301},
	};
	unsigned short reg = 0;
	int result;

	ap0101_ReadSensorReg(index, 0x3000, &reg);
	if (reg != 0x0051) {
		dev_err(&max9286_data[0].i2c_client->dev, 
			"%s: AR0140 hasn't been found, reg = 0x%x, index = %d.\n", __func__, reg, index);
		return RESULT_EUNKNOW;
	}
	dev_info(&max9286_data[0].i2c_client->dev, "%s: AR0140 was found, index = %d.\n", __func__, index);

	result = ap0101_SensorInitialization(index);
	if (result != RESULT_ENOERR)
		return result;

	result = ap0101_WriteSensorRegList(index, sensor_settings, sizeof(sensor_settings) / sizeof(sensor_settings[0]));

	return result;
}

reg_item_u16_t ap0101_contents0111[] = {
	{0xA68C, 0xC0F1},
	{0xA68E, 0xC5E1},
	{0xA690, 0x7528},
	{0xA692, 0x73CF},
	{0xA694, 0xFFFF},
	{0xA696, 0xAE1C},
	{0xA698, 0x9B21},
	{0xA69A, 0x2148},
	{0xA69C, 0x0002},
	{0xA69E, 0x71CF},
	{0xA6A0, 0x0000},
	{0xA6A2, 0x7FFF},
	{0xA6A4, 0x2209},
	{0xA6A6, 0x0041},
	{0xA6A8, 0x934A},
	{0xA6AA, 0x792E},
	{0xA6AC, 0x7A0C},
	{0xA6AE, 0x9309},
	{0xA6B0, 0x2942},
	{0xA6B2, 0x7142},
	{0xA6B4, 0x621A},
	{0xA6B6, 0x8500},
	{0xA6B8, 0x2805},
	{0xA6BA, 0x003E},
	{0xA6BC, 0x702F},
	{0xA6BE, 0x2905},
	{0xA6C0, 0x00BE},
	{0xA6C2, 0x2015},
	{0xA6C4, 0x0E40},
	{0xA6C6, 0xD90A},
	{0xA6C8, 0x0CB2},
	{0xA6CA, 0x0720},
	{0xA6CC, 0xDA05},
	{0xA6CE, 0x0599},
	{0xA6D0, 0x0EA0},
	{0xA6D2, 0xA500},
	{0xA6D4, 0xC0F1},
	{0xA6D6, 0x0D72},
	{0xA6D8, 0x0E80},
	{0xA6DA, 0xC1A5},
	{0xA6DC, 0xC041},
	{0xA6DE, 0x7528},
	{0xA6E0, 0x71CF},
	{0xA6E2, 0xFFFF},
	{0xA6E4, 0x9784},
	{0xA6E6, 0x1129},
	{0xA6E8, 0x8700},
	{0xA6EA, 0x230A},
	{0xA6EC, 0x1F80},
	{0xA6EE, 0xFFFF},
	{0xA6F0, 0x95A4},
	{0xA6F2, 0xA500},
	{0xA6F4, 0x112A},
	{0xA6F6, 0x8700},
	{0xA6F8, 0x7748},
	{0xA6FA, 0xA200},
	{0xA6FC, 0xD800},
	{0xA6FE, 0xF004},
	{0xA700, 0xE001},
	{0xA702, 0x7810},
	{0xA704, 0x08AD},
	{0xA706, 0x0113},
	{0xA708, 0x2316},
	{0xA70A, 0x1002},
	{0xA70C, 0x1404},
	{0xA70E, 0x300A},
	{0xA710, 0x92CC},
	{0xA712, 0x0EF1},
	{0xA714, 0x9282},
	{0xA716, 0xE89A},
	{0xA718, 0x0B0F},
	{0xA71A, 0x0051},
	{0xA71C, 0x920D},
	{0xA71E, 0xA500},
	{0xA720, 0x920E},
	{0xA722, 0xA700},
	{0xA724, 0xF046},
	{0xA726, 0x7915},
	{0xA728, 0x0B13},
	{0xA72A, 0x0091},
	{0xA72C, 0x1166},
	{0xA72E, 0x8700},
	{0xA730, 0xA500},
	{0xA732, 0x1167},
	{0xA734, 0x8700},
	{0xA736, 0xA700},
	{0xA738, 0xF03C},
	{0xA73A, 0x116E},
	{0xA73C, 0x8700},
	{0xA73E, 0xA500},
	{0xA740, 0x116F},
	{0xA742, 0x8700},
	{0xA744, 0xA700},
	{0xA746, 0xF036},
	{0xA748, 0x1210},
	{0xA74A, 0x010A},
	{0xA74C, 0x1C10},
	{0xA74E, 0x3280},
	{0xA750, 0x0B11},
	{0xA752, 0x0051},
	{0xA754, 0x9209},
	{0xA756, 0x926A},
	{0xA758, 0x922D},
	{0xA75A, 0xC342},
	{0xA75C, 0x924E},
	{0xA75E, 0xF019},
	{0xA760, 0x2115},
	{0xA762, 0x0002},
	{0xA764, 0x0B19},
	{0xA766, 0x0091},
	{0xA768, 0x1264},
	{0xA76A, 0x8700},
	{0xA76C, 0x1265},
	{0xA76E, 0x8703},
	{0xA770, 0x1266},
	{0xA772, 0x8701},
	{0xA774, 0xC342},
	{0xA776, 0x1267},
	{0xA778, 0x8702},
	{0xA77A, 0xF00B},
	{0xA77C, 0x126C},
	{0xA77E, 0x8700},
	{0xA780, 0x126D},
	{0xA782, 0x8703},
	{0xA784, 0x126E},
	{0xA786, 0x8701},
	{0xA788, 0x126F},
	{0xA78A, 0x8702},
	{0xA78C, 0xC342},
	{0xA78E, 0xC243},
	{0xA790, 0xC201},
	{0xA792, 0xC240},
	{0xA794, 0xC204},
	{0xA796, 0x0A16},
	{0xA798, 0x0720},
	{0xA79A, 0x73C9},
	{0xA79C, 0xA500},
	{0xA79E, 0xC001},
	{0xA7A0, 0xC103},
	{0xA7A2, 0xC040},
	{0xA7A4, 0xC002},
	{0xA7A6, 0xC204},
	{0xA7A8, 0x0A02},
	{0xA7AA, 0x0720},
	{0xA7AC, 0x73C9},
	{0xA7AE, 0xA700},
	{0xA7B0, 0xC001},
	{0xA7B2, 0x71A9},
	{0xA7B4, 0xFFB6},
	{0xA7B6, 0x04A1},
	{0xA7B8, 0x0EA0},
	{0xA7BA, 0xC0A5},
	{0xA7BC, 0xC0F1},
	{0xA7BE, 0x0C8A},
	{0xA7C0, 0x0E80},
	{0xA7C2, 0x2482},
	{0xA7C4, 0x3202},
	{0xA7C6, 0xC040},
	{0xA7C8, 0xC141},
	{0xA7CA, 0xC242},
	{0xA7CC, 0x8120},
	{0xA7CE, 0xC343},
	{0xA7D0, 0x75CF},
	{0xA7D2, 0xFFFF},
	{0xA7D4, 0x9388},
	{0xA7D6, 0x2153},
	{0xA7D8, 0x0102},
	{0xA7DA, 0x2140},
	{0xA7DC, 0x0800},
	{0xA7DE, 0xE280},
	{0xA7E0, 0x20CA},
	{0xA7E2, 0x0041},
	{0xA7E4, 0xB825},
	{0xA7E6, 0x7810},
	{0xA7E8, 0xC049},
	{0xA7EA, 0x1544},
	{0xA7EC, 0x1100},
	{0xA7EE, 0xB8C2},
	{0xA7F0, 0xC04C},
	{0xA7F2, 0x70CF},
	{0xA7F4, 0xFFFF},
	{0xA7F6, 0x97EC},
	{0xA7F8, 0x90CD},
	{0xA7FA, 0x900E},
	{0xA7FC, 0xBE25},
	{0xA7FE, 0x78CC},
	{0xA800, 0x2942},
	{0xA802, 0x7100},
	{0xA804, 0xB83B},
	{0xA806, 0x7027},
	{0xA808, 0xB845},
	{0xA80A, 0x7810},
	{0xA80C, 0xC04D},
	{0xA80E, 0x60D8},
	{0xA810, 0xC04E},
	{0xA812, 0xB808},
	{0xA814, 0x2056},
	{0xA816, 0x080F},
	{0xA818, 0x6E01},
	{0xA81A, 0xC04F},
	{0xA81C, 0x78CC},
	{0xA81E, 0x70E9},
	{0xA820, 0x0C6A},
	{0xA822, 0x0EA0},
	{0xA824, 0x712F},
	{0xA826, 0x7810},
	{0xA828, 0xC050},
	{0xA82A, 0x70E9},
	{0xA82C, 0x0C5E},
	{0xA82E, 0x0EA0},
	{0xA830, 0xC10D},
	{0xA832, 0x7810},
	{0xA834, 0xC051},
	{0xA836, 0x2E40},
	{0xA838, 0x1200},
	{0xA83A, 0x2056},
	{0xA83C, 0x0800},
	{0xA83E, 0x7810},
	{0xA840, 0xC052},
	{0xA842, 0x70CF},
	{0xA844, 0xFFFF},
	{0xA846, 0xAE1C},
	{0xA848, 0x1086},
	{0xA84A, 0x0100},
	{0xA84C, 0xC05F},
	{0xA84E, 0x70CF},
	{0xA850, 0xFFFF},
	{0xA852, 0xAE1C},
	{0xA854, 0x1088},
	{0xA856, 0x0100},
	{0xA858, 0x1C80},
	{0xA85A, 0x3000},
	{0xA85C, 0x70CF},
	{0xA85E, 0xFFFF},
	{0xA860, 0xAE1C},
	{0xA862, 0x108A},
	{0xA864, 0x0100},
	{0xA866, 0x1C84},
	{0xA868, 0x3000},
	{0xA86A, 0x70CF},
	{0xA86C, 0xFFFF},
	{0xA86E, 0x994C},
	{0xA870, 0x9001},
	{0xA872, 0x081F},
	{0xA874, 0x019E},
	{0xA876, 0x15BA},
	{0xA878, 0x1600},
	{0xA87A, 0x15BB},
	{0xA87C, 0x160F},
	{0xA87E, 0xC053},
	{0xA880, 0x15BC},
	{0xA882, 0x1600},
	{0xA884, 0xC047},
	{0xA886, 0x15BD},
	{0xA888, 0x1600},
	{0xA88A, 0xC048},
	{0xA88C, 0xF063},
	{0xA88E, 0x70CF},
	{0xA890, 0xFFFF},
	{0xA892, 0x88A4},
	{0xA894, 0x8800},
	{0xA896, 0xC054},
	{0xA898, 0xC00C},
	{0xA89A, 0x0875},
	{0xA89C, 0x0091},
	{0xA89E, 0x1558},
	{0xA8A0, 0x1100},
	{0xA8A2, 0x1556},
	{0xA8A4, 0x110F},
	{0xA8A6, 0xC055},
	{0xA8A8, 0xC00F},
	{0xA8AA, 0x7F0C},
	{0xA8AC, 0x702F},
	{0xA8AE, 0x0BDE},
	{0xA8B0, 0x0EA0},
	{0xA8B2, 0x71C9},
	{0xA8B4, 0xC053},
	{0xA8B6, 0xC014},
	{0xA8B8, 0x0843},
	{0xA8BA, 0x00D1},
	{0xA8BC, 0xC50E},
	{0xA8BE, 0xE501},
	{0xA8C0, 0x7DEC},
	{0xA8C2, 0x702F},
	{0xA8C4, 0x0BC6},
	{0xA8C6, 0x0EA0},
	{0xA8C8, 0xC10D},
	{0xA8CA, 0x7708},
	{0xA8CC, 0xC00E},
	{0xA8CE, 0xC10F},
	{0xA8D0, 0x7D0C},
	{0xA8D2, 0xC015},
	{0xA8D4, 0xC20D},
	{0xA8D6, 0x2804},
	{0xA8D8, 0x0E7E},
	{0xA8DA, 0x702F},
	{0xA8DC, 0x794C},
	{0xA8DE, 0x0BAE},
	{0xA8E0, 0x0EA0},
	{0xA8E2, 0x712F},
	{0xA8E4, 0xC047},
	{0xA8E6, 0xC015},
	{0xA8E8, 0x7DCC},
	{0xA8EA, 0x2904},
	{0xA8EC, 0x703E},
	{0xA8EE, 0x702F},
	{0xA8F0, 0x0B9A},
	{0xA8F2, 0x0EA0},
	{0xA8F4, 0xC10D},
	{0xA8F6, 0xC048},
	{0xA8F8, 0xF02D},
	{0xA8FA, 0xC10F},
	{0xA8FC, 0xC015},
	{0xA8FE, 0x782C},
	{0xA900, 0x1C20},
	{0xA902, 0x3E40},
	{0xA904, 0x1C1C},
	{0xA906, 0x3E40},
	{0xA908, 0xC713},
	{0xA90A, 0xF025},
	{0xA90C, 0xC014},
	{0xA90E, 0x0813},
	{0xA910, 0x0093},
	{0xA912, 0x70CF},
	{0xA914, 0x000F},
	{0xA916, 0xFFFF},
	{0xA918, 0xC053},
	{0xA91A, 0x7708},
	{0xA91C, 0xF009},
	{0xA91E, 0x70CF},
	{0xA920, 0xFFFF},
	{0xA922, 0x88A4},
	{0xA924, 0x9021},
	{0xA926, 0x90E2},
	{0xA928, 0x2702},
	{0xA92A, 0x1040},
	{0xA92C, 0xC053},
	{0xA92E, 0xC014},
	{0xA930, 0x0811},
	{0xA932, 0x00D3},
	{0xA934, 0x70CF},
	{0xA936, 0x000F},
	{0xA938, 0xFFFF},
	{0xA93A, 0xC048},
	{0xA93C, 0xC047},
	{0xA93E, 0xF00B},
	{0xA940, 0xC013},
	{0xA942, 0x2E05},
	{0xA944, 0x103E},
	{0xA946, 0x1C1C},
	{0xA948, 0x3E40},
	{0xA94A, 0x2E05},
	{0xA94C, 0x13FE},
	{0xA94E, 0x1C20},
	{0xA950, 0x3E40},
	{0xA952, 0x70C9},
	{0xA954, 0xD900},
	{0xA956, 0x0A26},
	{0xA958, 0x0720},
	{0xA95A, 0xDA08},
	{0xA95C, 0xC102},
	{0xA95E, 0x8121},
	{0xA960, 0x2905},
	{0xA962, 0x003E},
	{0xA964, 0x2941},
	{0xA966, 0x7200},
	{0xA968, 0xC044},
	{0xA96A, 0xC00D},
	{0xA96C, 0xD900},
	{0xA96E, 0x0A0E},
	{0xA970, 0x0720},
	{0xA972, 0xDA08},
	{0xA974, 0xC102},
	{0xA976, 0x72CF},
	{0xA978, 0xFFFF},
	{0xA97A, 0x994C},
	{0xA97C, 0x8122},
	{0xA97E, 0x9241},
	{0xA980, 0x2905},
	{0xA982, 0x003E},
	{0xA984, 0x2941},
	{0xA986, 0x7200},
	{0xA988, 0xC101},
	{0xA98A, 0xC045},
	{0xA98C, 0x8101},
	{0xA98E, 0x8122},
	{0xA990, 0x0A2B},
	{0xA992, 0x021E},
	{0xA994, 0x149B},
	{0xA996, 0x3082},
	{0xA998, 0xBA02},
	{0xA99A, 0x7283},
	{0xA99C, 0x827F},
	{0xA99E, 0xC209},
	{0xA9A0, 0x2B05},
	{0xA9A2, 0x00BE},
	{0xA9A4, 0x2941},
	{0xA9A6, 0x71C2},
	{0xA9A8, 0x2805},
	{0xA9AA, 0x00FE},
	{0xA9AC, 0x2941},
	{0xA9AE, 0x71C0},
	{0xA9B0, 0x2B05},
	{0xA9B2, 0x007E},
	{0xA9B4, 0x2941},
	{0xA9B6, 0x71C1},
	{0xA9B8, 0xC249},
	{0xA9BA, 0xC209},
	{0xA9BC, 0x2805},
	{0xA9BE, 0x03BE},
	{0xA9C0, 0x2153},
	{0xA9C2, 0x7100},
	{0xA9C4, 0xE080},
	{0xA9C6, 0x7A50},
	{0xA9C8, 0xC24A},
	{0xA9CA, 0x2140},
	{0xA9CC, 0x7802},
	{0xA9CE, 0x22CA},
	{0xA9D0, 0x0E41},
	{0xA9D2, 0xC00D},
	{0xA9D4, 0xBA25},
	{0xA9D6, 0x2905},
	{0xA9D8, 0x003E},
	{0xA9DA, 0x2153},
	{0xA9DC, 0x7100},
	{0xA9DE, 0xE080},
	{0xA9E0, 0x2140},
	{0xA9E2, 0x7801},
	{0xA9E4, 0x21CA},
	{0xA9E6, 0x0E41},
	{0xA9E8, 0x2941},
	{0xA9EA, 0x0140},
	{0xA9EC, 0x7810},
	{0xA9EE, 0xC04B},
	{0xA9F0, 0xC00C},
	{0xA9F2, 0x7D50},
	{0xA9F4, 0x08B1},
	{0xA9F6, 0x0091},
	{0xA9F8, 0x70CF},
	{0xA9FA, 0xFFFF},
	{0xA9FC, 0x88A4},
	{0xA9FE, 0x8800},
	{0xAA00, 0x0841},
	{0xAA02, 0x00D3},
	{0xAA04, 0xC012},
	{0xAA06, 0x0A86},
	{0xAA08, 0x0EA0},
	{0xAA0A, 0x71C9},
	{0xAA0C, 0x7608},
	{0xAA0E, 0xD908},
	{0xAA10, 0x096A},
	{0xAA12, 0x0720},
	{0xAA14, 0x7228},
	{0xAA16, 0xC104},
	{0xAA18, 0x2805},
	{0xAA1A, 0x007E},
	{0xAA1C, 0x2941},
	{0xAA1E, 0x7200},
	{0xAA20, 0xC044},
	{0xAA22, 0xC001},
	{0xAA24, 0x7EAC},
	{0xAA26, 0x8000},
	{0xAA28, 0x2942},
	{0xAA2A, 0x71CD},
	{0xAA2C, 0xBD38},
	{0xAA2E, 0x7527},
	{0xAA30, 0x7816},
	{0xAA32, 0x7815},
	{0xAA34, 0xB82A},
	{0xAA36, 0x7810},
	{0xAA38, 0xC04A},
	{0xAA3A, 0xBD48},
	{0xAA3C, 0x7DB0},
	{0xAA3E, 0xF034},
	{0xAA40, 0xC010},
	{0xAA42, 0xD908},
	{0xAA44, 0x0936},
	{0xAA46, 0x0720},
	{0xAA48, 0x7228},
	{0xAA4A, 0xC104},
	{0xAA4C, 0x7DB6},
	{0xAA4E, 0x2805},
	{0xAA50, 0x007E},
	{0xAA52, 0x2941},
	{0xAA54, 0x7200},
	{0xAA56, 0xC044},
	{0xAA58, 0xC011},
	{0xAA5A, 0xD908},
	{0xAA5C, 0x091E},
	{0xAA5E, 0x0720},
	{0xAA60, 0x7228},
	{0xAA62, 0xC105},
	{0xAA64, 0x7DB5},
	{0xAA66, 0x2805},
	{0xAA68, 0x007E},
	{0xAA6A, 0x2941},
	{0xAA6C, 0x7200},
	{0xAA6E, 0xC045},
	{0xAA70, 0xC001},
	{0xAA72, 0xC10B},
	{0xAA74, 0x8000},
	{0xAA76, 0x2845},
	{0xAA78, 0x0DFE},
	{0xAA7A, 0x2941},
	{0xAA7C, 0x7280},
	{0xAA7E, 0x7810},
	{0xAA80, 0xC04A},
	{0xAA82, 0xC010},
	{0xAA84, 0x7D0C},
	{0xAA86, 0x2942},
	{0xAA88, 0x730D},
	{0xAA8A, 0xBD33},
	{0xAA8C, 0x7527},
	{0xAA8E, 0xBD4D},
	{0xAA90, 0xC011},
	{0xAA92, 0x7DB0},
	{0xAA94, 0x782C},
	{0xAA96, 0x2942},
	{0xAA98, 0x71C0},
	{0xAA9A, 0xB838},
	{0xAA9C, 0x7027},
	{0xAA9E, 0xB848},
	{0xAAA0, 0x7810},
	{0xAAA2, 0xC04B},
	{0xAAA4, 0x70CF},
	{0xAAA6, 0xFFFF},
	{0xAAA8, 0xAE1C},
	{0xAAAA, 0xC10A},
	{0xAAAC, 0x900B},
	{0xAAAE, 0x0809},
	{0xAAB0, 0x0045},
	{0xAAB2, 0xC04A},
	{0xAAB4, 0x70CF},
	{0xAAB6, 0xFFFF},
	{0xAAB8, 0xAE1C},
	{0xAABA, 0x900C},
	{0xAABC, 0x7510},
	{0xAABE, 0x25CA},
	{0xAAC0, 0x100D},
	{0xAAC2, 0x70CF},
	{0xAAC4, 0xFFFF},
	{0xAAC6, 0xAE1C},
	{0xAAC8, 0xC10B},
	{0xAACA, 0x900D},
	{0xAACC, 0x0807},
	{0xAACE, 0x0045},
	{0xAAD0, 0xC04B},
	{0xAAD2, 0xC002},
	{0xAAD4, 0x8020},
	{0xAAD6, 0xC003},
	{0xAAD8, 0x2905},
	{0xAADA, 0x003E},
	{0xAADC, 0x2941},
	{0xAADE, 0x7140},
	{0xAAE0, 0xC056},
	{0xAAE2, 0x7810},
	{0xAAE4, 0xC057},
	{0xAAE6, 0xC103},
	{0xAAE8, 0xC004},
	{0xAAEA, 0x2805},
	{0xAAEC, 0x007E},
	{0xAAEE, 0x2941},
	{0xAAF0, 0x7140},
	{0xAAF2, 0xC058},
	{0xAAF4, 0x7810},
	{0xAAF6, 0xC059},
	{0xAAF8, 0xC005},
	{0xAAFA, 0x2805},
	{0xAAFC, 0x007E},
	{0xAAFE, 0x2941},
	{0xAB00, 0x7140},
	{0xAB02, 0xC05A},
	{0xAB04, 0x7810},
	{0xAB06, 0xC05B},
	{0xAB08, 0xC013},
	{0xAB0A, 0xD900},
	{0xAB0C, 0x086E},
	{0xAB0E, 0x0720},
	{0xAB10, 0x7228},
	{0xAB12, 0xC117},
	{0xAB14, 0x2805},
	{0xAB16, 0x007E},
	{0xAB18, 0xC10A},
	{0xAB1A, 0x2941},
	{0xAB1C, 0x7200},
	{0xAB1E, 0x7030},
	{0xAB20, 0x21CA},
	{0xAB22, 0x000D},
	{0xAB24, 0xC007},
	{0xAB26, 0xC15C},
	{0xAB28, 0xD900},
	{0xAB2A, 0x0852},
	{0xAB2C, 0x0720},
	{0xAB2E, 0xDA08},
	{0xAB30, 0xC119},
	{0xAB32, 0x2805},
	{0xAB34, 0x007E},
	{0xAB36, 0x2941},
	{0xAB38, 0x7400},
	{0xAB3A, 0x60B8},
	{0xAB3C, 0xC05D},
	{0xAB3E, 0x70CF},
	{0xAB40, 0xFFFF},
	{0xAB42, 0x88A4},
	{0xAB44, 0x8800},
	{0xAB46, 0xE081},
	{0xAB48, 0x0048},
	{0xAB4A, 0x002C},
	{0xAB4C, 0xD800},
	{0xAB4E, 0x70E9},
	{0xAB50, 0xD900},
	{0xAB52, 0x082A},
	{0xAB54, 0x0720},
	{0xAB56, 0xDA08},
	{0xAB58, 0xC119},
	{0xAB5A, 0x2805},
	{0xAB5C, 0x007E},
	{0xAB5E, 0xC013},
	{0xAB60, 0x2941},
	{0xAB62, 0x740E},
	{0xAB64, 0xD900},
	{0xAB66, 0x0816},
	{0xAB68, 0x0720},
	{0xAB6A, 0xDA08},
	{0xAB6C, 0xC117},
	{0xAB6E, 0x66BE},
	{0xAB70, 0x2805},
	{0xAB72, 0x007E},
	{0xAB74, 0xC109},
	{0xAB76, 0x2941},
	{0xAB78, 0x7400},
	{0xAB7A, 0xC213},
	{0xAB7C, 0x7030},
	{0xAB7E, 0x21CA},
	{0xAB80, 0x000D},
	{0xAB82, 0x2602},
	{0xAB84, 0x1040},
	{0xAB86, 0xB808},
	{0xAB88, 0x08F2},
	{0xAB8A, 0x0EA0},
	{0xAB8C, 0x2702},
	{0xAB8E, 0x1081},
	{0xAB90, 0xC05E},
	{0xAB92, 0x70CF},
	{0xAB94, 0xFFFF},
	{0xAB96, 0x88A4},
	{0xAB98, 0x8800},
	{0xAB9A, 0xE082},
	{0xAB9C, 0x0044},
	{0xAB9E, 0x002C},
	{0xABA0, 0xD800},
	{0xABA2, 0xC008},
	{0xABA4, 0xD900},
	{0xABA6, 0x0FD6},
	{0xABA8, 0x06E0},
	{0xABAA, 0xDA08},
	{0xABAC, 0xC11B},
	{0xABAE, 0xC60B},
	{0xABB0, 0x2805},
	{0xABB2, 0x007E},
	{0xABB4, 0x2941},
	{0xABB6, 0x7400},
	{0xABB8, 0x661E},
	{0xABBA, 0xC007},
	{0xABBC, 0xD900},
	{0xABBE, 0x0FBE},
	{0xABC0, 0x06E0},
	{0xABC2, 0xDA08},
	{0xABC4, 0xC119},
	{0xABC6, 0x7EA2},
	{0xABC8, 0xC207},
	{0xABCA, 0x2805},
	{0xABCC, 0x007E},
	{0xABCE, 0xC108},
	{0xABD0, 0x2941},
	{0xABD2, 0x7400},
	{0xABD4, 0x7E02},
	{0xABD6, 0x2E40},
	{0xABD8, 0x1200},
	{0xABDA, 0x08A2},
	{0xABDC, 0x0EA0},
	{0xABDE, 0x7942},
	{0xABE0, 0x149B},
	{0xABE2, 0x3082},
	{0xABE4, 0xC300},
	{0xABE6, 0x6A34},
	{0xABE8, 0x6179},
	{0xABEA, 0xC313},
	{0xABEC, 0xBA02},
	{0xABEE, 0xA160},
	{0xABF0, 0xC307},
	{0xABF2, 0xA1E1},
	{0xABF4, 0xA162},
	{0xABF6, 0xC308},
	{0xABF8, 0x7A55},
	{0xABFA, 0xA163},
	{0xABFC, 0xC100},
	{0xABFE, 0x6159},
	{0xAC00, 0xC20A},
	{0xAC02, 0xB158},
	{0xAC04, 0xC216},
	{0xAC06, 0xB159},
	{0xAC08, 0xC218},
	{0xAC0A, 0xB1BA},
	{0xAC0C, 0xB15B},
	{0xAC0E, 0xC20B},
	{0xAC10, 0xB15C},
	{0xAC12, 0xC21A},
	{0xAC14, 0xB15D},
	{0xAC16, 0xC21C},
	{0xAC18, 0xB15E},
	{0xAC1A, 0xC21E},
	{0xAC1C, 0xB15F},
	{0xAC1E, 0xC21D},
	{0xAC20, 0x1940},
	{0xAC22, 0x0084},
	{0xAC24, 0x1942},
	{0xAC26, 0x0004},
	{0xAC28, 0x002D},
	{0xAC2A, 0x0EA0},
	{0xAC2C, 0x2480},
	{0xAC2E, 0x3202},
	{0xAC30, 0xC0F1},
	{0xAC32, 0xC5E1},
	{0xAC34, 0x0C36},
	{0xAC36, 0x09E0},
	{0xAC38, 0x7508},
	{0xAC3A, 0x9522},
	{0xAC3C, 0x72CF},
	{0xAC3E, 0x0000},
	{0xAC40, 0xFFFF},
	{0xAC42, 0x73CF},
	{0xAC44, 0xFFFF},
	{0xAC46, 0xAE94},
	{0xAC48, 0x2905},
	{0xAC4A, 0x003E},
	{0xAC4C, 0x2941},
	{0xAC4E, 0x71C1},
	{0xAC50, 0x7150},
	{0xAC52, 0x21CA},
	{0xAC54, 0x0086},
	{0xAC56, 0xB324},
	{0xAC58, 0x9523},
	{0xAC5A, 0x2905},
	{0xAC5C, 0x003E},
	{0xAC5E, 0x2941},
	{0xAC60, 0x71C1},
	{0xAC62, 0x7150},
	{0xAC64, 0x21CA},
	{0xAC66, 0x0086},
	{0xAC68, 0xB325},
	{0xAC6A, 0x9525},
	{0xAC6C, 0x2905},
	{0xAC6E, 0x003E},
	{0xAC70, 0x2941},
	{0xAC72, 0x71C0},
	{0xAC74, 0x7050},
	{0xAC76, 0x22CA},
	{0xAC78, 0x0005},
	{0xAC7A, 0x9509},
	{0xAC7C, 0xB346},
	{0xAC7E, 0xB307},
	{0xAC80, 0x950A},
	{0xAC82, 0xB308},
	{0xAC84, 0x950C},
	{0xAC86, 0x07E1},
	{0xAC88, 0x0E60},
	{0xAC8A, 0xB309},
	{0xAC8C, 0xC0F1},
	{0xAC8E, 0x70CF},
	{0xAC90, 0xFFFF},
	{0xAC92, 0x994C},
	{0xAC94, 0x9001},
	{0xAC96, 0x080F},
	{0xAC98, 0x021E},
	{0xAC9A, 0x70CF},
	{0xAC9C, 0xFFFF},
	{0xAC9E, 0x9E84},
	{0xACA0, 0xFFE4},
	{0xACA2, 0x70CF},
	{0xACA4, 0x0000},
	{0xACA6, 0xE9CA},
	{0xACA8, 0x7840},
	{0xACAA, 0xC0D1},
	{0xACAC, 0x7EE0},
	{0xACAE, 0x78E0},
	{0xACB0, 0xC0F1},
	{0xACB2, 0x0F96},
	{0xACB4, 0x0E40},
	{0xACB6, 0xC1AF},
	{0xACB8, 0xC041},
	{0xACBA, 0x70CF},
	{0xACBC, 0xFFFF},
	{0xACBE, 0x994C},
	{0xACC0, 0x9021},
	{0xACC2, 0x091D},
	{0xACC4, 0x021E},
	{0xACC6, 0x72CF},
	{0xACC8, 0xFFFF},
	{0xACCA, 0xAE94},
	{0xACCC, 0x9204},
	{0xACCE, 0xC042},
	{0xACD0, 0xC046},
	{0xACD2, 0x9205},
	{0xACD4, 0xC043},
	{0xACD6, 0xC047},
	{0xACD8, 0x92C6},
	{0xACDA, 0xF00C},
	{0xACDC, 0x72CF},
	{0xACDE, 0xFFFF},
	{0xACE0, 0x952C},
	{0xACE2, 0x9215},
	{0xACE4, 0x92D7},
	{0xACE6, 0xC042},
	{0xACE8, 0xC046},
	{0xACEA, 0x9216},
	{0xACEC, 0xC043},
	{0xACEE, 0xC047},
	{0xACF0, 0xC648},
	{0xACF2, 0x0921},
	{0xACF4, 0x005E},
	{0xACF6, 0xC102},
	{0xACF8, 0xC003},
	{0xACFA, 0x7030},
	{0xACFC, 0x20CA},
	{0xACFE, 0x004E},
	{0xAD00, 0x70D1},
	{0xAD02, 0x26CA},
	{0xAD04, 0x100D},
	{0xAD06, 0xC646},
	{0xAD08, 0xC647},
	{0xAD0A, 0xC648},
	{0xAD0C, 0xC643},
	{0xAD0E, 0xC642},
	{0xAD10, 0xDF00},
	{0xAD12, 0xF00B},
	{0xAD14, 0x78EF},
	{0xAD16, 0xC040},
	{0xAD18, 0xC001},
	{0xAD1A, 0xC189},
	{0xAD1C, 0xC28C},
	{0xAD1E, 0x0BC2},
	{0xAD20, 0x0A20},
	{0xAD22, 0xC305},
	{0xAD24, 0xE701},
	{0xAD26, 0x0F37},
	{0xAD28, 0x10D3},
	{0xAD2A, 0x6F12},
	{0xAD2C, 0x7083},
	{0xAD2E, 0x8006},
	{0xAD30, 0xD905},
	{0xAD32, 0xC044},
	{0xAD34, 0x0E46},
	{0xAD36, 0x06E0},
	{0xAD38, 0x7228},
	{0xAD3A, 0x7810},
	{0xAD3C, 0xC045},
	{0xAD3E, 0xDD00},
	{0xAD40, 0x0DD5},
	{0xAD42, 0x90D3},
	{0xAD44, 0xC004},
	{0xAD46, 0xC189},
	{0xAD48, 0xC28C},
	{0xAD4A, 0x79B5},
	{0xAD4C, 0x7810},
	{0xAD4E, 0x7AB5},
	{0xAD50, 0xE501},
	{0xAD52, 0x0AAE},
	{0xAD54, 0x0A20},
	{0xAD56, 0x7BAF},
	{0xAD58, 0xF1F4},
	{0xAD5A, 0xC001},
	{0xAD5C, 0xC102},
	{0xAD5E, 0xC203},
	{0xAD60, 0x0806},
	{0xAD62, 0x0A60},
	{0xAD64, 0x73C9},
	{0xAD66, 0x06F1},
	{0xAD68, 0x0E60},
	{0xAD6A, 0xC0AF},
	{0xAD6C, 0xC0F1},
	{0xAD6E, 0xC5E1},
	{0xAD70, 0x75CF},
	{0xAD72, 0xFFFF},
	{0xAD74, 0x994C},
	{0xAD76, 0x9502},
	{0xAD78, 0x71CF},
	{0xAD7A, 0xFFFF},
	{0xAD7C, 0x88A4},
	{0xAD7E, 0x0819},
	{0xAD80, 0x019E},
	{0xAD82, 0x9501},
	{0xAD84, 0x0811},
	{0xAD86, 0x001E},
	{0xAD88, 0x2140},
	{0xAD8A, 0x0200},
	{0xAD8C, 0xFFC9},
	{0xAD8E, 0x9500},
	{0xAD90, 0xB880},
	{0xAD92, 0xF004},
	{0xAD94, 0x9500},
	{0xAD96, 0xB8A0},
	{0xAD98, 0x06CD},
	{0xAD9A, 0x0E60},
	{0xAD9C, 0xB500},
	{0xAD9E, 0x78E0},
	{0xADA0, 0xD900},
	{0xADA2, 0xF00A},
	{0xADA4, 0x70CF},
	{0xADA6, 0xFFFF},
	{0xADA8, 0xAEA8},
	{0xADAA, 0x7835},
	{0xADAC, 0x8041},
	{0xADAE, 0x8000},
	{0xADB0, 0xE102},
	{0xADB2, 0xA040},
	{0xADB4, 0x09F1},
	{0xADB6, 0x8214},
	{0xADB8, 0x72CF},
	{0xADBA, 0xFFFF},
	{0xADBC, 0xAE1C},
	{0xADBE, 0x7148},
	{0xADC0, 0x70CF},
	{0xADC2, 0xFFFF},
	{0xADC4, 0x9348},
	{0xADC6, 0xB031},
	{0xADC8, 0xD920},
	{0xADCA, 0xD800},
	{0xADCC, 0x1A80},
	{0xADCE, 0x0044},
	{0xADD0, 0x1A82},
	{0xADD2, 0x0044},
	{0xADD4, 0x1A84},
	{0xADD6, 0x0044},
	{0xADD8, 0x1A86},
	{0xADDA, 0x0044},
	{0xADDC, 0x1A88},
	{0xADDE, 0x0044},
	{0xADE0, 0x1A8A},
	{0xADE2, 0x0044},
	{0xADE4, 0xB201},
	{0xADE6, 0x72CF},
	{0xADE8, 0xFFFF},
	{0xADEA, 0x994C},
	{0xADEC, 0x9221},
	{0xADEE, 0x2185},
	{0xADF0, 0x0005},
	{0xADF2, 0xB221},
	{0xADF4, 0x71CF},
	{0xADF6, 0xFFFF},
	{0xADF8, 0x9670},
	{0xADFA, 0x1900},
	{0xADFC, 0x0F80},
	{0xADFE, 0x0000},
	{0xAE00, 0x0BB8},
	{0xAE02, 0x1904},
	{0xAE04, 0x0F80},
	{0xAE06, 0x0000},
	{0xAE08, 0x0DAC},
	{0xAE0A, 0x1908},
	{0xAE0C, 0x0F80},
	{0xAE0E, 0x000F},
	{0xAE10, 0xFFFF},
	{0xAE12, 0x190C},
	{0xAE14, 0x0F80},
	{0xAE16, 0x000F},
	{0xAE18, 0xFFFF},
	{0xAE1A, 0x7EE0},
	{0xAE1C, 0x01E0},
	{0xAE1E, 0x0000},
	{0xAE20, 0x0000},
	{0xAE22, 0x0000},
	{0xAE24, 0x0000},
	{0xAE26, 0x0000},
	{0xAE28, 0x0000},
	{0xAE2A, 0x0000},
	{0xAE2C, 0x104A},
	{0xAE2E, 0x000C},
	{0xAE30, 0x0033},
	{0xAE32, 0xFFFF},
	{0xAE34, 0xFFFF},
	{0xAE36, 0xFFFF},
	{0xAE38, 0x0000},
	{0xAE3A, 0x0000},
	{0xAE3C, 0x0000},
	{0xAE3E, 0x0000},
	{0xAE40, 0x0000},
	{0xAE42, 0x0000},
	{0xAE44, 0x0000},
	{0xAE46, 0x0000},
	{0xAE48, 0x0000},
	{0xAE4A, 0x0000},
	{0xAE4C, 0x0000},
	{0xAE4E, 0x0000},
	{0xAE50, 0x0000},
	{0xAE52, 0x0000},
	{0xAE54, 0x0000},
	{0xAE56, 0x0000},
	{0xAE58, 0x0000},
	{0xAE5A, 0x0000},
	{0xAE5C, 0x0000},
	{0xAE5E, 0x0000},
	{0xAE60, 0x0000},
	{0xAE62, 0x0000},
	{0xAE64, 0x0000},
	{0xAE66, 0x0000},
	{0xAE68, 0x0000},
	{0xAE6A, 0x0000},
	{0xAE6C, 0x0000},
	{0xAE6E, 0x0000},
	{0xAE70, 0x0000},
	{0xAE72, 0x0000},
	{0xAE74, 0x0000},
	{0xAE76, 0x0000},
	{0xAE78, 0x0000},
	{0xAE7A, 0x0000},
	{0xAE7C, 0x0000},
	{0xAE7E, 0x0000},
	{0xAE80, 0x0000},
	{0xAE82, 0x0000},
	{0xAE84, 0x0000},
	{0xAE86, 0x0000},
	{0xAE88, 0x0000},
	{0xAE8A, 0x0000},
	{0xAE8C, 0x0000},
	{0xAE8E, 0x0000},
	{0xAE90, 0x0000},
	{0xAE92, 0x0000},
	{0xAE94, 0x0000},
	{0xAE96, 0x0000},
	{0xAE98, 0x0000},
	{0xAE9A, 0x0000},
	{0xAE9C, 0x0000},
	{0xAE9E, 0x0000},
	{0xAEA0, 0x0000},
	{0xAEA2, 0x0000},
	{0xAEA4, 0x0000},
	{0xAEA6, 0x0000},
	{0xAEA8, 0xFFFF},
	{0xAEAA, 0xA444},
	{0xAEAC, 0xFFFF},
	{0xAEAE, 0xA6D4},
	{0xAEB0, 0xFFFF},
	{0xAEB2, 0xA448},
	{0xAEB4, 0xFFFF},
	{0xAEB6, 0xA7BC},
	{0xAEB8, 0xFFFF},
	{0xAEBA, 0xA42C},
	{0xAEBC, 0xFFFF},
	{0xAEBE, 0xAC8C},
	{0xAEC0, 0xFFFF},
	{0xAEC2, 0xA450},
	{0xAEC4, 0xFFFF},
	{0xAEC6, 0xAD6C},
};

reg_item_u16_t ap0101_contents0211[] = {
	{0xAEC8, 0xC0F1},
	{0xAECA, 0x0D7E},
	{0xAECC, 0x0E40},
	{0xAECE, 0x70CF},
	{0xAED0, 0xFFFF},
	{0xAED2, 0x994C},
	{0xAED4, 0x10C4},
	{0xAED6, 0x0101},
	{0xAED8, 0x70CF},
	{0xAEDA, 0xFFFF},
	{0xAEDC, 0x9B5C},
	{0xAEDE, 0x1052},
	{0xAEE0, 0x0100},
	{0xAEE2, 0xDE00},
	{0xAEE4, 0x782C},
	{0xAEE6, 0x702F},
	{0xAEE8, 0xD919},
	{0xAEEA, 0x0DA2},
	{0xAEEC, 0x0E60},
	{0xAEEE, 0xB90A},
	{0xAEF0, 0x7508},
	{0xAEF2, 0xB823},
	{0xAEF4, 0x2502},
	{0xAEF6, 0x900F},
	{0xAEF8, 0x25CA},
	{0xAEFA, 0x1061},
	{0xAEFC, 0x27CA},
	{0xAEFE, 0x1341},
	{0xAF00, 0x0B02},
	{0xAF02, 0x0500},
	{0xAF04, 0x0AA2},
	{0xAF06, 0x0B00},
	{0xAF08, 0x71CF},
	{0xAF0A, 0xFFFF},
	{0xAF0C, 0x994C},
	{0xAF0E, 0x9121},
	{0xAF10, 0xB9E7},
	{0xAF12, 0x20C2},
	{0xAF14, 0x0062},
	{0xAF16, 0x72CF},
	{0xAF18, 0xFFFF},
	{0xAF1A, 0xB490},
	{0xAF1C, 0xF002},
	{0xAF1E, 0xB861},
	{0xAF20, 0x081D},
	{0xAF22, 0x0054},
	{0xAF24, 0x71CF},
	{0xAF26, 0xFFFF},
	{0xAF28, 0x9C78},
	{0xAF2A, 0x21F4},
	{0xAF2C, 0x0001},
	{0xAF2E, 0x663E},
	{0xAF30, 0x8A2C},
	{0xAF32, 0x7030},
	{0xAF34, 0x25CA},
	{0xAF36, 0x13C1},
	{0xAF38, 0x0EE7},
	{0xAF3A, 0x9344},
	{0xAF3C, 0x0A9E},
	{0xAF3E, 0x0B20},
	{0xAF40, 0xAA0C},
	{0xAF42, 0x0515},
	{0xAF44, 0x0E40},
	{0xAF46, 0x78E0},
	{0xAF48, 0xC0F1},
	{0xAF4A, 0x0CFE},
	{0xAF4C, 0x0E40},
	{0xAF4E, 0x70CF},
	{0xAF50, 0xFFFF},
	{0xAF52, 0x9A00},
	{0xAF54, 0x9027},
	{0xAF56, 0x70CF},
	{0xAF58, 0xFFFF},
	{0xAF5A, 0x9B5C},
	{0xAF5C, 0x1052},
	{0xAF5E, 0x0100},
	{0xAF60, 0xDE00},
	{0xAF62, 0x782C},
	{0xAF64, 0x702F},
	{0xAF66, 0xD919},
	{0xAF68, 0x0D22},
	{0xAF6A, 0x0E60},
	{0xAF6C, 0xB90A},
	{0xAF6E, 0x7508},
	{0xAF70, 0xB823},
	{0xAF72, 0x2502},
	{0xAF74, 0x100F},
	{0xAF76, 0x70CF},
	{0xAF78, 0xFFFF},
	{0xAF7A, 0x9A00},
	{0xAF7C, 0x9006},
	{0xAF7E, 0x0A2A},
	{0xAF80, 0x0B20},
	{0xAF82, 0xB804},
	{0xAF84, 0x7108},
	{0xAF86, 0xE780},
	{0xAF88, 0x25CA},
	{0xAF8A, 0x1061},
	{0xAF8C, 0x27CA},
	{0xAF8E, 0x1341},
	{0xAF90, 0x73CF},
	{0xAF92, 0xFFFF},
	{0xAF94, 0xB490},
	{0xAF96, 0xD800},
	{0xAF98, 0xF002},
	{0xAF9A, 0xE001},
	{0xAF9C, 0x081D},
	{0xAF9E, 0x0045},
	{0xAFA0, 0x72CF},
	{0xAFA2, 0xFFFF},
	{0xAFA4, 0x9C78},
	{0xAFA6, 0x22F4},
	{0xAFA8, 0x0002},
	{0xAFAA, 0x665E},
	{0xAFAC, 0x8B4D},
	{0xAFAE, 0x7050},
	{0xAFB0, 0x25CA},
	{0xAFB2, 0x13C1},
	{0xAFB4, 0x0EE7},
	{0xAFB6, 0x9344},
	{0xAFB8, 0xAB0D},
	{0xAFBA, 0xE080},
	{0xAFBC, 0x20CA},
	{0xAFBE, 0x0021},
	{0xAFC0, 0x0A18},
	{0xAFC2, 0x0B22},
	{0xAFC4, 0x20C2},
	{0xAFC6, 0x0062},
	{0xAFC8, 0x048D},
	{0xAFCA, 0x0E40},
	{0xAFCC, 0xC0F1},
	{0xAFCE, 0x0C7A},
	{0xAFD0, 0x0E40},
	{0xAFD2, 0xC1A8},
	{0xAFD4, 0x77CF},
	{0xAFD6, 0xFFFF},
	{0xAFD8, 0x9A00},
	{0xAFDA, 0x9702},
	{0xAFDC, 0xC042},
	{0xAFDE, 0xFFDB},
	{0xAFE0, 0x7608},
	{0xAFE2, 0xFFBA},
	{0xAFE4, 0x70D1},
	{0xAFE6, 0x20CA},
	{0xAFE8, 0x038E},
	{0xAFEA, 0xC041},
	{0xAFEC, 0x71CF},
	{0xAFEE, 0xFFFF},
	{0xAFF0, 0x9B5C},
	{0xAFF2, 0xA104},
	{0xAFF4, 0x9105},
	{0xAFF6, 0x75CF},
	{0xAFF8, 0xFFFF},
	{0xAFFA, 0xB490},
	{0xAFFC, 0xB843},
	{0xAFFE, 0xC040},
	{0xB000, 0x8502},
	{0xB002, 0xA1C3},
	{0xB004, 0xE826},
	{0xB006, 0x70CF},
	{0xB008, 0xFFFF},
	{0xB00A, 0x9690},
	{0xB00C, 0x9000},
	{0xB00E, 0x0895},
	{0xB010, 0x001E},
	{0xB012, 0x70CF},
	{0xB014, 0xFFFF},
	{0xB016, 0x98B4},
	{0xB018, 0x9805},
	{0xB01A, 0x0889},
	{0xB01C, 0x0010},
	{0xB01E, 0x71CF},
	{0xB020, 0xFFFF},
	{0xB022, 0x803C},
	{0xB024, 0x8926},
	{0xB026, 0xE180},
	{0xB028, 0x21CC},
	{0xB02A, 0x8062},
	{0xB02C, 0xF43A},
	{0xB02E, 0x0831},
	{0xB030, 0x0052},
	{0xB032, 0xD908},
	{0xB034, 0x0A32},
	{0xB036, 0x06E0},
	{0xB038, 0xDA07},
	{0xB03A, 0xC101},
	{0xB03C, 0x2805},
	{0xB03E, 0x007E},
	{0xB040, 0x2941},
	{0xB042, 0x71C1},
	{0xB044, 0xC141},
	{0xB046, 0x2805},
	{0xB048, 0x03BE},
	{0xB04A, 0x2941},
	{0xB04C, 0x71CE},
	{0xB04E, 0xF019},
	{0xB050, 0xC001},
	{0xB052, 0xA5C0},
	{0xB054, 0xA501},
	{0xB056, 0xC000},
	{0xB058, 0xA502},
	{0xB05A, 0xF1D7},
	{0xB05C, 0x7811},
	{0xB05E, 0xD908},
	{0xB060, 0x0A06},
	{0xB062, 0x06E0},
	{0xB064, 0xDA07},
	{0xB066, 0xC043},
	{0xB068, 0xC001},
	{0xB06A, 0xB807},
	{0xB06C, 0x0C0E},
	{0xB06E, 0x0E60},
	{0xB070, 0xC103},
	{0xB072, 0xC041},
	{0xB074, 0x6E17},
	{0xB076, 0x0C06},
	{0xB078, 0x0E60},
	{0xB07A, 0xC103},
	{0xB07C, 0x7608},
	{0xB07E, 0x70CF},
	{0xB080, 0xFFFF},
	{0xB082, 0x98B4},
	{0xB084, 0xC100},
	{0xB086, 0x9805},
	{0xB088, 0x6119},
	{0xB08A, 0x7811},
	{0xB08C, 0x208C},
	{0xB08E, 0x8002},
	{0xB090, 0x2148},
	{0xB092, 0x0001},
	{0xB094, 0xC140},
	{0xB096, 0xF6C4},
	{0xB098, 0x9700},
	{0xB09A, 0xF003},
	{0xB09C, 0x9701},
	{0xB09E, 0xC042},
	{0xB0A0, 0xC100},
	{0xB0A2, 0xD804},
	{0xB0A4, 0xE184},
	{0xB0A6, 0x20CA},
	{0xB0A8, 0x004D},
	{0xB0AA, 0xC044},
	{0xB0AC, 0x9703},
	{0xB0AE, 0x9724},
	{0xB0B0, 0x2805},
	{0xB0B2, 0x03BE},
	{0xB0B4, 0x2941},
	{0xB0B6, 0x7100},
	{0xB0B8, 0xC045},
	{0xB0BA, 0xC001},
	{0xB0BC, 0x2905},
	{0xB0BE, 0x003E},
	{0xB0C0, 0x9726},
	{0xB0C2, 0x2941},
	{0xB0C4, 0x7100},
	{0xB0C6, 0xB904},
	{0xB0C8, 0x7110},
	{0xB0CA, 0x26CA},
	{0xB0CC, 0x100E},
	{0xB0CE, 0x0936},
	{0xB0D0, 0x0520},
	{0xB0D2, 0x26CA},
	{0xB0D4, 0x104D},
	{0xB0D6, 0x70D1},
	{0xB0D8, 0x22CA},
	{0xB0DA, 0x0386},
	{0xB0DC, 0x22CA},
	{0xB0DE, 0x0005},
	{0xB0E0, 0xC005},
	{0xB0E2, 0x9725},
	{0xB0E4, 0x7822},
	{0xB0E6, 0x2048},
	{0xB0E8, 0x0000},
	{0xB0EA, 0x2009},
	{0xB0EC, 0x0083},
	{0xB0EE, 0xC102},
	{0xB0F0, 0xD840},
	{0xB0F2, 0xE1C0},
	{0xB0F4, 0x21CA},
	{0xB0F6, 0x0006},
	{0xB0F8, 0x1500},
	{0xB0FA, 0x100A},
	{0xB0FC, 0x7822},
	{0xB0FE, 0x2805},
	{0xB100, 0x02BE},
	{0xB102, 0x762F},
	{0xB104, 0x2905},
	{0xB106, 0x00FE},
	{0xB108, 0x7627},
	{0xB10A, 0x8561},
	{0xB10C, 0xBE26},
	{0xB10E, 0x2805},
	{0xB110, 0x00FE},
	{0xB112, 0x772F},
	{0xB114, 0x2905},
	{0xB116, 0x00BE},
	{0xB118, 0x7727},
	{0xB11A, 0x8542},
	{0xB11C, 0xBF26},
	{0xB11E, 0x2805},
	{0xB120, 0x00BE},
	{0xB122, 0xC204},
	{0xB124, 0x702F},
	{0xB126, 0xA5C0},
	{0xB128, 0xA5E1},
	{0xB12A, 0x2905},
	{0xB12C, 0x00BE},
	{0xB12E, 0x7027},
	{0xB130, 0x2841},
	{0xB132, 0x0181},
	{0xB134, 0xA522},
	{0xB136, 0x75CF},
	{0xB138, 0xFFFF},
	{0xB13A, 0x9388},
	{0xB13C, 0x15C4},
	{0xB13E, 0x1701},
	{0xB140, 0x0919},
	{0xB142, 0x00DE},
	{0xB144, 0x15C0},
	{0xB146, 0x160F},
	{0xB148, 0x15BF},
	{0xB14A, 0x160E},
	{0xB14C, 0x71CF},
	{0xB14E, 0xFFFF},
	{0xB150, 0x9B5C},
	{0xB152, 0xA1E4},
	{0xB154, 0xA1C3},
	{0xB156, 0xF006},
	{0xB158, 0x1DC0},
	{0xB15A, 0x13D8},
	{0xB15C, 0x1DBF},
	{0xB15E, 0x1398},
	{0xB160, 0xB826},
	{0xB162, 0xD908},
	{0xB164, 0x0902},
	{0xB166, 0x06E0},
	{0xB168, 0x7228},
	{0xB16A, 0x7108},
	{0xB16C, 0x2702},
	{0xB16E, 0x1380},
	{0xB170, 0x0B0A},
	{0xB172, 0x0E60},
	{0xB174, 0xB80C},
	{0xB176, 0xC046},
	{0xB178, 0x15C4},
	{0xB17A, 0x1700},
	{0xB17C, 0xB8E0},
	{0xB17E, 0x09E4},
	{0xB180, 0x0942},
	{0xB182, 0x15C5},
	{0xB184, 0x1700},
	{0xB186, 0xC106},
	{0xB188, 0xB804},
	{0xB18A, 0x6119},
	{0xB18C, 0x1563},
	{0xB18E, 0x1600},
	{0xB190, 0x0AEA},
	{0xB192, 0x0E60},
	{0xB194, 0xB804},
	{0xB196, 0xC047},
	{0xB198, 0x60D8},
	{0xB19A, 0xDE01},
	{0xB19C, 0xE081},
	{0xB19E, 0x20CA},
	{0xB1A0, 0x038E},
	{0xB1A2, 0x0936},
	{0xB1A4, 0x06E0},
	{0xB1A6, 0xD90C},
	{0xB1A8, 0x7508},
	{0xB1AA, 0xC007},
	{0xB1AC, 0x671F},
	{0xB1AE, 0xE781},
	{0xB1B0, 0x20CA},
	{0xB1B2, 0x038E},
	{0xB1B4, 0x20CA},
	{0xB1B6, 0x03CD},
	{0xB1B8, 0x091E},
	{0xB1BA, 0x06E0},
	{0xB1BC, 0xD90C},
	{0xB1BE, 0x78A2},
	{0xB1C0, 0x2008},
	{0xB1C2, 0x0381},
	{0xB1C4, 0xD800},
	{0xB1C6, 0x0AB6},
	{0xB1C8, 0x0E60},
	{0xB1CA, 0xB898},
	{0xB1CC, 0x72CF},
	{0xB1CE, 0x0001},
	{0xB1D0, 0xFFFF},
	{0xB1D2, 0x7550},
	{0xB1D4, 0x21CA},
	{0xB1D6, 0x0086},
	{0xB1D8, 0x21CA},
	{0xB1DA, 0x0345},
	{0xB1DC, 0x7050},
	{0xB1DE, 0x22CA},
	{0xB1E0, 0x0005},
	{0xB1E2, 0x0DAE},
	{0xB1E4, 0x0B60},
	{0xB1E6, 0xC007},
	{0xB1E8, 0x026D},
	{0xB1EA, 0x0E60},
	{0xB1EC, 0xC0A8},
	{0xB1EE, 0x78E0},
	{0xB1F0, 0xC0F1},
	{0xB1F2, 0xC5E1},
	{0xB1F4, 0x70CF},
	{0xB1F6, 0xFFFF},
	{0xB1F8, 0x994C},
	{0xB1FA, 0x9000},
	{0xB1FC, 0x0829},
	{0xB1FE, 0x005E},
	{0xB200, 0x75CF},
	{0xB202, 0xFFFF},
	{0xB204, 0x952C},
	{0xB206, 0x8D09},
	{0xB208, 0x080D},
	{0xB20A, 0x015F},
	{0xB20C, 0xFF70},
	{0xB20E, 0x8D09},
	{0xB210, 0xB885},
	{0xB212, 0xAD09},
	{0xB214, 0x70CF},
	{0xB216, 0xFFFF},
	{0xB218, 0x94F0},
	{0xB21A, 0x9010},
	{0xB21C, 0xB8E1},
	{0xB21E, 0x0D18},
	{0xB220, 0x0942},
	{0xB222, 0xF006},
	{0xB224, 0x70CF},
	{0xB226, 0x0000},
	{0xB228, 0xDF76},
	{0xB22A, 0x7840},
	{0xB22C, 0x0239},
	{0xB22E, 0x0E40},
	{0xB230, 0xC0F1},
	{0xB232, 0x0A16},
	{0xB234, 0x0E40},
	{0xB236, 0x7708},
	{0xB238, 0x085B},
	{0xB23A, 0x00D1},
	{0xB23C, 0x75CF},
	{0xB23E, 0xFFFF},
	{0xB240, 0x9388},
	{0xB242, 0x76CF},
	{0xB244, 0xFFFF},
	{0xB246, 0xB490},
	{0xB248, 0x8E0E},
	{0xB24A, 0x158C},
	{0xB24C, 0x1081},
	{0xB24E, 0x091F},
	{0xB250, 0x0000},
	{0xB252, 0xE187},
	{0xB254, 0x21CC},
	{0xB256, 0x8222},
	{0xB258, 0xF409},
	{0xB25A, 0x0BC2},
	{0xB25C, 0x0D40},
	{0xB25E, 0x15C4},
	{0xB260, 0x1701},
	{0xB262, 0xE885},
	{0xB264, 0xB984},
	{0xB266, 0x1DC4},
	{0xB268, 0x105C},
	{0xB26A, 0x158C},
	{0xB26C, 0x1080},
	{0xB26E, 0x15C4},
	{0xB270, 0x1702},
	{0xB272, 0xAE0E},
	{0xB274, 0x71CF},
	{0xB276, 0xFFFF},
	{0xB278, 0x994C},
	{0xB27A, 0x9100},
	{0xB27C, 0x0A0D},
	{0xB27E, 0x011E},
	{0xB280, 0xB881},
	{0xB282, 0xB100},
	{0xB284, 0xD800},
	{0xB286, 0xF005},
	{0xB288, 0xB8A1},
	{0xB28A, 0xB100},
	{0xB28C, 0xD801},
	{0xB28E, 0x0CEA},
	{0xB290, 0x0B40},
	{0xB292, 0x71CF},
	{0xB294, 0x0000},
	{0xB296, 0xE06A},
	{0xB298, 0x7960},
	{0xB29A, 0x70E9},
	{0xB29C, 0x01B9},
	{0xB29E, 0x0E40},
	{0xB2A0, 0xC0F1},
	{0xB2A2, 0x70CF},
	{0xB2A4, 0x0001},
	{0xB2A6, 0x0F7E},
	{0xB2A8, 0x7840},
	{0xB2AA, 0xE88E},
	{0xB2AC, 0x70CF},
	{0xB2AE, 0xFFFF},
	{0xB2B0, 0x9B5C},
	{0xB2B2, 0x9001},
	{0xB2B4, 0x0811},
	{0xB2B6, 0x015E},
	{0xB2B8, 0x71CF},
	{0xB2BA, 0xFFFF},
	{0xB2BC, 0x952C},
	{0xB2BE, 0x8909},
	{0xB2C0, 0xB8A5},
	{0xB2C2, 0xA909},
	{0xB2C4, 0xD800},
	{0xB2C6, 0xC0D1},
	{0xB2C8, 0x7EE0},
	{0xB2CA, 0x78E0},
	{0xB2CC, 0xC0F1},
	{0xB2CE, 0x097A},
	{0xB2D0, 0x0E40},
	{0xB2D2, 0x77CF},
	{0xB2D4, 0xFFFF},
	{0xB2D6, 0x888C},
	{0xB2D8, 0x8F14},
	{0xB2DA, 0x75CF},
	{0xB2DC, 0xFFFF},
	{0xB2DE, 0x9388},
	{0xB2E0, 0xC1A1},
	{0xB2E2, 0x15E5},
	{0xB2E4, 0x174E},
	{0xB2E6, 0xE806},
	{0xB2E8, 0x15CB},
	{0xB2EA, 0x170A},
	{0xB2EC, 0x7049},
	{0xB2EE, 0xF032},
	{0xB2F0, 0x70CF},
	{0xB2F2, 0xFFFF},
	{0xB2F4, 0x9890},
	{0xB2F6, 0x9004},
	{0xB2F8, 0xD908},
	{0xB2FA, 0x0F6E},
	{0xB2FC, 0x06A0},
	{0xB2FE, 0x7228},
	{0xB300, 0x7108},
	{0xB302, 0x72CF},
	{0xB304, 0xFFFF},
	{0xB306, 0x9B5C},
	{0xB308, 0x8204},
	{0xB30A, 0x8243},
	{0xB30C, 0x7842},
	{0xB30E, 0x096E},
	{0xB310, 0x0E60},
	{0xB312, 0xB80C},
	{0xB314, 0xD904},
	{0xB316, 0x0866},
	{0xB318, 0x06E0},
	{0xB31A, 0xDA00},
	{0xB31C, 0x71CF},
	{0xB31E, 0x0000},
	{0xB320, 0xFFFF},
	{0xB322, 0x7030},
	{0xB324, 0x22CA},
	{0xB326, 0x0046},
	{0xB328, 0x22CA},
	{0xB32A, 0x0005},
	{0xB32C, 0x71CF},
	{0xB32E, 0xFFFF},
	{0xB330, 0xAE1C},
	{0xB332, 0x15CA},
	{0xB334, 0x1740},
	{0xB336, 0xB146},
	{0xB338, 0x784C},
	{0xB33A, 0x9108},
	{0xB33C, 0x15CB},
	{0xB33E, 0x170A},
	{0xB340, 0x7027},
	{0xB342, 0x2208},
	{0xB344, 0x1001},
	{0xB346, 0x15CC},
	{0xB348, 0x1700},
	{0xB34A, 0x2009},
	{0xB34C, 0x0040},
	{0xB34E, 0x7810},
	{0xB350, 0x72CF},
	{0xB352, 0xFFFF},
	{0xB354, 0xAE1C},
	{0xB356, 0xB207},
	{0xB358, 0x7910},
	{0xB35A, 0x70CF},
	{0xB35C, 0xFFFF},
	{0xB35E, 0x9658},
	{0xB360, 0x9854},
	{0xB362, 0x0E0D},
	{0xB364, 0x1083},
	{0xB366, 0x15CC},
	{0xB368, 0x170A},
	{0xB36A, 0xF015},
	{0xB36C, 0x15CE},
	{0xB36E, 0x174B},
	{0xB370, 0x0B23},
	{0xB372, 0x1382},
	{0xB374, 0x9875},
	{0xB376, 0x0B0F},
	{0xB378, 0x03A2},
	{0xB37A, 0xC640},
	{0xB37C, 0x15CC},
	{0xB37E, 0x1700},
	{0xB380, 0xF006},
	{0xB382, 0x15CD},
	{0xB384, 0x1742},
	{0xB386, 0x7028},
	{0xB388, 0x7149},
	{0xB38A, 0x7369},
	{0xB38C, 0x0E1E},
	{0xB38E, 0x0680},
	{0xB390, 0x7059},
	{0xB392, 0x222F},
	{0xB394, 0x0288},
	{0xB396, 0x70E9},
	{0xB398, 0x0EAA},
	{0xB39A, 0x06A0},
	{0xB39C, 0x15CF},
	{0xB39E, 0x1701},
	{0xB3A0, 0x15C5},
	{0xB3A2, 0x1702},
	{0xB3A4, 0x2A05},
	{0xB3A6, 0x003E},
	{0xB3A8, 0x7048},
	{0xB3AA, 0x15C8},
	{0xB3AC, 0x1702},
	{0xB3AE, 0x712F},
	{0xB3B0, 0x1D63},
	{0xB3B2, 0x1E58},
	{0xB3B4, 0x0BEA},
	{0xB3B6, 0x0B60},
	{0xB3B8, 0x15C9},
	{0xB3BA, 0x1703},
	{0xB3BC, 0x0099},
	{0xB3BE, 0x0E60},
	{0xB3C0, 0xC0A1},
	{0xB3C2, 0x78E0},
	{0xB3C4, 0xC0F1},
	{0xB3C6, 0x0886},
	{0xB3C8, 0x0E60},
	{0xB3CA, 0xD900},
	{0xB3CC, 0xF009},
	{0xB3CE, 0x70CF},
	{0xB3D0, 0xFFFF},
	{0xB3D2, 0xB4A0},
	{0xB3D4, 0x7835},
	{0xB3D6, 0x8041},
	{0xB3D8, 0x8000},
	{0xB3DA, 0xE102},
	{0xB3DC, 0xA040},
	{0xB3DE, 0x09F3},
	{0xB3E0, 0x8214},
	{0xB3E2, 0x76CF},
	{0xB3E4, 0xFFFF},
	{0xB3E6, 0x9388},
	{0xB3E8, 0x16C4},
	{0xB3EA, 0x1700},
	{0xB3EC, 0x75CF},
	{0xB3EE, 0xFFFF},
	{0xB3F0, 0x994C},
	{0xB3F2, 0xB884},
	{0xB3F4, 0x1EC4},
	{0xB3F6, 0x101C},
	{0xB3F8, 0x9500},
	{0xB3FA, 0xB881},
	{0xB3FC, 0xB500},
	{0xB3FE, 0x0B7A},
	{0xB400, 0x0B60},
	{0xB402, 0xD800},
	{0xB404, 0x9501},
	{0xB406, 0x71CF},
	{0xB408, 0xFFFF},
	{0xB40A, 0x9B5C},
	{0xB40C, 0x1ECA},
	{0xB40E, 0x1F9C},
	{0xB410, 0x0000},
	{0xB412, 0xFFB6},
	{0xB414, 0x1ECE},
	{0xB416, 0x1F9C},
	{0xB418, 0x0000},
	{0xB41A, 0x0D00},
	{0xB41C, 0xB887},
	{0xB41E, 0xB501},
	{0xB420, 0x9101},
	{0xB422, 0x1ECD},
	{0xB424, 0x1F9C},
	{0xB426, 0x0000},
	{0xB428, 0x0C80},
	{0xB42A, 0xB8A9},
	{0xB42C, 0xB101},
	{0xB42E, 0xD83F},
	{0xB430, 0x1DB4},
	{0xB432, 0x1004},
	{0xB434, 0xD807},
	{0xB436, 0x1DB6},
	{0xB438, 0x1004},
	{0xB43A, 0xD803},
	{0xB43C, 0x1DB8},
	{0xB43E, 0x1004},
	{0xB440, 0xD808},
	{0xB442, 0x1DBA},
	{0xB444, 0x1004},
	{0xB446, 0xD820},
	{0xB448, 0x1DBC},
	{0xB44A, 0x1004},
	{0xB44C, 0x1DBE},
	{0xB44E, 0x1004},
	{0xB450, 0x208A},
	{0xB452, 0x0004},
	{0xB454, 0x1DC0},
	{0xB456, 0x1004},
	{0xB458, 0x1DC2},
	{0xB45A, 0x1004},
	{0xB45C, 0x1DC4},
	{0xB45E, 0x1004},
	{0xB460, 0xD806},
	{0xB462, 0x1ECF},
	{0xB464, 0x101C},
	{0xB466, 0x208A},
	{0xB468, 0x0609},
	{0xB46A, 0x1ECB},
	{0xB46C, 0x101C},
	{0xB46E, 0x168C},
	{0xB470, 0x1081},
	{0xB472, 0x70CF},
	{0xB474, 0xFFFF},
	{0xB476, 0xAE1C},
	{0xB478, 0x1810},
	{0xB47A, 0x0F84},
	{0xB47C, 0x0000},
	{0xB47E, 0x104A},
	{0xB480, 0x70CF},
	{0xB482, 0xFFFF},
	{0xB484, 0xB490},
	{0xB486, 0xA82E},
	{0xB488, 0x07D5},
	{0xB48A, 0x0E20},
	{0xB48C, 0xD800},
	{0xB48E, 0x0000},
	{0xB490, 0x0000},
	{0xB492, 0x0000},
	{0xB494, 0x0000},
	{0xB496, 0x1000},
	{0xB498, 0x0000},
	{0xB49A, 0x0000},
	{0xB49C, 0x0000},
	{0xB49E, 0x0000},
	{0xB4A0, 0xFFFF},
	{0xB4A2, 0xA3DC},
	{0xB4A4, 0xFFFF},
	{0xB4A6, 0xB1F0},
	{0xB4A8, 0xFFFF},
	{0xB4AA, 0xA3E0},
	{0xB4AC, 0xFFFF},
	{0xB4AE, 0xB230},
	{0xB4B0, 0xFFFF},
	{0xB4B2, 0xA4C0},
	{0xB4B4, 0xFFFF},
	{0xB4B6, 0xB2A0},
	{0xB4B8, 0xFFFF},
	{0xB4BA, 0xA3CC},
	{0xB4BC, 0xFFFF},
	{0xB4BE, 0xB2CC}
};

reg_item_u16_t ap0101_contents0311[] = {
	{0xB4C0, 0xC0F1},
	{0xB4C2, 0x0F8A},
	{0xB4C4, 0x0E00},
	{0xB4C6, 0x75CF},
	{0xB4C8, 0xFFFF},
	{0xB4CA, 0x9388},
	{0xB4CC, 0x153D},
	{0xB4CE, 0x1080},
	{0xB4D0, 0x2086},
	{0xB4D2, 0x03FC},
	{0xB4D4, 0x2842},
	{0xB4D6, 0x813E},
	{0xB4D8, 0x000E},
	{0xB4DA, 0x000F},
	{0xB4DC, 0xD805},
	{0xB4DE, 0x082A},
	{0xB4E0, 0x0520},
	{0xB4E2, 0x7108},
	{0xB4E4, 0xF0DB},
	{0xB4E6, 0x70CF},
	{0xB4E8, 0x0001},
	{0xB4EA, 0x8430},
	{0xB4EC, 0x8021},
	{0xB4EE, 0xE00A},
	{0xB4F0, 0x8140},
	{0xB4F2, 0x0A0F},
	{0xB4F4, 0x0001},
	{0xB4F6, 0x70CF},
	{0xB4F8, 0x0001},
	{0xB4FA, 0x4084},
	{0xB4FC, 0xA100},
	{0xB4FE, 0x70CF},
	{0xB500, 0x0001},
	{0xB502, 0x8104},
	{0xB504, 0x8021},
	{0xB506, 0xE00A},
	{0xB508, 0x8140},
	{0xB50A, 0x0A0F},
	{0xB50C, 0x0001},
	{0xB50E, 0x70CF},
	{0xB510, 0x0001},
	{0xB512, 0x4598},
	{0xB514, 0xA100},
	{0xB516, 0x70CF},
	{0xB518, 0x0001},
	{0xB51A, 0x80E4},
	{0xB51C, 0x8021},
	{0xB51E, 0xE00A},
	{0xB520, 0x8140},
	{0xB522, 0x0A0F},
	{0xB524, 0x0001},
	{0xB526, 0x70CF},
	{0xB528, 0x0001},
	{0xB52A, 0x480C},
	{0xB52C, 0xA100},
	{0xB52E, 0x70CF},
	{0xB530, 0x0001},
	{0xB532, 0x80F4},
	{0xB534, 0x8021},
	{0xB536, 0xE00A},
	{0xB538, 0x8140},
	{0xB53A, 0x0A0F},
	{0xB53C, 0x0001},
	{0xB53E, 0x70CF},
	{0xB540, 0x0001},
	{0xB542, 0x4898},
	{0xB544, 0xA100},
	{0xB546, 0x70CF},
	{0xB548, 0x0001},
	{0xB54A, 0x81E8},
	{0xB54C, 0x8021},
	{0xB54E, 0xE00A},
	{0xB550, 0x8140},
	{0xB552, 0x0A0F},
	{0xB554, 0x0001},
	{0xB556, 0x70CF},
	{0xB558, 0x0001},
	{0xB55A, 0x47E4},
	{0xB55C, 0xA100},
	{0xB55E, 0x70CF},
	{0xB560, 0x0001},
	{0xB562, 0x822C},
	{0xB564, 0x8021},
	{0xB566, 0xE00A},
	{0xB568, 0x8140},
	{0xB56A, 0x0A0F},
	{0xB56C, 0x0001},
	{0xB56E, 0x70CF},
	{0xB570, 0x0001},
	{0xB572, 0x3FAC},
	{0xB574, 0xA100},
	{0xB576, 0x70CF},
	{0xB578, 0x0001},
	{0xB57A, 0x8250},
	{0xB57C, 0x8021},
	{0xB57E, 0xE00A},
	{0xB580, 0x8140},
	{0xB582, 0x0A0F},
	{0xB584, 0x0001},
	{0xB586, 0x70CF},
	{0xB588, 0x0001},
	{0xB58A, 0x3FC8},
	{0xB58C, 0xA100},
	{0xB58E, 0x70CF},
	{0xB590, 0x0001},
	{0xB592, 0x8274},
	{0xB594, 0x8021},
	{0xB596, 0xE00A},
	{0xB598, 0x8140},
	{0xB59A, 0x0A0F},
	{0xB59C, 0x0001},
	{0xB59E, 0x70CF},
	{0xB5A0, 0x0001},
	{0xB5A2, 0x4644},
	{0xB5A4, 0xA100},
	{0xB5A6, 0x70CF},
	{0xB5A8, 0x0001},
	{0xB5AA, 0x80B0},
	{0xB5AC, 0x8021},
	{0xB5AE, 0xE00A},
	{0xB5B0, 0x8140},
	{0xB5B2, 0x0A0F},
	{0xB5B4, 0x0001},
	{0xB5B6, 0x70CF},
	{0xB5B8, 0x0001},
	{0xB5BA, 0x48C0},
	{0xB5BC, 0xA100},
	{0xB5BE, 0x70CF},
	{0xB5C0, 0x0001},
	{0xB5C2, 0x8500},
	{0xB5C4, 0x8021},
	{0xB5C6, 0xE00A},
	{0xB5C8, 0x8140},
	{0xB5CA, 0x0A0F},
	{0xB5CC, 0x0001},
	{0xB5CE, 0x70CF},
	{0xB5D0, 0x0001},
	{0xB5D2, 0x4798},
	{0xB5D4, 0xA100},
	{0xB5D6, 0x850C},
	{0xB5D8, 0xDE20},
	{0xB5DA, 0xB8A0},
	{0xB5DC, 0xA50C},
	{0xB5DE, 0x208A},
	{0xB5E0, 0x0010},
	{0xB5E2, 0xB50E},
	{0xB5E4, 0x1544},
	{0xB5E6, 0x1100},
	{0xB5E8, 0xB5D1},
	{0xB5EA, 0x71CF},
	{0xB5EC, 0xFFFF},
	{0xB5EE, 0x94F4},
	{0xB5F0, 0x2086},
	{0xB5F2, 0x001F},
	{0xB5F4, 0x1D44},
	{0xB5F6, 0x1004},
	{0xB5F8, 0xD854},
	{0xB5FA, 0xB514},
	{0xB5FC, 0xD82F},
	{0xB5FE, 0x1DCA},
	{0xB600, 0x1004},
	{0xB602, 0xD88C},
	{0xB604, 0x1DC6},
	{0xB606, 0x1004},
	{0xB608, 0xD835},
	{0xB60A, 0x1DCE},
	{0xB60C, 0x1004},
	{0xB60E, 0x208A},
	{0xB610, 0x0D45},
	{0xB612, 0x1DCC},
	{0xB614, 0x1004},
	{0xB616, 0x208A},
	{0xB618, 0x0FCF},
	{0xB61A, 0x1DC8},
	{0xB61C, 0x1004},
	{0xB61E, 0x208A},
	{0xB620, 0x001A},
	{0xB622, 0xB515},
	{0xB624, 0xD834},
	{0xB626, 0xB516},
	{0xB628, 0xD800},
	{0xB62A, 0x1D2F},
	{0xB62C, 0x1002},
	{0xB62E, 0xD802},
	{0xB630, 0x1D2E},
	{0xB632, 0x1002},
	{0xB634, 0x8911},
	{0xB636, 0xB5D0},
	{0xB638, 0xB884},
	{0xB63A, 0xA911},
	{0xB63C, 0x71CF},
	{0xB63E, 0xFFFF},
	{0xB640, 0x994C},
	{0xB642, 0x9101},
	{0xB644, 0xB5CF},
	{0xB646, 0xB8A1},
	{0xB648, 0xB101},
	{0xB64A, 0x208A},
	{0xB64C, 0x0015},
	{0xB64E, 0xB500},
	{0xB650, 0x208A},
	{0xB652, 0x040D},
	{0xB654, 0xB501},
	{0xB656, 0x950B},
	{0xB658, 0xD9A0},
	{0xB65A, 0xB509},
	{0xB65C, 0xB508},
	{0xB65E, 0x70CF},
	{0xB660, 0xFFFF},
	{0xB662, 0x97B0},
	{0xB664, 0x0EF2},
	{0xB666, 0x0260},
	{0xB668, 0xB03B},
	{0xB66A, 0xE898},
	{0xB66C, 0xB5C3},
	{0xB66E, 0x208A},
	{0xB670, 0x07D4},
	{0xB672, 0xB505},
	{0xB674, 0xD840},
	{0xB676, 0xB502},
	{0xB678, 0x208A},
	{0xB67A, 0x03CC},
	{0xB67C, 0xB504},
	{0xB67E, 0x228A},
	{0xB680, 0x0014},
	{0xB682, 0x1DA0},
	{0xB684, 0x1084},
	{0xB686, 0x218A},
	{0xB688, 0x040B},
	{0xB68A, 0x1DA2},
	{0xB68C, 0x1044},
	{0xB68E, 0x70CF},
	{0xB690, 0xFFFF},
	{0xB692, 0x95E0},
	{0xB694, 0xB05C},
	{0xB696, 0xB03D},
	{0xB698, 0xD800},
	{0xB69A, 0x05C5},
	{0xB69C, 0x0E00},
	{0xB69E, 0x78E0},
	{0xB6A0, 0x73CF},
	{0xB6A2, 0xFFFF},
	{0xB6A4, 0x85A8},
	{0xB6A6, 0x8341},
	{0xB6A8, 0x0A17},
	{0xB6AA, 0x0911},
	{0xB6AC, 0x9344},
	{0xB6AE, 0xE282},
	{0xB6B0, 0x22CA},
	{0xB6B2, 0x0521},
	{0xB6B4, 0xF204},
	{0xB6B6, 0x0A0B},
	{0xB6B8, 0x0551},
	{0xB6BA, 0xDA1A},
	{0xB6BC, 0xB344},
	{0xB6BE, 0x01C9},
	{0xB6C0, 0x0C40},
	{0xB6C2, 0x78E0},
	{0xB6C4, 0xD900},
	{0xB6C6, 0xF00A},
	{0xB6C8, 0x70CF},
	{0xB6CA, 0xFFFF},
	{0xB6CC, 0xB6E0},
	{0xB6CE, 0x7835},
	{0xB6D0, 0x8041},
	{0xB6D2, 0x8000},
	{0xB6D4, 0xE102},
	{0xB6D6, 0xA040},
	{0xB6D8, 0x09F1},
	{0xB6DA, 0x8114},
	{0xB6DC, 0x7FE0},
	{0xB6DE, 0xD800},
	{0xB6E0, 0xFFFF},
	{0xB6E2, 0xA5D8},
	{0xB6E4, 0xFFFF},
	{0xB6E6, 0xB4C0},
	{0xB6E8, 0xFFFF},
	{0xB6EA, 0xA59C},
	{0xB6EC, 0xFFFF},
	{0xB6EE, 0xB6A0}
};

reg_item_u16_t ap0101_contents0611[] = {
	{0xB6F0, 0xC0F1},
	{0xB6F2, 0x0D56},
	{0xB6F4, 0x0E00},
	{0xB6F6, 0x09B6},
	{0xB6F8, 0x0560},
	{0xB6FA, 0xC1A1},
	{0xB6FC, 0x7508},
	{0xB6FE, 0x70CF},
	{0xB700, 0x0001},
	{0xB702, 0x17FE},
	{0xB704, 0x7840},
	{0xB706, 0x77CF},
	{0xB708, 0xFFFF},
	{0xB70A, 0x9690},
	{0xB70C, 0x9700},
	{0xB70E, 0x0839},
	{0xB710, 0x001E},
	{0xB712, 0x70CF},
	{0xB714, 0xFFFF},
	{0xB716, 0x98B4},
	{0xB718, 0x9824},
	{0xB71A, 0x70CF},
	{0xB71C, 0xFFFF},
	{0xB71E, 0xB97A},
	{0xB720, 0x73CF},
	{0xB722, 0xFFFF},
	{0xB724, 0xAE1C},
	{0xB726, 0xB024},
	{0xB728, 0x71CF},
	{0xB72A, 0xFFFF},
	{0xB72C, 0x952C},
	{0xB72E, 0x9156},
	{0xB730, 0x9324},
	{0xB732, 0x0A0D},
	{0xB734, 0x0062},
	{0xB736, 0x8800},
	{0xB738, 0x9343},
	{0xB73A, 0xF003},
	{0xB73C, 0x9342},
	{0xB73E, 0x090E},
	{0xB740, 0x0560},
	{0xB742, 0xD901},
	{0xB744, 0xE580},
	{0xB746, 0xF2BF},
	{0xB748, 0x75CF},
	{0xB74A, 0xFFFF},
	{0xB74C, 0xB97A},
	{0xB74E, 0x8D03},
	{0xB750, 0xE837},
	{0xB752, 0x094E},
	{0xB754, 0x0560},
	{0xB756, 0x8D01},
	{0xB758, 0x71CF},
	{0xB75A, 0x0FFF},
	{0xB75C, 0xF000},
	{0xB75E, 0x76CF},
	{0xB760, 0xFFFF},
	{0xB762, 0xAE1C},
	{0xB764, 0x6038},
	{0xB766, 0x7827},
	{0xB768, 0xB601},
	{0xB76A, 0x70CF},
	{0xB76C, 0xFFFF},
	{0xB76E, 0x85A8},
	{0xB770, 0x881E},
	{0xB772, 0x0825},
	{0xB774, 0x0091},
	{0xB776, 0x71CF},
	{0xB778, 0x0000},
	{0xB77A, 0x3194},
	{0xB77C, 0x0916},
	{0xB77E, 0x0560},
	{0xB780, 0x8D02},
	{0xB782, 0x9E21},
	{0xB784, 0x8D02},
	{0xB786, 0x72CF},
	{0xB788, 0xFFFF},
	{0xB78A, 0x93C4},
	{0xB78C, 0x2148},
	{0xB78E, 0x0001},
	{0xB790, 0x924D},
	{0xB792, 0xF012},
	{0xB794, 0x082B},
	{0xB796, 0x0051},
	{0xB798, 0x71CF},
	{0xB79A, 0x0000},
	{0xB79C, 0x318A},
	{0xB79E, 0x08F6},
	{0xB7A0, 0x0560},
	{0xB7A2, 0x8D02},
	{0xB7A4, 0x9E21},
	{0xB7A6, 0x8D02},
	{0xB7A8, 0x72CF},
	{0xB7AA, 0xFFFF},
	{0xB7AC, 0x93C4},
	{0xB7AE, 0x9248},
	{0xB7B0, 0x2148},
	{0xB7B2, 0x0001},
	{0xB7B4, 0x7A22},
	{0xB7B6, 0x7A50},
	{0xB7B8, 0x0892},
	{0xB7BA, 0x0560},
	{0xB7BC, 0xD901},
	{0xB7BE, 0x0A26},
	{0xB7C0, 0x04C0},
	{0xB7C2, 0xE080},
	{0xB7C4, 0x21CA},
	{0xB7C6, 0x0F82},
	{0xB7C8, 0x0000},
	{0xB7CA, 0x3182},
	{0xB7CC, 0x21CA},
	{0xB7CE, 0x0F81},
	{0xB7D0, 0x0000},
	{0xB7D2, 0x31A4},
	{0xB7D4, 0x08BE},
	{0xB7D6, 0x0560},
	{0xB7D8, 0x8D01},
	{0xB7DA, 0x8D01},
	{0xB7DC, 0xD900},
	{0xB7DE, 0x086E},
	{0xB7E0, 0x0560},
	{0xB7E2, 0x7228},
	{0xB7E4, 0xD801},
	{0xB7E6, 0xAD03},
	{0xB7E8, 0x70CF},
	{0xB7EA, 0xFFFF},
	{0xB7EC, 0x98B4},
	{0xB7EE, 0x8810},
	{0xB7F0, 0x08D3},
	{0xB7F2, 0x001F},
	{0xB7F4, 0x9700},
	{0xB7F6, 0x08CF},
	{0xB7F8, 0x001F},
	{0xB7FA, 0x77CF},
	{0xB7FC, 0xFFFF},
	{0xB7FE, 0xAE1C},
	{0xB800, 0x9700},
	{0xB802, 0x08C3},
	{0xB804, 0x0F83},
	{0xB806, 0x0000},
	{0xB808, 0x1000},
	{0xB80A, 0x9F21},
	{0xB80C, 0xB807},
	{0xB80E, 0xB523},
	{0xB810, 0x0C6A},
	{0xB812, 0x0E20},
	{0xB814, 0x2148},
	{0xB816, 0x0041},
	{0xB818, 0x0ABE},
	{0xB81A, 0x06A0},
	{0xB81C, 0xD908},
	{0xB81E, 0x9D24},
	{0xB820, 0x2099},
	{0xB822, 0x0803},
	{0xB824, 0x76CF},
	{0xB826, 0xFFFF},
	{0xB828, 0x9738},
	{0xB82A, 0x6038},
	{0xB82C, 0x780E},
	{0xB82E, 0x71CF},
	{0xB830, 0xFFFF},
	{0xB832, 0x9F6C},
	{0xB834, 0x964E},
	{0xB836, 0x8939},
	{0xB838, 0x2208},
	{0xB83A, 0x0000},
	{0xB83C, 0x9650},
	{0xB83E, 0x7A34},
	{0xB840, 0x7210},
	{0xB842, 0x20CA},
	{0xB844, 0x0085},
	{0xB846, 0x780E},
	{0xB848, 0xC040},
	{0xB84A, 0xC200},
	{0xB84C, 0x9611},
	{0xB84E, 0x7210},
	{0xB850, 0x22D7},
	{0xB852, 0x004A},
	{0xB854, 0x20CC},
	{0xB856, 0x808A},
	{0xB858, 0x006A},
	{0xB85A, 0x000A},
	{0xB85C, 0xD801},
	{0xB85E, 0x71CF},
	{0xB860, 0xFFFF},
	{0xB862, 0x98B4},
	{0xB864, 0x098E},
	{0xB866, 0x04E0},
	{0xB868, 0xA910},
	{0xB86A, 0xC100},
	{0xB86C, 0x9610},
	{0xB86E, 0x2009},
	{0xB870, 0x0040},
	{0xB872, 0xB611},
	{0xB874, 0x70CF},
	{0xB876, 0xFFFF},
	{0xB878, 0x9F6C},
	{0xB87A, 0x881A},
	{0xB87C, 0x76CF},
	{0xB87E, 0xFFFF},
	{0xB880, 0x85A8},
	{0xB882, 0x8E3E},
	{0xB884, 0x083B},
	{0xB886, 0x001E},
	{0xB888, 0xE991},
	{0xB88A, 0x9F41},
	{0xB88C, 0x9705},
	{0xB88E, 0x2248},
	{0xB890, 0x0002},
	{0xB892, 0x220E},
	{0xB894, 0x0F81},
	{0xB896, 0x0000},
	{0xB898, 0x1000},
	{0xB89A, 0x0BE2},
	{0xB89C, 0x0E20},
	{0xB89E, 0xB80C},
	{0xB8A0, 0x71CF},
	{0xB8A2, 0xFFFF},
	{0xB8A4, 0x943C},
	{0xB8A6, 0xB109},
	{0xB8A8, 0xF009},
	{0xB8AA, 0x8D04},
	{0xB8AC, 0x0913},
	{0xB8AE, 0x0000},
	{0xB8B0, 0x9725},
	{0xB8B2, 0x70CF},
	{0xB8B4, 0xFFFF},
	{0xB8B6, 0x943C},
	{0xB8B8, 0xB029},
	{0xB8BA, 0x0922},
	{0xB8BC, 0x0580},
	{0xB8BE, 0x8E1E},
	{0xB8C0, 0xAD04},
	{0xB8C2, 0x0395},
	{0xB8C4, 0x0E20},
	{0xB8C6, 0xC0A1},
	{0xB8C8, 0xC0F1},
	{0xB8CA, 0x0B7E},
	{0xB8CC, 0x0E20},
	{0xB8CE, 0xD900},
	{0xB8D0, 0xC1A1},
	{0xB8D2, 0xF00A},
	{0xB8D4, 0x70CF},
	{0xB8D6, 0xFFFF},
	{0xB8D8, 0xB984},
	{0xB8DA, 0x7835},
	{0xB8DC, 0x8041},
	{0xB8DE, 0x8000},
	{0xB8E0, 0xE102},
	{0xB8E2, 0xA040},
	{0xB8E4, 0x09F1},
	{0xB8E6, 0x8094},
	{0xB8E8, 0x70CF},
	{0xB8EA, 0xFFFF},
	{0xB8EC, 0x9348},
	{0xB8EE, 0x9031},
	{0xB8F0, 0x75CF},
	{0xB8F2, 0xFFFF},
	{0xB8F4, 0xAE1C},
	{0xB8F6, 0x78B0},
	{0xB8F8, 0x7110},
	{0xB8FA, 0xD808},
	{0xB8FC, 0xF43E},
	{0xB8FE, 0x1D04},
	{0xB900, 0x1F84},
	{0xB902, 0x0000},
	{0xB904, 0x183C},
	{0xB906, 0x1D06},
	{0xB908, 0x1F84},
	{0xB90A, 0x0000},
	{0xB90C, 0x187C},
	{0xB90E, 0xD860},
	{0xB910, 0xB504},
	{0xB912, 0x76CF},
	{0xB914, 0xFFFF},
	{0xB916, 0xB97A},
	{0xB918, 0x71CF},
	{0xB91A, 0x0000},
	{0xB91C, 0x3198},
	{0xB91E, 0x0EFA},
	{0xB920, 0x0520},
	{0xB922, 0x70C9},
	{0xB924, 0xC040},
	{0xB926, 0x6E01},
	{0xB928, 0xDF00},
	{0xB92A, 0x0EEE},
	{0xB92C, 0x0520},
	{0xB92E, 0x71E9},
	{0xB930, 0xC100},
	{0xB932, 0x7825},
	{0xB934, 0xC040},
	{0xB936, 0x6E02},
	{0xB938, 0x0EDE},
	{0xB93A, 0x0520},
	{0xB93C, 0x71E9},
	{0xB93E, 0xC100},
	{0xB940, 0x72CF},
	{0xB942, 0xFFFF},
	{0xB944, 0x9F6C},
	{0xB946, 0x7825},
	{0xB948, 0x8A3A},
	{0xB94A, 0xAEE3},
	{0xB94C, 0xB980},
	{0xB94E, 0xAA3A},
	{0xB950, 0xD940},
	{0xB952, 0xAA39},
	{0xB954, 0x218A},
	{0xB956, 0x0008},
	{0xB958, 0xB520},
	{0xB95A, 0x71CF},
	{0xB95C, 0xFFFF},
	{0xB95E, 0x943C},
	{0xB960, 0x9129},
	{0xB962, 0xB525},
	{0xB964, 0x71CF},
	{0xB966, 0xFFFF},
	{0xB968, 0x85A8},
	{0xB96A, 0x893E},
	{0xB96C, 0xAE24},
	{0xB96E, 0x71CF},
	{0xB970, 0xFFFF},
	{0xB972, 0x98B4},
	{0xB974, 0x9924},
	{0xB976, 0xB624},
	{0xB978, 0xF1A5},
	{0xB97A, 0x0000},
	{0xB97C, 0x0000},
	{0xB97E, 0x0000},
	{0xB980, 0x0000},
	{0xB982, 0x0000},
	{0xB984, 0xFFFF},
	{0xB986, 0xA4F8},
	{0xB988, 0xFFFF},
	{0xB98A, 0xB6F0}
};

reg_item_u16_t ap0101_contents1411[] = {
	{0xB98C, 0xC0F1},
	{0xB98E, 0x70CF},
	{0xB990, 0x0000},
	{0xB992, 0x195E},
	{0xB994, 0x7840},
	{0xB996, 0x70CF},
	{0xB998, 0xFFFF},
	{0xB99A, 0xAE1C},
	{0xB99C, 0x9821},
	{0xB99E, 0x72CF},
	{0xB9A0, 0xFFFF},
	{0xB9A2, 0x9644},
	{0xB9A4, 0x8A02},
	{0xB9A6, 0x0923},
	{0xB9A8, 0x0002},
	{0xB9AA, 0x71CF},
	{0xB9AC, 0xFFFF},
	{0xB9AE, 0xB9F2},
	{0xB9B0, 0x9100},
	{0xB9B2, 0xE888},
	{0xB9B4, 0x8A03},
	{0xB9B6, 0x083E},
	{0xB9B8, 0x04E0},
	{0xB9BA, 0xB100},
	{0xB9BC, 0xC0D1},
	{0xB9BE, 0x7EE0},
	{0xB9C0, 0xB861},
	{0xB9C2, 0xB100},
	{0xB9C4, 0xF1FC},
	{0xB9C6, 0xF1FC},
	{0xB9C8, 0xD900},
	{0xB9CA, 0xF00A},
	{0xB9CC, 0x70CF},
	{0xB9CE, 0xFFFF},
	{0xB9D0, 0xB9F4},
	{0xB9D2, 0x7835},
	{0xB9D4, 0x8041},
	{0xB9D6, 0x8000},
	{0xB9D8, 0xE102},
	{0xB9DA, 0xA040},
	{0xB9DC, 0x09F1},
	{0xB9DE, 0x8094},
	{0xB9E0, 0xD814},
	{0xB9E2, 0x71CF},
	{0xB9E4, 0xFFFF},
	{0xB9E6, 0x9644},
	{0xB9E8, 0xA902},
	{0xB9EA, 0xD80F},
	{0xB9EC, 0xA903},
	{0xB9EE, 0x7FE0},
	{0xB9F0, 0xD800},
	{0xB9F2, 0x0000},
	{0xB9F4, 0xFFFF},
	{0xB9F6, 0xA018},
	{0xB9F8, 0xFFFF},
	{0xB9FA, 0xB98C}
};

reg_item_u16_t ap0101_contents1511[] = {
	{0xB9FC, 0xDA00},
	{0xB9FE, 0xF003},
	{0xBA00, 0xE201},
	{0xBA02, 0x0A15},
	{0xBA04, 0x0003},
	{0xBA06, 0xD900},
	{0xBA08, 0x09F9},
	{0xBA0A, 0x8F83},
	{0xBA0C, 0x0000},
	{0xBA0E, 0x08C0},
	{0xBA10, 0xE101},
	{0xBA12, 0xF1FC},
	{0xBA14, 0x7EE0},
	{0xBA16, 0x78E0},
	{0xBA18, 0xC0F1},
	{0xBA1A, 0x0A32},
	{0xBA1C, 0x0E20},
	{0xBA1E, 0x218A},
	{0xBA20, 0x0008},
	{0xBA22, 0xC1A1},
	{0xBA24, 0x70CF},
	{0xBA26, 0x0000},
	{0xBA28, 0x304C},
	{0xBA2A, 0x0F52},
	{0xBA2C, 0x05A0},
	{0xBA2E, 0xDD00},
	{0xBA30, 0xD910},
	{0xBA32, 0x70CF},
	{0xBA34, 0x0000},
	{0xBA36, 0x304A},
	{0xBA38, 0x0F42},
	{0xBA3A, 0x05A0},
	{0xBA3C, 0x1C00},
	{0xBA3E, 0x3044},
	{0xBA40, 0x76CF},
	{0xBA42, 0xFFFF},
	{0xBA44, 0x9388},
	{0xBA46, 0x1637},
	{0xBA48, 0x1080},
	{0xBA4A, 0xFFED},
	{0xBA4C, 0xF002},
	{0xBA4E, 0xE501},
	{0xBA50, 0x258C},
	{0xBA52, 0x9A0F},
	{0xBA54, 0xF64C},
	{0xBA56, 0x70CF},
	{0xBA58, 0x0000},
	{0xBA5A, 0x304A},
	{0xBA5C, 0x0F62},
	{0xBA5E, 0x05A0},
	{0xBA60, 0x718B},
	{0xBA62, 0xE8F7},
	{0xBA64, 0x1400},
	{0xBA66, 0x3100},
	{0xBA68, 0x08E7},
	{0xBA6A, 0x815E},
	{0xBA6C, 0x1400},
	{0xBA6E, 0x3100},
	{0xBA70, 0x080B},
	{0xBA72, 0x019E},
	{0xBA74, 0x258C},
	{0xBA76, 0x9A0F},
	{0xBA78, 0xF686},
	{0xBA7A, 0xD801},
	{0xBA7C, 0x0A8A},
	{0xBA7E, 0x04E0},
	{0xBA80, 0x7108},
	{0xBA82, 0xF013},
	{0xBA84, 0x70CF},
	{0xBA86, 0x0000},
	{0xBA88, 0x300E},
	{0xBA8A, 0x0C82},
	{0xBA8C, 0x0BA0},
	{0xBA8E, 0x2440},
	{0xBA90, 0x3081},
	{0xBA92, 0x1402},
	{0xBA94, 0x3100},
	{0xBA96, 0x2004},
	{0xBA98, 0x0F80},
	{0xBA9A, 0x0000},
	{0xBA9C, 0xFF00},
	{0xBA9E, 0xB848},
	{0xBAA0, 0x1E3D},
	{0xBAA2, 0x1002},
	{0xBAA4, 0xD800},
	{0xBAA6, 0x01B9},
	{0xBAA8, 0x0E20},
	{0xBAAA, 0xC0A1},
	{0xBAAC, 0xC0F1},
	{0xBAAE, 0x099A},
	{0xBAB0, 0x0E20},
	{0xBAB2, 0xD808},
	{0xBAB4, 0xC1A2},
	{0xBAB6, 0xDE00},
	{0xBAB8, 0x0C56},
	{0xBABA, 0x0D20},
	{0xBABC, 0x7108},
	{0xBABE, 0xD801},
	{0xBAC0, 0xFFCF},
	{0xBAC2, 0xD810},
	{0xBAC4, 0x0AD2},
	{0xBAC6, 0x0D20},
	{0xBAC8, 0x7108},
	{0xBACA, 0x0B7A},
	{0xBACC, 0x0D20},
	{0xBACE, 0x70C9},
	{0xBAD0, 0xD801},
	{0xBAD2, 0xFFCB},
	{0xBAD4, 0x0B6E},
	{0xBAD6, 0x0D20},
	{0xBAD8, 0xD801},
	{0xBADA, 0xD801},
	{0xBADC, 0xFFC8},
	{0xBADE, 0x0B66},
	{0xBAE0, 0x0D20},
	{0xBAE2, 0x70C9},
	{0xBAE4, 0x75CF},
	{0xBAE6, 0xFFFF},
	{0xBAE8, 0x9388},
	{0xBAEA, 0x1536},
	{0xBAEC, 0x1080},
	{0xBAEE, 0xFFC4},
	{0xBAF0, 0x77C9},
	{0xBAF2, 0xF003},
	{0xBAF4, 0xE601},
	{0xBAF6, 0x268C},
	{0xBAF8, 0x9A0F},
	{0xBAFA, 0xF658},
	{0xBAFC, 0x0E0B},
	{0xBAFE, 0x101F},
	{0xBB00, 0x1534},
	{0xBB02, 0x108F},
	{0xBB04, 0xF003},
	{0xBB06, 0x1535},
	{0xBB08, 0x108F},
	{0xBB0A, 0xBFA0},
	{0xBB0C, 0x0BE6},
	{0xBB0E, 0x0BA0},
	{0xBB10, 0x78EF},
	{0xBB12, 0x70CF},
	{0xBB14, 0x0000},
	{0xBB16, 0x304A},
	{0xBB18, 0x0EA6},
	{0xBB1A, 0x05A0},
	{0xBB1C, 0xC181},
	{0xBB1E, 0xE8EC},
	{0xBB20, 0x1404},
	{0xBB22, 0x3100},
	{0xBB24, 0x08D1},
	{0xBB26, 0x815E},
	{0xBB28, 0x268C},
	{0xBB2A, 0x9A0F},
	{0xBB2C, 0xF686},
	{0xBB2E, 0xD801},
	{0xBB30, 0x09D6},
	{0xBB32, 0x04E0},
	{0xBB34, 0x7108},
	{0xBB36, 0xF026},
	{0xBB38, 0x718B},
	{0xBB3A, 0xD803},
	{0xBB3C, 0x0BCE},
	{0xBB3E, 0x0BA0},
	{0xBB40, 0xB80C},
	{0xBB42, 0x70CF},
	{0xBB44, 0x0000},
	{0xBB46, 0x300E},
	{0xBB48, 0x0BC2},
	{0xBB4A, 0x0BA0},
	{0xBB4C, 0x2440},
	{0xBB4E, 0x3081},
	{0xBB50, 0x1400},
	{0xBB52, 0x3100},
	{0xBB54, 0xB51F},
	{0xBB56, 0x1402},
	{0xBB58, 0x3100},
	{0xBB5A, 0x2004},
	{0xBB5C, 0x0F80},
	{0xBB5E, 0x0000},
	{0xBB60, 0xFF00},
	{0xBB62, 0xB848},
	{0xBB64, 0x1D3D},
	{0xBB66, 0x1002},
	{0xBB68, 0x153D},
	{0xBB6A, 0x1080},
	{0xBB6C, 0x2086},
	{0xBB6E, 0x03FC},
	{0xBB70, 0x2842},
	{0xBB72, 0x813E},
	{0xBB74, 0x1D3C},
	{0xBB76, 0x13C2},
	{0xBB78, 0xF403},
	{0xBB7A, 0xFFA8},
	{0xBB7C, 0xF002},
	{0xBB7E, 0xD800},
	{0xBB80, 0x00D5},
	{0xBB82, 0x0E20},
	{0xBB84, 0xC0A2},
	{0xBB86, 0x78E0},
	{0xBB88, 0xD900},
	{0xBB8A, 0xF00A},
	{0xBB8C, 0x70CF},
	{0xBB8E, 0xFFFF},
	{0xBB90, 0xBBBC},
	{0xBB92, 0x7835},
	{0xBB94, 0x8041},
	{0xBB96, 0x8000},
	{0xBB98, 0xE102},
	{0xBB9A, 0xA040},
	{0xBB9C, 0x09F1},
	{0xBB9E, 0x8094},
	{0xBBA0, 0x71CF},
	{0xBBA2, 0xFFFF},
	{0xBBA4, 0x93A4},
	{0xBBA6, 0x891A},
	{0xBBA8, 0xE883},
	{0xBBAA, 0xD801},
	{0xBBAC, 0xA91A},
	{0xBBAE, 0x891B},
	{0xBBB0, 0xE883},
	{0xBBB2, 0xD81F},
	{0xBBB4, 0xA91B},
	{0xBBB6, 0x7FE0},
	{0xBBB8, 0xD800},
	{0xBBBA, 0x0000},
	{0xBBBC, 0xFFFF},
	{0xBBBE, 0xA568},
	{0xBBC0, 0xFFFF},
	{0xBBC2, 0xBAAC}
};

static int  ap0101_LoadAp0101Patch(int index)
{
	unsigned short size, start, phy_addr;
	unsigned short loader_addr, patch_id;
	unsigned short firmware_id_msb, firmware_id_lsb, reserve;
	int result;
	
	//Patch 0111; CRITICAL ; Ap0101 ADACD Update
	start = 0;
	phy_addr = 0x268c;
	size = sizeof(ap0101_contents0111) / sizeof(ap0101_contents0111[0]) * 2;
	loader_addr = 0x0714;
	patch_id = 0x0111;
	firmware_id_msb = 0x6103;
	firmware_id_lsb = 0x0201;
	reserve = 0x083c;
	result = ap0101_LoadPatch(index, start, size, phy_addr, ap0101_contents0111, loader_addr, patch_id, firmware_id_msb, firmware_id_lsb, reserve);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s: Patch%x load failed, result = %d \n", __func__, patch_id, result);
		return result;
	}
	dev_dbg(&max9286_data[0].i2c_client->dev,
		"%s: Patch%x loaded.\n start=0x%x, size=0x%x, phyAddr=0x%x, loaderAddr=0x%x, reserve=0x%x \n", __func__, patch_id, start, size, phy_addr, loader_addr, reserve);

	//Patch 0211; CRITICAL ; FW ALTM control for AP0101 (all sensor configurations)
	start = 0x83c;
	phy_addr = 0x2ec8;
	size = sizeof(ap0101_contents0211) / sizeof(ap0101_contents0211[0]) * 2;
	loader_addr = 0x0d38;
	patch_id = 0x0211;
	reserve = 0x05f8;

	result = ap0101_LoadPatch(index, start, size, phy_addr, ap0101_contents0211, loader_addr, patch_id, firmware_id_msb, firmware_id_lsb, reserve);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s: Patch%x load failed, result = %d \n", __func__, patch_id, result);
		return result;
	}
	dev_dbg(&max9286_data[0].i2c_client->dev,
		"%s: Patch%x loaded.\n start=0x%x, size=0x%x, phyAddr=0x%x, loaderAddr=0x%x, reserve=0x%x \n", __func__, patch_id, start, size, phy_addr, loader_addr, reserve);

	//Patch 0311; CRITICAL ; This patch update defaults for the AR0140 Rev2 sensor and prevents the ROM from overwriting those new defaults.
	start = 0xe34;
	phy_addr = 0x34c0;
	size = sizeof(ap0101_contents0311) / sizeof(ap0101_contents0311[0]) * 2;
	loader_addr = 0x1038;
	patch_id = 0x0311;
	reserve = 0x230;

	result = ap0101_LoadPatch(index, start, size, phy_addr, ap0101_contents0311, loader_addr, patch_id, firmware_id_msb, firmware_id_lsb, reserve);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s: Patch%x load failed, result = %d \n", __func__, patch_id, result);
		return result;
	}
	dev_dbg(&max9286_data[0].i2c_client->dev,
		"%s: Patch%x loaded.\n start=0x%x, size=0x%x, phyAddr=0x%x, loaderAddr=0x%x, reserve=0x%x \n", __func__, patch_id, start, size, phy_addr, loader_addr, reserve);

	//Patch 0611; CRITICAL ; Camera Adaptation for AP0101 (all sensor configurations)
	start = 0x1064;
	phy_addr = 0x36f0;
	size = sizeof(ap0101_contents0611) / sizeof(ap0101_contents0611[0]) * 2;
	loader_addr = 0x123c;
	patch_id = 0x0611;
	reserve = 0x29c;

	result = ap0101_LoadPatch(index, start, size, phy_addr, ap0101_contents0611, loader_addr, patch_id, firmware_id_msb, firmware_id_lsb, reserve);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s: Patch%x load failed, result = %d \n", __func__, patch_id, result);
		return result;
	}
	dev_dbg(&max9286_data[0].i2c_client->dev,
		"%s: Patch%x loaded.\n start=0x%x, size=0x%x, phyAddr=0x%x, loaderAddr=0x%x, reserve=0x%x \n", __func__, patch_id, start, size, phy_addr, loader_addr, reserve);

	//Patch 1411; CRITICAL ; Camera Adaptation delta dk retrigger patch
	start = 0x1300;
	phy_addr = 0x398c;
	size = sizeof(ap0101_contents1411) / sizeof(ap0101_contents1411[0]) * 2;
	loader_addr = 0x133c;
	patch_id = 0x1411;
	reserve = 0x0070;

	result = ap0101_LoadPatch(index, start, size, phy_addr, ap0101_contents1411, loader_addr, patch_id, firmware_id_msb, firmware_id_lsb, reserve);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s: Patch%x load failed, result = %d \n", __func__, patch_id, result);
		return result;
	}
	dev_dbg(&max9286_data[0].i2c_client->dev,
		"%s: Patch%x loaded.\n start=0x%x, size=0x%x, phyAddr=0x%x, loaderAddr=0x%x, reserve=0x%x \n", __func__, patch_id, start, size, phy_addr, loader_addr, reserve);

	//Patch 1511; CRITICAL ; Sensor Discovery timing
	start = 0x1370;
	phy_addr = 0x39fc;
	size = sizeof(ap0101_contents1511) / sizeof(ap0101_contents1511[0]) * 2;
	loader_addr = 0x14fc;
	patch_id = 0x1511;
	reserve = 0x1c8;

	result = ap0101_LoadPatch(index, start, size, phy_addr, ap0101_contents1511, loader_addr, patch_id, firmware_id_msb, firmware_id_lsb, reserve);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s: Patch%x load failed, result = %d \n", __func__, patch_id, result);
		return result;
	}
	dev_dbg(&max9286_data[0].i2c_client->dev,
		"%s: Patch%x loaded.\n start=0x%x, size=0x%x, phyAddr=0x%x, loaderAddr=0x%x, reserve=0x%x \n", __func__, patch_id, start, size, phy_addr, loader_addr, reserve);

	return result;
}

// AP0101_AR0140_YUV8 setting
static int ap0101_initialize(int index)
{
	int result;
	unsigned int reg = 0;

	ap0101_read_reg(index, REG_CHIP_VERSION, 2, &reg);
	if ((reg & 0xffff) != 0x0160) {
		dev_err(&max9286_data[0].i2c_client->dev, 
			"%s: AP0101 hasn't been found, reg = 0x%x, index = %d.\n", __func__, reg, index);
		return -1;
	}
	dev_info(&max9286_data[0].i2c_client->dev, "%s: AP0101 was found, index = %d.\n", __func__, index);

	//Check if AP0101 had already been initialized.
	ap0101_read_reg(index, 0xC88D, 1, &reg);
	if((reg & 0xff) == 3) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s:AP0101 is already in Slave (Shutter-Sync) Mode.\n",__func__);
		return RESULT_ENOERR;
	}

	//SYSMGR_CONFIG_MODE
	ap0101_read_reg(index, REG_SYSMGR_CONFIG_MODE, 1, &reg);
	if((reg & 0xff) != 3) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s:AP0101 is not in Host Configuration Mode. reg = 0x%x.\n",__func__, reg);
	}

	result = ap0101_LoadAp0101Patch(index);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s: LoadAp0101Patch failed, result = %d \n", __func__, result);
		return result;
	}
	msleep(10);

	result = ap0101_SensorDiscovery(index);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s: SensorDiscovery failed, result = %d \n", __func__, result);
		return result;
	}

	ap0101_LoadSensorSequencer(index);

	result = ap0101_InitSensor(index);
	if (result != RESULT_ENOERR) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s: InitSensor failed, result = %d \n", __func__, result);
		return result;
	}

	ap0101_InitAp0101Pipeline(index);
	msleep(10);

	result = ap0101_ChangeConfig(index);
	if (result != RESULT_ENOERR)
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s:ChangeConfig failed, result = %d.\n",__func__, result);

	return result;
}
#endif  //CONFIG_SENSOR_AP0101

#ifdef CONFIG_SENSOR_OV10635
#define OV10635_REG_PID		0x300A
#define OV10635_REG_VER		0x300B

struct reg_value {
	unsigned short reg_addr;
	unsigned char val;
	unsigned int delay_ms;
};

static struct reg_value ov10635_init_data[] = {
	{0x0103,0x01,5},  //SOFTWARE RESET
	{0x300c,0x61,5},
	{0x301b,0xff,0},
	{0x301c,0xff,0},
	{0x301a,0xff,0},
	{0x3011,0x42,0},
	{0x6900,0x0c,0},
	{0x6901,0x11,0},
	{0x3503,0x10,0},
	{0x3025,0x03,0},
	{0x3003,0x20,0},
	{0x3004,0x21,0},
	{0x3005,0x20,0},
	{0x3006,0x91,0},
	{0x3600,0x74,0},
	{0x3601,0x2b,0},
	{0x3612,0x00,0},
	{0x3611,0x67,0},
	{0x3633,0xca,0},
	{0x3602,0x2f,0},
	{0x3603,0x00,0},
	{0x3630,0x28,0},
	{0x3631,0x16,0},
	{0x3714,0x10,0},
	{0x371d,0x01,0},
	{0x4300,0x3A,0},
	{0x3007,0x01,0},
	{0x3024,0x01,0},
	{0x3020,0x0b,0},
	{0x3702,0x20,0},
	{0x3703,0x48,0},
	{0x3704,0x32,0},
	{0x3709,0xa8,1},
	{0x3709,0xa8,1},
	{0x370c,0xc7,0},
	{0x370d,0x80,0},
	{0x3712,0x00,0},
	{0x3713,0x20,0},
	{0x3715,0x04,0},
	{0x381d,0x40,0},
	{0x381c,0x00,0},
	{0x3822,0x50,0},
	{0x3824,0x50,0},
	{0x3815,0x8c,0},
	{0x3804,0x05,0},
	{0x3805,0x1f,0},
	{0x3800,0x00,0},
	{0x3801,0x00,0},
	{0x3806,0x03,0},
	{0x3807,0x29,0},
	{0x3802,0x00,0},
	{0x3803,0x04,0},
	{0x3808,0x05,0},
	{0x3809,0x00,0},
	{0x380a,0x03,0},
	{0x380b,0x20,0},
	{0x380c,0x07,0},
	{0x380d,0x71,0},
	{0x6e42,0x03,0},
	{0x6e43,0x48,0},
	{0x380e,0x03,0},
	{0x380f,0x48,0},
	{0x3813,0x02,0},
	{0x3811,0x10,0},
	{0x381f,0x0c,0},
	{0x3828,0x03,0},
	{0x3829,0x10,0},
	{0x382a,0x10,0},
	{0x382b,0x10,0},
	{0x3621,0x64,0},
	{0x5005,0x08,0},
	{0x56d5,0x00,0},
	{0x56d6,0x80,0},
	{0x56d7,0x00,0},
	{0x56d8,0x00,0},
	{0x56d9,0x00,0},
	{0x56da,0x80,0},
	{0x56db,0x00,0},
	{0x56dc,0x00,0},
	{0x56e8,0x00,0},
	{0x56e9,0x7f,0},
	{0x56ea,0x00,0},
	{0x56eb,0x7f,0},
	{0x5100,0x00,0},
	{0x5101,0x80,0},
	{0x5102,0x00,0},
	{0x5103,0x80,0},
	{0x5104,0x00,0},
	{0x5105,0x80,0},
	{0x5106,0x00,0},
	{0x5107,0x80,0},
	{0x5108,0x00,0},
	{0x5109,0x00,0},
	{0x510a,0x00,0},
	{0x510b,0x00,0},
	{0x510c,0x00,0},
	{0x510d,0x00,0},
	{0x510e,0x00,0},
	{0x510f,0x00,0},
	{0x5110,0x00,0},
	{0x5111,0x80,0},
	{0x5112,0x00,0},
	{0x5113,0x80,0},
	{0x5114,0x00,0},
	{0x5115,0x80,0},
	{0x5116,0x00,0},
	{0x5117,0x80,0},
	{0x5118,0x00,0},
	{0x5119,0x00,0},
	{0x511a,0x00,0},
	{0x511b,0x00,0},
	{0x511c,0x00,0},
	{0x511d,0x00,0},
	{0x511e,0x00,0},
	{0x511f,0x00,0},
	{0x56d0,0x00,0},
	{0x5006,0x24,0},
	{0x5608,0x0d,0},
	{0x52d7,0x06,0},
	{0x528d,0x08,0},
	{0x5293,0x12,0},
	{0x52d3,0x12,0},
	{0x5288,0x06,0},
	{0x5289,0x20,0},
	{0x52c8,0x06,0},
	{0x52c9,0x20,0},
	{0x52cd,0x04,0},
	{0x5381,0x00,0},
	{0x5382,0xff,0},
	{0x5589,0x76,0},
	{0x558a,0x47,0},
	{0x558b,0xef,0},
	{0x558c,0xc9,0},
	{0x558d,0x49,0},
	{0x558e,0x30,0},
	{0x558f,0x67,0},
	{0x5590,0x3f,0},
	{0x5591,0xf0,0},
	{0x5592,0x10,0},
	{0x55a2,0x6d,0},
	{0x55a3,0x55,0},
	{0x55a4,0xc3,0},
	{0x55a5,0xb5,0},
	{0x55a6,0x43,0},
	{0x55a7,0x38,0},
	{0x55a8,0x5f,0},
	{0x55a9,0x4b,0},
	{0x55aa,0xf0,0},
	{0x55ab,0x10,0},
	{0x5581,0x52,0},
	{0x5300,0x01,0},
	{0x5301,0x00,0},
	{0x5302,0x00,0},
	{0x5303,0x0e,0},
	{0x5304,0x00,0},
	{0x5305,0x0e,0},
	{0x5306,0x00,0},
	{0x5307,0x36,0},
	{0x5308,0x00,0},
	{0x5309,0xd9,0},
	{0x530a,0x00,0},
	{0x530b,0x0f,0},
	{0x530c,0x00,0},
	{0x530d,0x2c,0},
	{0x530e,0x00,0},
	{0x530f,0x59,0},
	{0x5310,0x00,0},
	{0x5311,0x7b,0},
	{0x5312,0x00,0},
	{0x5313,0x22,0},
	{0x5314,0x00,0},
	{0x5315,0xd5,0},
	{0x5316,0x00,0},
	{0x5317,0x13,0},
	{0x5318,0x00,0},
	{0x5319,0x18,0},
	{0x531a,0x00,0},
	{0x531b,0x26,0},
	{0x531c,0x00,0},
	{0x531d,0xdc,0},
	{0x531e,0x00,0},
	{0x531f,0x02,0},
	{0x5320,0x00,0},
	{0x5321,0x24,0},
	{0x5322,0x00,0},
	{0x5323,0x56,0},
	{0x5324,0x00,0},
	{0x5325,0x85,0},
	{0x5326,0x00,0},
	{0x5327,0x20,0},
	{0x5609,0x01,0},
	{0x560a,0x40,0},
	{0x560b,0x01,0},
	{0x560c,0x40,0},
	{0x560d,0x00,0},
	{0x560e,0xfa,0},
	{0x560f,0x00,0},
	{0x5610,0xfa,0},
	{0x5611,0x02,0},
	{0x5612,0x80,0},
	{0x5613,0x02,0},
	{0x5614,0x80,0},
	{0x5615,0x01,0},
	{0x5616,0x2c,0},
	{0x5617,0x01,0},
	{0x5618,0x2c,0},
	{0x563b,0x01,0},
	{0x563c,0x01,0},
	{0x563d,0x01,0},
	{0x563e,0x01,0},
	{0x563f,0x03,0},
	{0x5640,0x03,0},
	{0x5641,0x03,0},
	{0x5642,0x05,0},
	{0x5643,0x09,0},
	{0x5644,0x05,0},
	{0x5645,0x05,0},
	{0x5646,0x05,0},
	{0x5647,0x05,0},
	{0x5651,0x00,0},
	{0x5652,0x80,0},
	{0x521a,0x01,0},
	{0x521b,0x03,0},
	{0x521c,0x06,0},
	{0x521d,0x0a,0},
	{0x521e,0x0e,0},
	{0x521f,0x12,0},
	{0x5220,0x16,0},
	{0x5223,0x02,0},
	{0x5225,0x04,0},
	{0x5227,0x08,0},
	{0x5229,0x0c,0},
	{0x522b,0x12,0},
	{0x522d,0x18,0},
	{0x522f,0x1e,0},
	{0x5241,0x04,0},
	{0x5242,0x01,0},
	{0x5243,0x03,0},
	{0x5244,0x06,0},
	{0x5245,0x0a,0},
	{0x5246,0x0e,0},
	{0x5247,0x12,0},
	{0x5248,0x16,0},
	{0x524a,0x03,0},
	{0x524c,0x04,0},
	{0x524e,0x08,0},
	{0x5250,0x0c,0},
	{0x5252,0x12,0},
	{0x5254,0x18,0},
	{0x5256,0x1e,0},
	{0x4606,0x07,0},
	{0x4607,0x71,0},
	{0x460a,0x02,0},
	{0x460b,0x70,0},
	{0x460c,0x00,0},
	{0x4620,0x0e,0},
	{0x4700,0x04,0},
	{0x4701,0x00,0},
	{0x4702,0x01,0},
	{0x4703,0x01,0},
	{0x4004,0x04,0},
	{0x4005,0x18,0},
	{0x4001,0x06,0},
	{0x4050,0x22,0},
	{0x4051,0x24,0},
	{0x4052,0x02,0},
	{0x4057,0x9c,0},
	{0x405a,0x00,0},
	{0x4202,0x02,0},
	{0x3023,0x10,0},
	{0x0100,0x01,5},  //STREAM MODE, on
	{0x6f10,0x07,0},
	{0x6f11,0x82,0},
	{0x6f12,0x04,0},
	{0x6f13,0x00,0},
	{0x6f14,0x1f,0},
	{0x6f15,0xdd,0},
	{0x6f16,0x04,0},
	{0x6f17,0x04,0},
	{0x6f18,0x36,0},
	{0x6f19,0x66,0},
	{0x6f1a,0x04,0},
	{0x6f1b,0x08,0},
	{0x6f1c,0x0c,0},
	{0x6f1d,0xe7,0},
	{0x6f1e,0x04,0},
	{0x6f1f,0x0c,0},
	{0xd000,0x19,0},
	{0xd001,0xa0,0},
	{0xd002,0x00,0},
	{0xd003,0x01,0},
	{0xd004,0xa9,0},
	{0xd005,0xad,0},
	{0xd006,0x10,0},
	{0xd007,0x40,0},
	{0xd008,0x44,0},
	{0xd009,0x00,0},
	{0xd00a,0x68,0},
	{0xd00b,0x00,0},
	{0xd00c,0x15,0},
	{0xd00d,0x00,0},
	{0xd00e,0x00,0},
	{0xd00f,0x00,0},
	{0xd010,0x19,0},
	{0xd011,0xa0,0},
	{0xd012,0x00,0},
	{0xd013,0x01,0},
	{0xd014,0xa9,0},
	{0xd015,0xad,0},
	{0xd016,0x12,0},
	{0xd017,0x30,0},
	{0xd018,0x44,0},
	{0xd019,0x00,0},
	{0xd01a,0x68,0},
	{0xd01b,0x00,0},
	{0xd01c,0x15,0},
	{0xd01d,0x00,0},
	{0xd01e,0x00,0},
	{0xd01f,0x00,0},
	{0xd020,0x19,0},
	{0xd021,0xa0,0},
	{0xd022,0x00,0},
	{0xd023,0x01,0},
	{0xd024,0xa9,0},
	{0xd025,0xad,0},
	{0xd026,0x13,0},
	{0xd027,0x18,0},
	{0xd028,0x44,0},
	{0xd029,0x00,0},
	{0xd02a,0x68,0},
	{0xd02b,0x00,0},
	{0xd02c,0x15,0},
	{0xd02d,0x00,0},
	{0xd02e,0x00,0},
	{0xd02f,0x00,0},
	{0xd030,0x19,0},
	{0xd031,0xa0,0},
	{0xd032,0x00,0},
	{0xd033,0x01,0},
	{0xd034,0xa9,0},
	{0xd035,0xad,0},
	{0xd036,0x13,0},
	{0xd037,0x3c,0},
	{0xd038,0x44,0},
	{0xd039,0x00,0},
	{0xd03a,0x68,0},
	{0xd03b,0x00,0},
	{0xd03c,0x15,0},
	{0xd03d,0x00,0},
	{0xd03e,0x00,0},
	{0xd03f,0x00,0},
	{0xd040,0x18,0},
	{0xd041,0x60,0},
	{0xd042,0x00,0},
	{0xd043,0x01,0},
	{0xd044,0x9c,0},
	{0xd045,0x80,0},
	{0xd046,0x00,0},
	{0xd047,0x00,0},
	{0xd048,0xa8,0},
	{0xd049,0x63,0},
	{0xd04a,0x06,0},
	{0xd04b,0x4c,0},
	{0xd04c,0xd8,0},
	{0xd04d,0x03,0},
	{0xd04e,0x20,0},
	{0xd04f,0x00,0},
	{0xd050,0x18,0},
	{0xd051,0x60,0},
	{0xd052,0x00,0},
	{0xd053,0x01,0},
	{0xd054,0xa8,0},
	{0xd055,0x63,0},
	{0xd056,0x02,0},
	{0xd057,0xa4,0},
	{0xd058,0x84,0},
	{0xd059,0xa3,0},
	{0xd05a,0x00,0},
	{0xd05b,0x00,0},
	{0xd05c,0x18,0},
	{0xd05d,0x60,0},
	{0xd05e,0x00,0},
	{0xd05f,0x01,0},
	{0xd060,0xa8,0},
	{0xd061,0x63,0},
	{0xd062,0x03,0},
	{0xd063,0xf0,0},
	{0xd064,0x8c,0},
	{0xd065,0x85,0},
	{0xd066,0x00,0},
	{0xd067,0x6e,0},
	{0xd068,0x98,0},
	{0xd069,0x63,0},
	{0xd06a,0x00,0},
	{0xd06b,0x00,0},
	{0xd06c,0xe5,0},
	{0xd06d,0x83,0},
	{0xd06e,0x20,0},
	{0xd06f,0x00,0},
	{0xd070,0x10,0},
	{0xd071,0x00,0},
	{0xd072,0x00,0},
	{0xd073,0x4b,0},
	{0xd074,0x18,0},
	{0xd075,0x80,0},
	{0xd076,0x80,0},
	{0xd077,0x06,0},
	{0xd078,0x18,0},
	{0xd079,0x60,0},
	{0xd07a,0x00,0},
	{0xd07b,0x01,0},
	{0xd07c,0xa8,0},
	{0xd07d,0x63,0},
	{0xd07e,0x01,0},
	{0xd07f,0x00,0},
	{0xd080,0x94,0},
	{0xd081,0x85,0},
	{0xd082,0x00,0},
	{0xd083,0x70,0},
	{0xd084,0x94,0},
	{0xd085,0x63,0},
	{0xd086,0x00,0},
	{0xd087,0x02,0},
	{0xd088,0xe4,0},
	{0xd089,0x83,0},
	{0xd08a,0x20,0},
	{0xd08b,0x00,0},
	{0xd08c,0x10,0},
	{0xd08d,0x00,0},
	{0xd08e,0x00,0},
	{0xd08f,0x44,0},
	{0xd090,0x18,0},
	{0xd091,0x80,0},
	{0xd092,0x80,0},
	{0xd093,0x06,0},
	{0xd094,0x8c,0},
	{0xd095,0x65,0},
	{0xd096,0x00,0},
	{0xd097,0x76,0},
	{0xd098,0xbc,0},
	{0xd099,0x23,0},
	{0xd09a,0x00,0},
	{0xd09b,0x00,0},
	{0xd09c,0x0c,0},
	{0xd09d,0x00,0},
	{0xd09e,0x00,0},
	{0xd09f,0x41,0},
	{0xd0a0,0xa8,0},
	{0xd0a1,0x64,0},
	{0xd0a2,0x38,0},
	{0xd0a3,0x29,0},
	{0xd0a4,0x18,0},
	{0xd0a5,0x80,0},
	{0xd0a6,0x80,0},
	{0xd0a7,0x06,0},
	{0xd0a8,0xa8,0},
	{0xd0a9,0x64,0},
	{0xd0aa,0x38,0},
	{0xd0ab,0x24,0},
	{0xd0ac,0x8c,0},
	{0xd0ad,0x63,0},
	{0xd0ae,0x00,0},
	{0xd0af,0x00,0},
	{0xd0b0,0xa4,0},
	{0xd0b1,0x63,0},
	{0xd0b2,0x00,0},
	{0xd0b3,0x40,0},
	{0xd0b4,0xbc,0},
	{0xd0b5,0x23,0},
	{0xd0b6,0x00,0},
	{0xd0b7,0x00,0},
	{0xd0b8,0x10,0},
	{0xd0b9,0x00,0},
	{0xd0ba,0x00,0},
	{0xd0bb,0x0a,0},
	{0xd0bc,0x18,0},
	{0xd0bd,0xe0,0},
	{0xd0be,0x80,0},
	{0xd0bf,0x06,0},
	{0xd0c0,0x9c,0},
	{0xd0c1,0xc0,0},
	{0xd0c2,0x00,0},
	{0xd0c3,0x40,0},
	{0xd0c4,0xa8,0},
	{0xd0c5,0x64,0},
	{0xd0c6,0x55,0},
	{0xd0c7,0x86,0},
	{0xd0c8,0xa8,0},
	{0xd0c9,0xa4,0},
	{0xd0ca,0x55,0},
	{0xd0cb,0x87,0},
	{0xd0cc,0xd8,0},
	{0xd0cd,0x03,0},
	{0xd0ce,0x30,0},
	{0xd0cf,0x00,0},
	{0xd0d0,0xa8,0},
	{0xd0d1,0x64,0},
	{0xd0d2,0x55,0},
	{0xd0d3,0x88,0},
	{0xd0d4,0xd8,0},
	{0xd0d5,0x05,0},
	{0xd0d6,0x30,0},
	{0xd0d7,0x00,0},
	{0xd0d8,0xd8,0},
	{0xd0d9,0x03,0},
	{0xd0da,0x30,0},
	{0xd0db,0x00,0},
	{0xd0dc,0x18,0},
	{0xd0dd,0xe0,0},
	{0xd0de,0x80,0},
	{0xd0df,0x06,0},
	{0xd0e0,0xa9,0},
	{0xd0e1,0x07,0},
	{0xd0e2,0x3d,0},
	{0xd0e3,0x05,0},
	{0xd0e4,0x8c,0},
	{0xd0e5,0x68,0},
	{0xd0e6,0x00,0},
	{0xd0e7,0x00,0},
	{0xd0e8,0xb8,0},
	{0xd0e9,0x63,0},
	{0xd0ea,0x00,0},
	{0xd0eb,0x18,0},
	{0xd0ec,0xb8,0},
	{0xd0ed,0x63,0},
	{0xd0ee,0x00,0},
	{0xd0ef,0x98,0},
	{0xd0f0,0xbc,0},
	{0xd0f1,0x03,0},
	{0xd0f2,0x00,0},
	{0xd0f3,0x00,0},
	{0xd0f4,0x10,0},
	{0xd0f5,0x00,0},
	{0xd0f6,0x00,0},
	{0xd0f7,0x10,0},
	{0xd0f8,0xa9,0},
	{0xd0f9,0x67,0},
	{0xd0fa,0x67,0},
	{0xd0fb,0x02,0},
	{0xd0fc,0xb8,0},
	{0xd0fd,0xa3,0},
	{0xd0fe,0x00,0},
	{0xd0ff,0x19,0},
	{0xd100,0x8c,0},
	{0xd101,0x8b,0},
	{0xd102,0x00,0},
	{0xd103,0x00,0},
	{0xd104,0xa9,0},
	{0xd105,0xa7,0},
	{0xd106,0x67,0},
	{0xd107,0x03,0},
	{0xd108,0xb8,0},
	{0xd109,0xc4,0},
	{0xd10a,0x00,0},
	{0xd10b,0x08,0},
	{0xd10c,0x8c,0},
	{0xd10d,0x6d,0},
	{0xd10e,0x00,0},
	{0xd10f,0x00,0},
	{0xd110,0xb8,0},
	{0xd111,0x85,0},
	{0xd112,0x00,0},
	{0xd113,0x98,0},
	{0xd114,0xe0,0},
	{0xd115,0x63,0},
	{0xd116,0x30,0},
	{0xd117,0x04,0},
	{0xd118,0xe0,0},
	{0xd119,0x64,0},
	{0xd11a,0x18,0},
	{0xd11b,0x00,0},
	{0xd11c,0xa4,0},
	{0xd11d,0x83,0},
	{0xd11e,0xff,0},
	{0xd11f,0xff,0},
	{0xd120,0xb8,0},
	{0xd121,0x64,0},
	{0xd122,0x00,0},
	{0xd123,0x48,0},
	{0xd124,0xd8,0},
	{0xd125,0x0b,0},
	{0xd126,0x18,0},
	{0xd127,0x00,0},
	{0xd128,0xd8,0},
	{0xd129,0x0d,0},
	{0xd12a,0x20,0},
	{0xd12b,0x00,0},
	{0xd12c,0x9c,0},
	{0xd12d,0x60,0},
	{0xd12e,0x00,0},
	{0xd12f,0x00,0},
	{0xd130,0xd8,0},
	{0xd131,0x08,0},
	{0xd132,0x18,0},
	{0xd133,0x00,0},
	{0xd134,0x9c,0},
	{0xd135,0x80,0},
	{0xd136,0x00,0},
	{0xd137,0x70,0},
	{0xd138,0xa8,0},
	{0xd139,0x67,0},
	{0xd13a,0x38,0},
	{0xd13b,0x22,0},
	{0xd13c,0xa8,0},
	{0xd13d,0xc7,0},
	{0xd13e,0x38,0},
	{0xd13f,0x43,0},
	{0xd140,0xd8,0},
	{0xd141,0x03,0},
	{0xd142,0x20,0},
	{0xd143,0x00,0},
	{0xd144,0xa8,0},
	{0xd145,0xa7,0},
	{0xd146,0x38,0},
	{0xd147,0x42,0},
	{0xd148,0x9c,0},
	{0xd149,0x80,0},
	{0xd14a,0x00,0},
	{0xd14b,0x00,0},
	{0xd14c,0x9c,0},
	{0xd14d,0x84,0},
	{0xd14e,0x00,0},
	{0xd14f,0x01,0},
	{0xd150,0x8c,0},
	{0xd151,0x65,0},
	{0xd152,0x00,0},
	{0xd153,0x00,0},
	{0xd154,0xbc,0},
	{0xd155,0x03,0},
	{0xd156,0x00,0},
	{0xd157,0x00,0},
	{0xd158,0x10,0},
	{0xd159,0x00,0},
	{0xd15a,0x00,0},
	{0xd15b,0x0c,0},
	{0xd15c,0xa4,0},
	{0xd15d,0x84,0},
	{0xd15e,0x00,0},
	{0xd15f,0xff,0},
	{0xd160,0x8c,0},
	{0xd161,0x66,0},
	{0xd162,0x00,0},
	{0xd163,0x00,0},
	{0xd164,0xbc,0},
	{0xd165,0x03,0},
	{0xd166,0x00,0},
	{0xd167,0x00,0},
	{0xd168,0x10,0},
	{0xd169,0x00,0},
	{0xd16a,0x00,0},
	{0xd16b,0x09,0},
	{0xd16c,0xbc,0},
	{0xd16d,0x04,0},
	{0xd16e,0x00,0},
	{0xd16f,0x02,0},
	{0xd170,0x9c,0},
	{0xd171,0x80,0},
	{0xd172,0x00,0},
	{0xd173,0x00,0},
	{0xd174,0x8c,0},
	{0xd175,0x65,0},
	{0xd176,0x00,0},
	{0xd177,0x00,0},
	{0xd178,0x9c,0},
	{0xd179,0x84,0},
	{0xd17a,0x00,0},
	{0xd17b,0x01,0},
	{0xd17c,0xbc,0},
	{0xd17d,0x03,0},
	{0xd17e,0x00,0},
	{0xd17f,0x00,0},
	{0xd180,0x0f,0},
	{0xd181,0xff,0},
	{0xd182,0xff,0},
	{0xd183,0xf8,0},
	{0xd184,0xa4,0},
	{0xd185,0x84,0},
	{0xd186,0x00,0},
	{0xd187,0xff,0},
	{0xd188,0xbc,0},
	{0xd189,0x04,0},
	{0xd18a,0x00,0},
	{0xd18b,0x02,0},
	{0xd18c,0x0f,0},
	{0xd18d,0xff,0},
	{0xd18e,0xff,0},
	{0xd18f,0xf1,0},
	{0xd190,0x9c,0},
	{0xd191,0x84,0},
	{0xd192,0x00,0},
	{0xd193,0x01,0},
	{0xd194,0x00,0},
	{0xd195,0x00,0},
	{0xd196,0x00,0},
	{0xd197,0x1d,0},
	{0xd198,0x18,0},
	{0xd199,0x60,0},
	{0xd19a,0x80,0},
	{0xd19b,0x06,0},
	{0xd19c,0xa8,0},
	{0xd19d,0x64,0},
	{0xd19e,0x38,0},
	{0xd19f,0x29,0},
	{0xd1a0,0xa8,0},
	{0xd1a1,0xa4,0},
	{0xd1a2,0x40,0},
	{0xd1a3,0x08,0},
	{0xd1a4,0x8c,0},
	{0xd1a5,0x63,0},
	{0xd1a6,0x00,0},
	{0xd1a7,0x00,0},
	{0xd1a8,0xa8,0},
	{0xd1a9,0xe4,0},
	{0xd1aa,0x38,0},
	{0xd1ab,0x2a,0},
	{0xd1ac,0xa8,0},
	{0xd1ad,0xc4,0},
	{0xd1ae,0x40,0},
	{0xd1af,0x09,0},
	{0xd1b0,0xd8,0},
	{0xd1b1,0x05,0},
	{0xd1b2,0x18,0},
	{0xd1b3,0x00,0},
	{0xd1b4,0x8c,0},
	{0xd1b5,0x67,0},
	{0xd1b6,0x00,0},
	{0xd1b7,0x00,0},
	{0xd1b8,0xa8,0},
	{0xd1b9,0xa4,0},
	{0xd1ba,0x38,0},
	{0xd1bb,0x24,0},
	{0xd1bc,0xd8,0},
	{0xd1bd,0x06,0},
	{0xd1be,0x18,0},
	{0xd1bf,0x00,0},
	{0xd1c0,0x8c,0},
	{0xd1c1,0x65,0},
	{0xd1c2,0x00,0},
	{0xd1c3,0x00,0},
	{0xd1c4,0xa4,0},
	{0xd1c5,0x63,0},
	{0xd1c6,0x00,0},
	{0xd1c7,0x40,0},
	{0xd1c8,0xbc,0},
	{0xd1c9,0x23,0},
	{0xd1ca,0x00,0},
	{0xd1cb,0x00,0},
	{0xd1cc,0x13,0},
	{0xd1cd,0xff,0},
	{0xd1ce,0xff,0},
	{0xd1cf,0xc4,0},
	{0xd1d0,0xa8,0},
	{0xd1d1,0x64,0},
	{0xd1d2,0x6e,0},
	{0xd1d3,0x44,0},
	{0xd1d4,0xa8,0},
	{0xd1d5,0xa4,0},
	{0xd1d6,0x55,0},
	{0xd1d7,0x86,0},
	{0xd1d8,0x8c,0},
	{0xd1d9,0x63,0},
	{0xd1da,0x00,0},
	{0xd1db,0x00,0},
	{0xd1dc,0xa8,0},
	{0xd1dd,0xc4,0},
	{0xd1de,0x6e,0},
	{0xd1df,0x45,0},
	{0xd1e0,0xa8,0},
	{0xd1e1,0xe4,0},
	{0xd1e2,0x55,0},
	{0xd1e3,0x87,0},
	{0xd1e4,0xd8,0},
	{0xd1e5,0x05,0},
	{0xd1e6,0x18,0},
	{0xd1e7,0x00,0},
	{0xd1e8,0x8c,0},
	{0xd1e9,0x66,0},
	{0xd1ea,0x00,0},
	{0xd1eb,0x00,0},
	{0xd1ec,0xa8,0},
	{0xd1ed,0xa4,0},
	{0xd1ee,0x6e,0},
	{0xd1ef,0x46,0},
	{0xd1f0,0xd8,0},
	{0xd1f1,0x07,0},
	{0xd1f2,0x18,0},
	{0xd1f3,0x00,0},
	{0xd1f4,0xa8,0},
	{0xd1f5,0x84,0},
	{0xd1f6,0x55,0},
	{0xd1f7,0x88,0},
	{0xd1f8,0x8c,0},
	{0xd1f9,0x65,0},
	{0xd1fa,0x00,0},
	{0xd1fb,0x00,0},
	{0xd1fc,0xd8,0},
	{0xd1fd,0x04,0},
	{0xd1fe,0x18,0},
	{0xd1ff,0x00,0},
	{0xd200,0x03,0},
	{0xd201,0xff,0},
	{0xd202,0xff,0},
	{0xd203,0xb8,0},
	{0xd204,0x18,0},
	{0xd205,0xe0,0},
	{0xd206,0x80,0},
	{0xd207,0x06,0},
	{0xd208,0xa8,0},
	{0xd209,0x83,0},
	{0xd20a,0x6e,0},
	{0xd20b,0x43,0},
	{0xd20c,0xa8,0},
	{0xd20d,0xe3,0},
	{0xd20e,0x38,0},
	{0xd20f,0x0f,0},
	{0xd210,0x8c,0},
	{0xd211,0x84,0},
	{0xd212,0x00,0},
	{0xd213,0x00,0},
	{0xd214,0xa8,0},
	{0xd215,0xa3,0},
	{0xd216,0x38,0},
	{0xd217,0x0e,0},
	{0xd218,0xa8,0},
	{0xd219,0xc3,0},
	{0xd21a,0x6e,0},
	{0xd21b,0x42,0},
	{0xd21c,0xd8,0},
	{0xd21d,0x07,0},
	{0xd21e,0x20,0},
	{0xd21f,0x00,0},
	{0xd220,0x8c,0},
	{0xd221,0x66,0},
	{0xd222,0x00,0},
	{0xd223,0x00,0},
	{0xd224,0xd8,0},
	{0xd225,0x05,0},
	{0xd226,0x18,0},
	{0xd227,0x00,0},
	{0xd228,0x44,0},
	{0xd229,0x00,0},
	{0xd22a,0x48,0},
	{0xd22b,0x00,0},
	{0xd22c,0x15,0},
	{0xd22d,0x00,0},
	{0xd22e,0x00,0},
	{0xd22f,0x00,0},
	{0xd230,0x9c,0},
	{0xd231,0x21,0},
	{0xd232,0xff,0},
	{0xd233,0xfc,0},
	{0xd234,0xd4,0},
	{0xd235,0x01,0},
	{0xd236,0x48,0},
	{0xd237,0x00,0},
	{0xd238,0x18,0},
	{0xd239,0x60,0},
	{0xd23a,0x00,0},
	{0xd23b,0x01,0},
	{0xd23c,0xa8,0},
	{0xd23d,0x63,0},
	{0xd23e,0x07,0},
	{0xd23f,0x80,0},
	{0xd240,0x8c,0},
	{0xd241,0x63,0},
	{0xd242,0x00,0},
	{0xd243,0x68,0},
	{0xd244,0xbc,0},
	{0xd245,0x03,0},
	{0xd246,0x00,0},
	{0xd247,0x00,0},
	{0xd248,0x10,0},
	{0xd249,0x00,0},
	{0xd24a,0x00,0},
	{0xd24b,0x0c,0},
	{0xd24c,0x15,0},
	{0xd24d,0x00,0},
	{0xd24e,0x00,0},
	{0xd24f,0x00,0},
	{0xd250,0x07,0},
	{0xd251,0xff,0},
	{0xd252,0xda,0},
	{0xd253,0x00,0},
	{0xd254,0x15,0},
	{0xd255,0x00,0},
	{0xd256,0x00,0},
	{0xd257,0x00,0},
	{0xd258,0x18,0},
	{0xd259,0x60,0},
	{0xd25a,0x80,0},
	{0xd25b,0x06,0},
	{0xd25c,0xa8,0},
	{0xd25d,0x63,0},
	{0xd25e,0xc4,0},
	{0xd25f,0xb8,0},
	{0xd260,0x8c,0},
	{0xd261,0x63,0},
	{0xd262,0x00,0},
	{0xd263,0x00,0},
	{0xd264,0xbc,0},
	{0xd265,0x23,0},
	{0xd266,0x00,0},
	{0xd267,0x01,0},
	{0xd268,0x10,0},
	{0xd269,0x00,0},
	{0xd26a,0x00,0},
	{0xd26b,0x25,0},
	{0xd26c,0x9d,0},
	{0xd26d,0x00,0},
	{0xd26e,0x00,0},
	{0xd26f,0x00,0},
	{0xd270,0x00,0},
	{0xd271,0x00,0},
	{0xd272,0x00,0},
	{0xd273,0x0b,0},
	{0xd274,0xb8,0},
	{0xd275,0xe8,0},
	{0xd276,0x00,0},
	{0xd277,0x02,0},
	{0xd278,0x07,0},
	{0xd279,0xff,0},
	{0xd27a,0xd6,0},
	{0xd27b,0x8c,0},
	{0xd27c,0x15,0},
	{0xd27d,0x00,0},
	{0xd27e,0x00,0},
	{0xd27f,0x00,0},
	{0xd280,0x18,0},
	{0xd281,0x60,0},
	{0xd282,0x80,0},
	{0xd283,0x06,0},
	{0xd284,0xa8,0},
	{0xd285,0x63,0},
	{0xd286,0xc4,0},
	{0xd287,0xb8,0},
	{0xd288,0x8c,0},
	{0xd289,0x63,0},
	{0xd28a,0x00,0},
	{0xd28b,0x00,0},
	{0xd28c,0xbc,0},
	{0xd28d,0x23,0},
	{0xd28e,0x00,0},
	{0xd28f,0x01,0},
	{0xd290,0x10,0},
	{0xd291,0x00,0},
	{0xd292,0x00,0},
	{0xd293,0x1b,0},
	{0xd294,0x9d,0},
	{0xd295,0x00,0},
	{0xd296,0x00,0},
	{0xd297,0x00,0},
	{0xd298,0xb8,0},
	{0xd299,0xe8,0},
	{0xd29a,0x00,0},
	{0xd29b,0x02,0},
	{0xd29c,0x9c,0},
	{0xd29d,0xc0,0},
	{0xd29e,0x00,0},
	{0xd29f,0x00,0},
	{0xd2a0,0x18,0},
	{0xd2a1,0xa0,0},
	{0xd2a2,0x80,0},
	{0xd2a3,0x06,0},
	{0xd2a4,0xe0,0},
	{0xd2a5,0x67,0},
	{0xd2a6,0x30,0},
	{0xd2a7,0x00,0},
	{0xd2a8,0xa8,0},
	{0xd2a9,0xa5,0},
	{0xd2aa,0xce,0},
	{0xd2ab,0xb0,0},
	{0xd2ac,0x19,0},
	{0xd2ad,0x60,0},
	{0xd2ae,0x00,0},
	{0xd2af,0x01,0},
	{0xd2b0,0xa9,0},
	{0xd2b1,0x6b,0},
	{0xd2b2,0x06,0},
	{0xd2b3,0x14,0},
	{0xd2b4,0xe0,0},
	{0xd2b5,0x83,0},
	{0xd2b6,0x28,0},
	{0xd2b7,0x00,0},
	{0xd2b8,0x9c,0},
	{0xd2b9,0xc6,0},
	{0xd2ba,0x00,0},
	{0xd2bb,0x01,0},
	{0xd2bc,0xe0,0},
	{0xd2bd,0x63,0},
	{0xd2be,0x18,0},
	{0xd2bf,0x00,0},
	{0xd2c0,0x8c,0},
	{0xd2c1,0x84,0},
	{0xd2c2,0x00,0},
	{0xd2c3,0x00,0},
	{0xd2c4,0xe0,0},
	{0xd2c5,0xa3,0},
	{0xd2c6,0x58,0},
	{0xd2c7,0x00,0},
	{0xd2c8,0xa4,0},
	{0xd2c9,0xc6,0},
	{0xd2ca,0x00,0},
	{0xd2cb,0xff,0},
	{0xd2cc,0xb8,0},
	{0xd2cd,0x64,0},
	{0xd2ce,0x00,0},
	{0xd2cf,0x18,0},
	{0xd2d0,0xbc,0},
	{0xd2d1,0x46,0},
	{0xd2d2,0x00,0},
	{0xd2d3,0x03,0},
	{0xd2d4,0x94,0},
	{0xd2d5,0x85,0},
	{0xd2d6,0x00,0},
	{0xd2d7,0x00,0},
	{0xd2d8,0xb8,0},
	{0xd2d9,0x63,0},
	{0xd2da,0x00,0},
	{0xd2db,0x98,0},
	{0xd2dc,0xe0,0},
	{0xd2dd,0x64,0},
	{0xd2de,0x18,0},
	{0xd2df,0x00,0},
	{0xd2e0,0x0f,0},
	{0xd2e1,0xff,0},
	{0xd2e2,0xff,0},
	{0xd2e3,0xf0,0},
	{0xd2e4,0xdc,0},
	{0xd2e5,0x05,0},
	{0xd2e6,0x18,0},
	{0xd2e7,0x00,0},
	{0xd2e8,0x9c,0},
	{0xd2e9,0x68,0},
	{0xd2ea,0x00,0},
	{0xd2eb,0x01,0},
	{0xd2ec,0xa5,0},
	{0xd2ed,0x03,0},
	{0xd2ee,0x00,0},
	{0xd2ef,0xff,0},
	{0xd2f0,0xbc,0},
	{0xd2f1,0x48,0},
	{0xd2f2,0x00,0},
	{0xd2f3,0x01,0},
	{0xd2f4,0x0f,0},
	{0xd2f5,0xff,0},
	{0xd2f6,0xff,0},
	{0xd2f7,0xea,0},
	{0xd2f8,0xb8,0},
	{0xd2f9,0xe8,0},
	{0xd2fa,0x00,0},
	{0xd2fb,0x02,0},
	{0xd2fc,0x18,0},
	{0xd2fd,0x60,0},
	{0xd2fe,0x00,0},
	{0xd2ff,0x01,0},
	{0xd300,0xa8,0},
	{0xd301,0x63,0},
	{0xd302,0x06,0},
	{0xd303,0x14,0},
	{0xd304,0x07,0},
	{0xd305,0xff,0},
	{0xd306,0xe4,0},
	{0xd307,0x6d,0},
	{0xd308,0x9c,0},
	{0xd309,0x83,0},
	{0xd30a,0x00,0},
	{0xd30b,0x10,0},
	{0xd30c,0x85,0},
	{0xd30d,0x21,0},
	{0xd30e,0x00,0},
	{0xd30f,0x00,0},
	{0xd310,0x44,0},
	{0xd311,0x00,0},
	{0xd312,0x48,0},
	{0xd313,0x00,0},
	{0xd314,0x9c,0},
	{0xd315,0x21,0},
	{0xd316,0x00,0},
	{0xd317,0x04,0},
	{0xd318,0x18,0},
	{0xd319,0x60,0},
	{0xd31a,0x00,0},
	{0xd31b,0x01,0},
	{0xd31c,0x9c,0},
	{0xd31d,0x80,0},
	{0xd31e,0xff,0},
	{0xd31f,0xff,0},
	{0xd320,0xa8,0},
	{0xd321,0x63,0},
	{0xd322,0x09,0},
	{0xd323,0xef,0},
	{0xd324,0xd8,0},
	{0xd325,0x03,0},
	{0xd326,0x20,0},
	{0xd327,0x00,0},
	{0xd328,0x18,0},
	{0xd329,0x60,0},
	{0xd32a,0x80,0},
	{0xd32b,0x06,0},
	{0xd32c,0xa8,0},
	{0xd32d,0x63,0},
	{0xd32e,0xc9,0},
	{0xd32f,0xef,0},
	{0xd330,0xd8,0},
	{0xd331,0x03,0},
	{0xd332,0x20,0},
	{0xd333,0x00,0},
	{0xd334,0x44,0},
	{0xd335,0x00,0},
	{0xd336,0x48,0},
	{0xd337,0x00,0},
	{0xd338,0x15,0},
	{0xd339,0x00,0},
	{0xd33a,0x00,0},
	{0xd33b,0x00,0},
	{0xd33c,0x18,0},
	{0xd33d,0x80,0},
	{0xd33e,0x00,0},
	{0xd33f,0x01,0},
	{0xd340,0xa8,0},
	{0xd341,0x84,0},
	{0xd342,0x0a,0},
	{0xd343,0x12,0},
	{0xd344,0x8c,0},
	{0xd345,0x64,0},
	{0xd346,0x00,0},
	{0xd347,0x00,0},
	{0xd348,0xbc,0},
	{0xd349,0x03,0},
	{0xd34a,0x00,0},
	{0xd34b,0x00,0},
	{0xd34c,0x13,0},
	{0xd34d,0xff,0},
	{0xd34e,0xff,0},
	{0xd34f,0xfe,0},
	{0xd350,0x15,0},
	{0xd351,0x00,0},
	{0xd352,0x00,0},
	{0xd353,0x00,0},
	{0xd354,0x44,0},
	{0xd355,0x00,0},
	{0xd356,0x48,0},
	{0xd357,0x00,0},
	{0xd358,0x15,0},
	{0xd359,0x00,0},
	{0xd35a,0x00,0},
	{0xd35b,0x00,0},
	{0xd35c,0x00,0},
	{0xd35d,0x00,0},
	{0xd35e,0x00,0},
	{0xd35f,0x00,0},
	{0xd360,0x00,0},
	{0xd361,0x00,0},
	{0xd362,0x00,0},
	{0xd363,0x00,0},
	{0x6f0e,0x33,0},
	{0x6f0f,0x33,0},
	{0x460e,0x08,0},
	{0x460f,0x01,0},
	{0x4610,0x00,0},
	{0x4611,0x01,0},
	{0x4612,0x00,0},
	{0x4613,0x01,0},
	{0x4605,0x00,0},
	{0x4608,0x00,0},
	{0x4609,0x08,0},
	{0x6804,0x00,0},
	{0x6805,0x06,0},
	{0x6806,0x00,0},
	{0x5120,0x00,0},
	{0x3510,0x00,0},
	{0x3504,0x00,0},
	{0x6800,0x00,0},
	{0x6f0d,0x0f,0},
	{0x5000,0xff,0},
	{0x5001,0xbf,0},
	{0x5002,0x7e,0},
	{0x503d,0x00,0},
	{0xc450,0x01,0},
	{0xc452,0x04,0},
	{0xc453,0x00,0},
	{0xc454,0x00,0},
	{0xc455,0x00,0},
	{0xc456,0x00,0},
	{0xc457,0x00,0},
	{0xc458,0x00,0},
	{0xc459,0x00,0},
	{0xc45b,0x00,0},
	{0xc45c,0x00,0},
	{0xc45d,0x00,0},
	{0xc45e,0x00,0},
	{0xc45f,0x00,0},
	{0xc460,0x00,0},
	{0xc461,0x01,0},
	{0xc462,0x01,0},
	{0xc464,0x88,0},
	{0xc465,0x00,0},
	{0xc466,0x8a,0},
	{0xc467,0x00,0},
	{0xc468,0x86,0},
	{0xc469,0x00,0},
	{0xc46a,0x40,0},
	{0xc46b,0x50,0},
	{0xc46c,0x30,0},
	{0xc46d,0x28,0},
	{0xc46e,0x60,0},
	{0xc46f,0x40,0},
	{0xc47c,0x01,0},
	{0xc47d,0x38,0},
	{0xc47e,0x00,0},
	{0xc47f,0x00,0},
	{0xc480,0x00,0},
	{0xc481,0xff,0},
	{0xc482,0x00,0},
	{0xc483,0x40,0},
	{0xc484,0x00,0},
	{0xc485,0x18,0},
	{0xc486,0x00,0},
	{0xc487,0x18,0},
	{0xc488,0x34,0},
	{0xc489,0x00,0},
	{0xc48a,0x34,0},
	{0xc48b,0x00,0},
	{0xc48c,0x00,0},
	{0xc48d,0x04,0},
	{0xc48e,0x00,0},
	{0xc48f,0x04,0},
	{0xc490,0x07,0},
	{0xc492,0x20,0},
	{0xc493,0x08,0},
	{0xc498,0x02,0},
	{0xc499,0x00,0},
	{0xc49a,0x02,0},
	{0xc49b,0x00,0},
	{0xc49c,0x02,0},
	{0xc49d,0x00,0},
	{0xc49e,0x02,0},
	{0xc49f,0x60,0},
	{0xc4a0,0x03,0},
	{0xc4a1,0x00,0},
	{0xc4a2,0x04,0},
	{0xc4a3,0x00,0},
	{0xc4a4,0x00,0},
	{0xc4a5,0x10,0},
	{0xc4a6,0x00,0},
	{0xc4a7,0x40,0},
	{0xc4a8,0x00,0},
	{0xc4a9,0x80,0},
	{0xc4aa,0x0d,0},
	{0xc4ab,0x00,0},
	{0xc4ac,0x0f,0},
	{0xc4ad,0xc0,0},
	{0xc4b4,0x01,0},
	{0xc4b5,0x01,0},
	{0xc4b6,0x00,0},
	{0xc4b7,0x01,0},
	{0xc4b8,0x00,0},
	{0xc4b9,0x01,0},
	{0xc4ba,0x01,0},
	{0xc4bb,0x00,0},
	{0xc4bc,0x01,0},
	{0xc4bd,0x60,0},
	{0xc4be,0x02,0},
	{0xc4bf,0x33,0},
	{0xc4c8,0x03,0},
	{0xc4c9,0xd0,0},
	{0xc4ca,0x0e,0},
	{0xc4cb,0x00,0},
	{0xc4cc,0x10,0},
	{0xc4cd,0x18,0},
	{0xc4ce,0x10,0},
	{0xc4cf,0x18,0},
	{0xc4d0,0x04,0},
	{0xc4d1,0x80,0},
	{0xc4e0,0x04,0},
	{0xc4e1,0x02,0},
	{0xc4e2,0x01,0},
	{0xc4e4,0x10,0},
	{0xc4e5,0x20,0},
	{0xc4e6,0x30,0},
	{0xc4e7,0x40,0},
	{0xc4e8,0x50,0},
	{0xc4e9,0x60,0},
	{0xc4ea,0x70,0},
	{0xc4eb,0x80,0},
	{0xc4ec,0x90,0},
	{0xc4ed,0xa0,0},
	{0xc4ee,0xb0,0},
	{0xc4ef,0xc0,0},
	{0xc4f0,0xd0,0},
	{0xc4f1,0xe0,0},
	{0xc4f2,0xf0,0},
	{0xc4f3,0x80,0},
	{0xc4f4,0x00,0},
	{0xc4f5,0x20,0},
	{0xc4f6,0x02,0},
	{0xc4f7,0x00,0},
	{0xc4f8,0x04,0},
	{0xc4f9,0x0b,0},
	{0xc4fa,0x00,0},
	{0xc4fb,0x01,0},
	{0xc4fc,0x01,0},
	{0xc4fd,0x00,0},
	{0xc4fe,0x04,0},
	{0xc4ff,0x02,0},
	{0xc500,0x48,0},
	{0xc501,0x74,0},
	{0xc502,0x58,0},
	{0xc503,0x80,0},
	{0xc504,0x05,0},
	{0xc505,0x80,0},
	{0xc506,0x03,0},
	{0xc507,0x80,0},
	{0xc508,0x01,0},
	{0xc509,0xc0,0},
	{0xc50a,0x01,0},
	{0xc50b,0xa0,0},
	{0xc50c,0x01,0},
	{0xc50d,0x2c,0},
	{0xc50e,0x01,0},
	{0xc50f,0x0a,0},
	{0xc510,0x00,0},
	{0xc511,0x01,0},
	{0xc512,0x01,0},
	{0xc513,0x80,0},
	{0xc514,0x04,0},
	{0xc515,0x00,0},
	{0xc518,0x03,0},
	{0xc519,0x48,0},
	{0xc51a,0x07,0},
	{0xc51b,0x70,0},
	{0xc2e0,0x00,0},
	{0xc2e1,0x51,0},
	{0xc2e2,0x00,0},
	{0xc2e3,0xd6,0},
	{0xc2e4,0x01,0},
	{0xc2e5,0x5e,0},
	{0xc2e9,0x01,0},
	{0xc2ea,0x7a,0},
	{0xc2eb,0x90,0},
	{0xc2ed,0x00,0},
	{0xc2ee,0x7a,0},
	{0xc2ef,0x64,0},
	{0xc308,0x00,0},
	{0xc309,0x00,0},
	{0xc30a,0x00,0},
	{0xc30c,0x00,0},
	{0xc30d,0x01,0},
	{0xc30e,0x00,0},
	{0xc30f,0x00,0},
	{0xc310,0x01,0},
	{0xc311,0x60,0},
	{0xc312,0xff,0},
	{0xc313,0x08,0},
	{0xc314,0x01,0},
	{0xc315,0x7f,0},
	{0xc316,0xff,0},
	{0xc317,0x0b,0},
	{0xc318,0x00,0},
	{0xc319,0x0c,0},
	{0xc31a,0x00,0},
	{0xc31b,0xe0,0},
	{0xc31c,0x00,0},
	{0xc31d,0x14,0},
	{0xc31e,0x00,0},
	{0xc31f,0xc5,0},
	{0xc320,0xff,0},
	{0xc321,0x4b,0},
	{0xc322,0xff,0},
	{0xc323,0xf0,0},
	{0xc324,0xff,0},
	{0xc325,0xe8,0},
	{0xc326,0x00,0},
	{0xc327,0x46,0},
	{0xc328,0xff,0},
	{0xc329,0xd2,0},
	{0xc32a,0xff,0},
	{0xc32b,0xe4,0},
	{0xc32c,0xff,0},
	{0xc32d,0xbb,0},
	{0xc32e,0x00,0},
	{0xc32f,0x61,0},
	{0xc330,0xff,0},
	{0xc331,0xf9,0},
	{0xc332,0x00,0},
	{0xc333,0xd9,0},
	{0xc334,0x00,0},
	{0xc335,0x2e,0},
	{0xc336,0x00,0},
	{0xc337,0xb1,0},
	{0xc338,0xff,0},
	{0xc339,0x64,0},
	{0xc33a,0xff,0},
	{0xc33b,0xeb,0},
	{0xc33c,0xff,0},
	{0xc33d,0xe8,0},
	{0xc33e,0x00,0},
	{0xc33f,0x48,0},
	{0xc340,0xff,0},
	{0xc341,0xd0,0},
	{0xc342,0xff,0},
	{0xc343,0xed,0},
	{0xc344,0xff,0},
	{0xc345,0xad,0},
	{0xc346,0x00,0},
	{0xc347,0x66,0},
	{0xc348,0x01,0},
	{0xc349,0x00,0},
	{0x6700,0x04,0},
	{0x6701,0x7b,0},
	{0x6702,0xfd,0},
	{0x6703,0xf9,0},
	{0x6704,0x3d,0},
	{0x6705,0x71,0},
	{0x6706,0x78,0},
	{0x6708,0x05,0},
	{0x6f06,0x6f,0},
	{0x6f07,0x00,0},
	{0x6f0a,0x6f,0},
	{0x6f0b,0x00,0},
	{0x6f00,0x03,0},
	{0xc34c,0x01,0},
	{0xc34d,0x00,0},
	{0xc34e,0x46,0},
	{0xc34f,0x55,0},
	{0xc350,0x00,0},
	{0xc351,0x40,0},
	{0xc352,0x00,0},
	{0xc353,0xff,0},
	{0xc354,0x04,0},
	{0xc355,0x08,0},
	{0xc356,0x01,0},
	{0xc357,0xef,0},
	{0xc358,0x30,0},
	{0xc359,0x01,0},
	{0xc35a,0x64,0},
	{0xc35b,0x46,0},
	{0xc35c,0x00,0},
	{0x3042,0xf0,5},
	{0x301b,0xf0,0},
	{0x301c,0xf0,0},
	{0x301a,0xf0,0},
	{0x4709,0x10,0},
	{0xceb0,0x00,0},
	{0xceb1,0x00,0},
	{0xceb2,0x00,0},
	{0xceb3,0x00,0},
	{0xceb4,0x00,0},
	{0xceb5,0x00,0},
	{0xceb6,0x00,0},
	{0xceb7,0x00,0},
	{0xc4bc,0x01,0},
	{0xc4bd,0x60,0},
	{0x3832,0x01,0},
	{0x3833,0x1A,0},
	{0x3834,0x03,0},
	{0x3835,0x48,0},
	{0x302E,0x01,0}
};

static inline int ov10635_read_reg(int index, unsigned short reg, unsigned char * val)
{
	unsigned char u8_buf[2] = {0};
	unsigned int buf_len = 2;
	int retry, timeout = 10;
	unsigned char u8_val = 0;

	u8_buf[0] = (reg >> 8) & 0xFF;
	u8_buf[1] = reg & 0xFF;

	max9286_data[0].i2c_client->addr = ADDR_OV_SENSOR + index;
	for (retry = 0; retry < timeout; retry ++) {
		if (i2c_master_send(max9286_data[0].i2c_client, u8_buf, buf_len) < 0) {
			dev_dbg(&max9286_data[0].i2c_client->dev,
				"%s:read reg error on send: reg=0x%x, retry = %d.\n", __func__, reg, retry);
			msleep(5);
			continue;
		}

		if (i2c_master_recv(max9286_data[0].i2c_client, &u8_val, 1) != 1) {
			dev_dbg(&max9286_data[0].i2c_client->dev,
				"%s:read reg error on recv: reg=0x%x, retry = %d.\n", __func__, reg, retry);
			msleep(5);
			continue;
		}
		break;
	}

	if (retry >= timeout) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s:read reg error: reg=0x%x.\n", __func__, reg);
		return -1;
	}

	*val = u8_val;

	return u8_val;
}

static inline int ov10635_write_reg(int index, unsigned short reg, unsigned char val)
{
	unsigned char u8_buf[3] = {0};
	unsigned int buf_len = 3;
	int retry, timeout = 10;

	u8_buf[0] = (reg >> 8) & 0xFF;
	u8_buf[1] = reg & 0xFF;
	u8_buf[2] = val;

	max9286_data[0].i2c_client->addr = ADDR_OV_SENSOR + index;
	for (retry = 0; retry < timeout; retry ++) {
		if (i2c_master_send(max9286_data[0].i2c_client, u8_buf, buf_len) < 0) {
			dev_dbg(&max9286_data[0].i2c_client->dev,
				"%s:write reg error: reg=0x%x, val=0x%x, retry = %d.\n", __func__, reg, val, retry);
			msleep(5);
			continue;
		}
		break;
	}

	if (retry >= timeout) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s:write reg error: reg=0x%x, val=0x%x.\n", __func__, reg, val);
		return -1;
	}

	return 0;
}

static int ov10635_check_device(int index)
{
	unsigned char reg = 0;

	ov10635_read_reg(index, OV10635_REG_PID, &reg);
	if (reg != 0xA6) {
		dev_err(&max9286_data[0].i2c_client->dev, 
			"%s: OV10635 hasn't been found, reg[0x%x] = 0x%x.\n", __func__, OV10635_REG_PID, reg);
		return -1;
	}
	ov10635_read_reg(index, OV10635_REG_VER, &reg);
	if (reg != 0x35) {
		dev_err(&max9286_data[0].i2c_client->dev, 
			"%s: OV10635 hasn't been found, reg[0x%x] = 0x%x.\n", __func__, OV10635_REG_VER, reg);
		return -1;
	}
	dev_info(&max9286_data[0].i2c_client->dev, "%s: OV10635 was found.\n", __func__);

	return 0;
}

// OV10635_YUV8 setting
static int ov10635_initialize(int index)
{
	int i, array_size;
	int retval;

	dev_info(&max9286_data[0].i2c_client->dev, "%s: index = %d.\n", __func__, index);
	array_size = ARRAY_SIZE(ov10635_init_data);
	for (i=0; i<array_size; i++) {
		retval = ov10635_write_reg(index, ov10635_init_data[i].reg_addr, ov10635_init_data[i].val);
		if (retval < 0)
			break;
		if (ov10635_init_data[i].delay_ms != 0)
			msleep(ov10635_init_data[i].delay_ms);
	}

	return 0;
}
#endif

static inline int max9271_read_reg(int index, u8 reg)
{
	int val;
	int retry, timeout = 10;

	max9286_data[0].i2c_client->addr = ADDR_MAX9271 + index;
	for (retry = 0; retry < timeout; retry ++) {
		val = i2c_smbus_read_byte_data(max9286_data[0].i2c_client, reg);
		if (val < 0)
			msleep(5);
		else
			break;
	}

	if (retry >= timeout) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s:read reg error: reg=%2x\n", __func__, reg);
		return -1;
	}

	return val;
}

static int max9271_write_reg(int index, u8 reg, u8 val)
{
	s32 ret;
	int retry, timeout = 10;

	max9286_data[0].i2c_client->addr = ADDR_MAX9271 + index;
	for (retry = 0; retry < timeout; retry ++) {
		ret = i2c_smbus_write_byte_data(max9286_data[0].i2c_client, reg, val);
		if (val < 0)
			msleep(5);
		else
			break;
	}

	if (retry >= timeout) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s:write reg error:reg=%2x,val=%2x\n", __func__,
			reg, val);
		return -1;
	}

	return 0;
}

/*! Read one register from a MAX9286 i2c slave device.
 *
 *  @param *reg		register in the device we wish to access.
 *
 *  @return		       0 if success, an error code otherwise.
 */
static inline int max9286_read_reg(u8 reg)
{
	int val;

	max9286_data[0].i2c_client->addr = ADDR_MAX9286;
	val = i2c_smbus_read_byte_data(max9286_data[0].i2c_client, reg);
	if (val < 0) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s:read reg error: reg=%2x\n", __func__, reg);
		return -1;
	}
	return val;
}

/*! Write one register of a MAX9286 i2c slave device.
 *
 *  @param *reg		register in the device we wish to access.
 *
 *  @return		       0 if success, an error code otherwise.
 */
static int max9286_write_reg(u8 reg, u8 val)
{
	s32 ret;

	max9286_data[0].i2c_client->addr = ADDR_MAX9286;
	ret = i2c_smbus_write_byte_data(max9286_data[0].i2c_client, reg, val);
	if (ret < 0) {
		dev_info(&max9286_data[0].i2c_client->dev,
			"%s:write reg error:reg=%2x,val=%2x\n", __func__,
			reg, val);
		return -1;
	}
	return 0;
}
/*
static void max9271_dump_registers(int index)
{
	unsigned char i;
	printk("max9271_dump_registers: index = %d.\r\n", index);
	for (i=0; i<0x20; i++)
		printk("MAX9271 Reg 0x%02x = 0x%x.\r\n", i, max9271_read_reg(index, i));
}

static void max9286_dump_registers(void)
{
	unsigned char i;
	for (i=0; i<0x72; i++)
		printk("MAX9286 Reg 0x%02x = 0x%x.\r\n", i, max9286_read_reg(i));
}
*/
static int max9286_hardware_init(struct sensor_data *sensor)
{
	int retval = 0;
	void *mipi_csi2_info;
	u32 mipi_reg;
	int i, lanes;
	u8 reg, sensor_addr = 0;

	//Disable CSI Output
	max9286_write_reg(0x15, 0x13);

	//Enable PRBS test
	max9286_write_reg(0x0E, 0x5F);
	msleep(10);

	//Enable Custom Reverse Channel & First Pulse Length
	max9286_write_reg(0x3F, 0x4F);
	msleep(2);

	//Reverse Channel Amplitude to mid level and transition time
	max9286_write_reg(0x3B, 0x1E);
	msleep(2);

	//Enable MAX9271 Configuration Link
	max9271_write_reg(0, 0x04, 0x43);
	msleep(2);

	//Increase serializer reverse channel input thresholds
#ifdef CONFIG_SENSOR_OV10635
#ifdef CONFIG_SERIALIZER_MAX96705
	max9271_write_reg(0, 0x40, 0x0F);
	msleep(2);
	max9271_write_reg(0, 0x97, 0x5F);
#endif
#ifdef CONFIG_SERIALIZER_MAX9271
	max9271_write_reg(0, 0x08, 0x01);
#endif
#endif

#ifdef CONFIG_SENSOR_AP0101
#ifdef CONFIG_SERIALIZER_MAX96705
	//Invert VSYNC
	max9271_write_reg(0, 0x40, 0x2F);
	msleep(2);
	max9271_write_reg(0, 0x97, 0x5F);
#endif
#ifdef CONFIG_SERIALIZER_MAX9271
	//Invert VSYNC
	max9271_write_reg(0, 0x08, 0x81);
#endif
#endif
	msleep(2);

	//Reverse Channel Amplitude level
	max9286_write_reg(0x3B, 0x19);
	msleep(5);

	//Set YUV422 8 bits mode, Double Data Rate, 4 data lane
	max9286_write_reg(0x12, 0xF3);

	//Frame Sync
#ifdef CONFIG_SENSOR_OV10635
	//Automatic Mode
	max9286_write_reg(0x01, 0x02);
#endif
#ifdef CONFIG_SENSOR_AP0101
	//Manual Mode
	max9286_write_reg(0x01, 0x00);

	max9286_write_reg(0x63, 0x00);
	max9286_write_reg(0x64, 0x00);

	max9286_write_reg(0x06, 0x00);
	max9286_write_reg(0x07, 0x00);
	max9286_write_reg(0x08, 0x26);
#endif
	msleep(100);

	// Detect link
	g_sensor_num = 0;
	reg = max9286_read_reg(0x49);
	g_sensor_is_there = ((reg >> 4) & 0xF) | (reg & 0xF);
	if (g_sensor_is_there & (0x1 << 0))
		g_sensor_num += 1;
	if (g_sensor_is_there & (0x1 << 1))
		g_sensor_num += 1;
	if (g_sensor_is_there & (0x1 << 2))
		g_sensor_num += 1;
	if (g_sensor_is_there & (0x1 << 3))
		g_sensor_num += 1;
	pr_info("max9286_mipi: sensor number = %d.\n", g_sensor_num);

	if (g_sensor_num == 0) {
		pr_err("%s: no camera connected.\n", __func__);
		return -1;
	}

	//Disable PRBS test
	max9286_write_reg(0x0E, 0x50);

	// Set link order in MIPI CSI-2 output
	reg = 0xE4;  //Default setting
	if (g_sensor_num == 1) {
		switch (g_sensor_is_there) {
			case 0x8:
				reg = 0x27;
				break;
			case 0x4:
				reg = 0xC6;
				break;
			case 0x2:
				reg = 0xE1;
				break;
			case 0x1:
			default:
				reg = 0xE4;
				break;
		}
	} else if (g_sensor_num == 2) {
		switch (g_sensor_is_there) {
			case 0xC:
				reg = 0x4E;
				break;
			case 0xA:
				reg = 0x72;
				break;
			case 0x9:
				reg = 0x78;
				break;
			case 0x6:
				reg = 0xD2;
				break;
			case 0x5:
				reg = 0xD8;
				break;
			case 0x3:
			default:
				reg = 0xE4;
				break;
		}
	} else if (g_sensor_num == 3) {
		switch (g_sensor_is_there) {
			case 0xE:
				reg = 0x93;
				break;
			case 0xD:
				reg = 0x9C;
				break;
			case 0xB:
				reg = 0xB4;
				break;
			case 0x7:
			default:
				reg = 0xE4;
				break;
		}
	}
	max9286_write_reg(0x0B, reg);

	mipi_csi2_info = mipi_csi2_get_info();

	/* initial mipi dphy */
	if (!mipi_csi2_info) {
		printk(KERN_ERR "%s() in %s: Fail to get s_mipi_csi2_info!\n",
		       __func__, __FILE__);
		return -1;
	}

	if (!mipi_csi2_get_status(mipi_csi2_info))
		mipi_csi2_enable(mipi_csi2_info);

	if (!mipi_csi2_get_status(mipi_csi2_info)) {
		pr_err("Can not enable mipi csi2 driver!\n");
		return -1;
	}

	lanes = mipi_csi2_set_lanes(mipi_csi2_info);

	/* Only reset MIPI CSI2 HW at sensor initialize */
#ifdef CONFIG_SENSOR_OV10635
	/* 48MHz pixel clock (1280*800@30fps) * 16 bits per pixel (YUV422) = 768Mbps mipi data rate for each camera */
	mipi_csi2_reset(mipi_csi2_info, (768 * g_sensor_num) / (lanes + 1));
#endif
#ifdef CONFIG_SENSOR_AP0101
	/* 37.125MHz pixel clock (1280*720@30fps) * 16 bits per pixel (YUV422) = 594Mbps mipi data rate for each camera */
	mipi_csi2_reset(mipi_csi2_info, (594 * g_sensor_num) / (lanes + 1));
#endif

	if (sensor->pix.pixelformat == V4L2_PIX_FMT_UYVY) {
		for (i=0; i<MAX_SENSOR_NUM; i++)
			mipi_csi2_set_datatype(mipi_csi2_info, i, MIPI_DT_YUV422);
	} else
		pr_err("currently this sensor format can not be supported!\n");

	//Enable all links
	reg = 0xE0 | g_sensor_is_there;
	max9286_write_reg(0x00, reg);

	//Set up links
#ifdef CONFIG_SENSOR_OV10635
	sensor_addr = ADDR_OV_SENSOR;
#endif
#ifdef CONFIG_SENSOR_AP0101
	sensor_addr = ADDR_AP_SENSOR;
#endif
	reg = 0;
	for (i=1; i<=MAX_SENSOR_NUM; i++) {
		if (((0x1 << (i-1)) & g_sensor_is_there) == 0)
			continue;

		//Enable Link control channel
		reg |= (0x11 << (i-1));
		max9286_write_reg(0x0A, reg);

		//Set MAX9271 new address for link 0
		max9271_write_reg(0, 0x00, (ADDR_MAX9271+ i) << 1);
		msleep(2);
		
		//Set MAX9271: Double Mode, PCLK latched on Rising Edge, HS/VS encoding
		max9271_write_reg(i, 0x07, 0x84);
		msleep(2);

		max9271_write_reg(i, 0x01, ADDR_MAX9286 << 1);
		max9271_write_reg(i, 0x09, (sensor_addr + i) << 1);
		max9271_write_reg(i, 0x0A, sensor_addr << 1);
		max9271_write_reg(i, 0x0B, ADDR_MAX9271_ALL << 1);
		max9271_write_reg(i, 0x0C, (ADDR_MAX9271 + i) << 1);

		msleep(1);
	}
	max9286_write_reg(0x0A, reg);
	max9286_write_reg(0x0A, reg);

	//Disable Local Auto I2C ACK
	max9286_write_reg(0x34, 0x36);

	//Initialize Camera Sensor
#ifdef CONFIG_SENSOR_OV10635
	// Broadcast
	if (g_sensor_is_there & (0x1 << 0)) {
		if (ov10635_check_device(1) == 0)
			ov10635_initialize(0);
	} else if (g_sensor_is_there & (0x1 << 1)) {
		if (ov10635_check_device(2) == 0)
			ov10635_initialize(0);
	} else if (g_sensor_is_there & (0x1 << 2)) {
		if (ov10635_check_device(3) == 0)
			ov10635_initialize(0);
	} else if (g_sensor_is_there & (0x1 << 3)) {
		if (ov10635_check_device(4) == 0)
			ov10635_initialize(0);
	}
#endif
#ifdef CONFIG_SENSOR_AP0101
	if (g_sensor_is_there & (0x1 << 0))
		ap0101_initialize(1);
	if (g_sensor_is_there & (0x1 << 1))
		ap0101_initialize(2);
	if (g_sensor_is_there & (0x1 << 2))
		ap0101_initialize(3);
	if (g_sensor_is_there & (0x1 << 3))
		ap0101_initialize(4);
#endif

	//Enable Local Auto I2C ACK
	max9286_write_reg(0x34, 0xB6);

	//MAX9271: Enable Serial Links and Disable Configuration Link
	max9271_write_reg(ADDR_MAX9271_ALL - ADDR_MAX9271, 0x04, 0x83);
	msleep(100);  //Wait for more than 2 frame time

	//Enable CSI output, set virtual channel according to the link number
	max9286_write_reg(0x15, 0x9B);
	msleep(10);
/*
	printk("Dump MAX9286 registers:\r\n");
	max9286_dump_registers();
*/
	if (mipi_csi2_info) {
		i = 0;

		/* wait for mipi sensor ready */
		mipi_reg = mipi_csi2_dphy_status(mipi_csi2_info);
		while (((mipi_reg & 0x700) != 0x300) && (i < 10)) {
			mipi_reg = mipi_csi2_dphy_status(mipi_csi2_info);
			i++;
			msleep(50);
		}

		if (i >= 10) {
			pr_err("mipi csi2 can not receive sensor clk! MIPI_CSI_PHY_STATE = 0x%x.\n", mipi_reg);
			return -1;
		}

		i = 0;

		/* wait for mipi stable */
		mipi_reg = mipi_csi2_get_error1(mipi_csi2_info);
		while ((mipi_reg != 0x0) && (i < 10)) {
			mipi_reg = mipi_csi2_get_error1(mipi_csi2_info);
			i++;
			msleep(10);
		}

		if (i >= 10) {
			pr_err("mipi csi2 can not reveive data correctly! MIPI_CSI_ERR1 = 0x%x.\n", mipi_reg);
			return -1;
		}
	}

	return retval;
}

/* --------------- IOCTL functions from v4l2_int_ioctl_desc --------------- */

static int ioctl_g_ifparm(struct v4l2_int_device *s, struct v4l2_ifparm *p)
{
	struct sensor_data *sensor;

	if (s == NULL) {
		pr_err("   ERROR!! no slave device set!\n");
		return -1;
	}
	sensor = s->priv;

	memset(p, 0, sizeof(*p));
	p->u.bt656.clock_curr = sensor->mclk;
	pr_debug("   clock_curr=mclk=%d\n", sensor->mclk);
	p->if_type = V4L2_IF_TYPE_BT656;
	p->u.bt656.mode = V4L2_IF_TYPE_BT656_MODE_NOBT_8BIT;
	p->u.bt656.bt_sync_correct = 1;  /* Indicate external vsync */

	return 0;
}

/*!
 * ioctl_s_power - V4L2 sensor interface handler for VIDIOC_S_POWER ioctl
 * @s: pointer to standard V4L2 device structure
 * @on: indicates power mode (on or off)
 *
 * Turns the power on or off, depending on the value of on and returns the
 * appropriate error code.
 */
static int ioctl_s_power(struct v4l2_int_device *s, int on)
{
	struct sensor_data *sensor = s->priv;

	sensor->on = on;

	return 0;
}

/*!
 * ioctl_g_parm - V4L2 sensor interface handler for VIDIOC_G_PARM ioctl
 * @s: pointer to standard V4L2 device structure
 * @a: pointer to standard V4L2 VIDIOC_G_PARM ioctl structure
 *
 * Returns the sensor's video CAPTURE parameters.
 */
static int ioctl_g_parm(struct v4l2_int_device *s, struct v4l2_streamparm *a)
{
	struct sensor_data *sensor = s->priv;
	struct v4l2_captureparm *cparm = &a->parm.capture;
	int ret = 0;

	switch (a->type) {
	/* This is the only case currently handled. */
	case V4L2_BUF_TYPE_VIDEO_CAPTURE:
		memset(a, 0, sizeof(*a));
		a->type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
		cparm->capability = sensor->streamcap.capability;
		cparm->timeperframe = sensor->streamcap.timeperframe;
		cparm->capturemode = sensor->streamcap.capturemode;
		ret = 0;
		break;

	/* These are all the possible cases. */
	case V4L2_BUF_TYPE_VIDEO_OUTPUT:
	case V4L2_BUF_TYPE_VIDEO_OVERLAY:
	case V4L2_BUF_TYPE_VBI_CAPTURE:
	case V4L2_BUF_TYPE_VBI_OUTPUT:
	case V4L2_BUF_TYPE_SLICED_VBI_CAPTURE:
	case V4L2_BUF_TYPE_SLICED_VBI_OUTPUT:
		ret = -EINVAL;
		break;

	default:
		pr_debug("   type is unknown - %d\n", a->type);
		ret = -EINVAL;
		break;
	}

	return ret;
}

/*!
 * ioctl_s_parm - V4L2 sensor interface handler for VIDIOC_S_PARM ioctl
 * @s: pointer to standard V4L2 device structure
 * @a: pointer to standard V4L2 VIDIOC_S_PARM ioctl structure
 *
 * Configures the sensor to use the input parameters, if possible.  If
 * not possible, reverts to the old parameters and returns the
 * appropriate error code.
 */
static int ioctl_s_parm(struct v4l2_int_device *s, struct v4l2_streamparm *a)
{
	int ret = 0;

	switch (a->type) {
	/* This is the only case currently handled. */
	case V4L2_BUF_TYPE_VIDEO_CAPTURE:
		break;

	/* These are all the possible cases. */
	case V4L2_BUF_TYPE_VIDEO_OUTPUT:
	case V4L2_BUF_TYPE_VIDEO_OVERLAY:
	case V4L2_BUF_TYPE_VBI_CAPTURE:
	case V4L2_BUF_TYPE_VBI_OUTPUT:
	case V4L2_BUF_TYPE_SLICED_VBI_CAPTURE:
	case V4L2_BUF_TYPE_SLICED_VBI_OUTPUT:
		pr_debug("   type is not " \
			"V4L2_BUF_TYPE_VIDEO_CAPTURE but %d\n",
			a->type);
		ret = -EINVAL;
		break;

	default:
		pr_debug("   type is unknown - %d\n", a->type);
		ret = -EINVAL;
		break;
	}

	return ret;
}

/*!
 * ioctl_g_fmt_cap - V4L2 sensor interface handler for ioctl_g_fmt_cap
 * @s: pointer to standard V4L2 device structure
 * @f: pointer to standard V4L2 v4l2_format structure
 *
 * Returns the sensor's current pixel format in the v4l2_format
 * parameter.
 */
static int ioctl_g_fmt_cap(struct v4l2_int_device *s, struct v4l2_format *f)
{
	struct sensor_data *sensor = s->priv;

	f->fmt.pix = sensor->pix;

	return 0;
}

static int ioctl_try_fmt_cap(struct v4l2_int_device *s, struct v4l2_format *f)
{
	struct sensor_data *sensor = s->priv;

	if (sensor->i2c_client != NULL) {
		g_max9286_width =  f->fmt.pix.width;
		g_max9286_height =  f->fmt.pix.height;
	}
	sensor->pix.width = g_max9286_width;
	sensor->pix.height = g_max9286_height;

	ioctl_dev_init(s);
	return 0;
}

/*!
 * ioctl_g_ctrl - V4L2 sensor interface handler for VIDIOC_G_CTRL ioctl
 * @s: pointer to standard V4L2 device structure
 * @vc: standard V4L2 VIDIOC_G_CTRL ioctl structure
 *
 * If the requested control is supported, returns the control's current
 * value from the video_control[] array.  Otherwise, returns -EINVAL
 * if the control is not supported.
 */
static int ioctl_g_ctrl(struct v4l2_int_device *s, struct v4l2_control *vc)
{
	int ret = 0;

	ret = -EINVAL;

	return ret;
}

/*!
 * ioctl_s_ctrl - V4L2 sensor interface handler for VIDIOC_S_CTRL ioctl
 * @s: pointer to standard V4L2 device structure
 * @vc: standard V4L2 VIDIOC_S_CTRL ioctl structure
 *
 * If the requested control is supported, sets the control's current
 * value in HW (and updates the video_control[] array).  Otherwise,
 * returns -EINVAL if the control is not supported.
 */
static int ioctl_s_ctrl(struct v4l2_int_device *s, struct v4l2_control *vc)
{
	int ret = 0;
	return ret;
}

/*!
 * ioctl_enum_framesizes - V4L2 sensor interface handler for
 *			   VIDIOC_ENUM_FRAMESIZES ioctl
 * @s: pointer to standard V4L2 device structure
 * @fsize: standard V4L2 VIDIOC_ENUM_FRAMESIZES ioctl structure
 *
 * Return 0 if successful, otherwise -EINVAL.
 */
static int ioctl_enum_framesizes(struct v4l2_int_device *s,
				 struct v4l2_frmsizeenum *fsize)
{
	struct sensor_data *sensor = s->priv;

	if (fsize->index > 0)
		return -EINVAL;

	fsize->pixel_format = sensor->pix.pixelformat;
	fsize->discrete.width = sensor->pix.width;
	fsize->discrete.height = sensor->pix.height;
	return 0;
}

/*!
 * ioctl_enum_frameintervals - V4L2 sensor interface handler for
 *			       VIDIOC_ENUM_FRAMEINTERVALS ioctl
 * @s: pointer to standard V4L2 device structure
 * @fival: standard V4L2 VIDIOC_ENUM_FRAMEINTERVALS ioctl structure
 *
 * Return 0 if successful, otherwise -EINVAL.
 */
static int ioctl_enum_frameintervals(struct v4l2_int_device *s,
					 struct v4l2_frmivalenum *fival)
{
	if (fival->index > 0)
		return -EINVAL;

	if (fival->pixel_format == 0 || fival->width == 0 ||
			fival->height == 0) {
		pr_warning("Please assign pixelformat, width and height.\n");
		return -EINVAL;
	}

	fival->type = V4L2_FRMIVAL_TYPE_DISCRETE;
	fival->discrete.numerator = 1;
	fival->discrete.denominator = 30;

	return 0;
}

/*!
 * ioctl_g_chip_ident - V4L2 sensor interface handler for
 *			VIDIOC_DBG_G_CHIP_IDENT ioctl
 * @s: pointer to standard V4L2 device structure
 * @id: pointer to int
 *
 * Return 0.
 */
static int ioctl_g_chip_ident(struct v4l2_int_device *s, int *id)
{
	((struct v4l2_dbg_chip_ident *)id)->match.type =
					V4L2_CHIP_MATCH_I2C_DRIVER;
	strcpy(((struct v4l2_dbg_chip_ident *)id)->match.name,
		"ovmax9286_mipi_deseiralizer");

	return 0;
}

/*!
 * ioctl_init - V4L2 sensor interface handler for VIDIOC_INT_INIT
 * @s: pointer to standard V4L2 device structure
 */
static int ioctl_init(struct v4l2_int_device *s)
{
	return 0;
}

/*!
 * ioctl_enum_fmt_cap - V4L2 sensor interface handler for VIDIOC_ENUM_FMT
 * @s: pointer to standard V4L2 device structure
 * @fmt: pointer to standard V4L2 fmt description structure
 *
 * Return 0.
 */
static int ioctl_enum_fmt_cap(struct v4l2_int_device *s,
			      struct v4l2_fmtdesc *fmt)
{
	struct sensor_data *sensor = s->priv;

	if (fmt->index > 0) /* only 1 pixelformat support so far */
		return -EINVAL;

	fmt->pixelformat = sensor->pix.pixelformat;

	return 0;
}

/*!
 * ioctl_dev_init - V4L2 sensor interface handler for vidioc_int_dev_init_num
 * @s: pointer to standard V4L2 device structure
 *
 * Initialise the device when slave attaches to the master.
 */
static int ioctl_dev_init(struct v4l2_int_device *s)
{
	struct sensor_data *sensor = s->priv;
	int ret = 0;
	void *mipi_csi2_info;

	sensor->on = true;

	if (sensor->i2c_client != NULL) {
		mipi_csi2_info = mipi_csi2_get_info();

		/* enable mipi csi2 */
		if (mipi_csi2_info)
			mipi_csi2_enable(mipi_csi2_info);
		else {
			printk(KERN_ERR "%s() in %s: Fail to get mipi_csi2_info!\n",
			       __func__, __FILE__);
			return -EPERM;
		}

		ret = max9286_hardware_init(sensor);
	}

	return ret;
}

/*!
 * ioctl_dev_exit - V4L2 sensor interface handler for vidioc_int_dev_exit_num
 * @s: pointer to standard V4L2 device structure
 *
 * Delinitialise the device when slave detaches to the master.
 */
static int ioctl_dev_exit(struct v4l2_int_device *s)
{
	struct sensor_data *sensor = s->priv;
	void *mipi_csi2_info;

	if (sensor->i2c_client != NULL) {
		mipi_csi2_info = mipi_csi2_get_info();

		/* disable mipi csi2 */
		if (mipi_csi2_info)
			if (mipi_csi2_get_status(mipi_csi2_info))
				mipi_csi2_disable(mipi_csi2_info);
	}

	return 0;
}

/*!
 * This structure defines all the ioctls for this module and links them to the
 * enumeration.
 */
static struct v4l2_int_ioctl_desc max9286_ioctl_desc[] = {
	{vidioc_int_dev_init_num, (v4l2_int_ioctl_func *) ioctl_dev_init},
	{vidioc_int_dev_exit_num, ioctl_dev_exit},
	{vidioc_int_s_power_num, (v4l2_int_ioctl_func *) ioctl_s_power},
	{vidioc_int_g_ifparm_num, (v4l2_int_ioctl_func *) ioctl_g_ifparm},
/*	{vidioc_int_g_needs_reset_num,
				(v4l2_int_ioctl_func *)ioctl_g_needs_reset}, */
/*	{vidioc_int_reset_num, (v4l2_int_ioctl_func *)ioctl_reset}, */
	{vidioc_int_init_num, (v4l2_int_ioctl_func *) ioctl_init},
	{vidioc_int_enum_fmt_cap_num,
				(v4l2_int_ioctl_func *) ioctl_enum_fmt_cap},
	{vidioc_int_try_fmt_cap_num,
				(v4l2_int_ioctl_func *)ioctl_try_fmt_cap},
	{vidioc_int_g_fmt_cap_num, (v4l2_int_ioctl_func *) ioctl_g_fmt_cap},
/*	{vidioc_int_s_fmt_cap_num, (v4l2_int_ioctl_func *) ioctl_s_fmt_cap}, */
	{vidioc_int_g_parm_num, (v4l2_int_ioctl_func *) ioctl_g_parm},
	{vidioc_int_s_parm_num, (v4l2_int_ioctl_func *) ioctl_s_parm},
/*	{vidioc_int_queryctrl_num, (v4l2_int_ioctl_func *)ioctl_queryctrl}, */
	{vidioc_int_g_ctrl_num, (v4l2_int_ioctl_func *) ioctl_g_ctrl},
	{vidioc_int_s_ctrl_num, (v4l2_int_ioctl_func *) ioctl_s_ctrl},
	{vidioc_int_enum_framesizes_num,
				(v4l2_int_ioctl_func *) ioctl_enum_framesizes},
	{vidioc_int_enum_frameintervals_num,
				(v4l2_int_ioctl_func *) ioctl_enum_frameintervals},
	{vidioc_int_g_chip_ident_num,
				(v4l2_int_ioctl_func *) ioctl_g_chip_ident},
};

static struct v4l2_int_slave max9286_slave[MAX_SENSOR_NUM] = {
	{
	.ioctls = max9286_ioctl_desc,
	.num_ioctls = ARRAY_SIZE(max9286_ioctl_desc),
	},

	{
	.ioctls = max9286_ioctl_desc,
	.num_ioctls = ARRAY_SIZE(max9286_ioctl_desc),
	},

	{
	.ioctls = max9286_ioctl_desc,
	.num_ioctls = ARRAY_SIZE(max9286_ioctl_desc),
	},

	{
	.ioctls = max9286_ioctl_desc,
	.num_ioctls = ARRAY_SIZE(max9286_ioctl_desc),
	}
};

static struct v4l2_int_device max9286_int_device[MAX_SENSOR_NUM] = {
	{
		.module = THIS_MODULE,
		.name = "max9286",
		.type = v4l2_int_type_slave,
		.u = {
			.slave = &max9286_slave[0],
		},
	}, 

	{
		.module = THIS_MODULE,
		.name = "max9286",
		.type = v4l2_int_type_slave,
		.u = {
			.slave = &max9286_slave[1],
		},
	}, 

	{
		.module = THIS_MODULE,
		.name = "max9286",
		.type = v4l2_int_type_slave,
		.u = {
			.slave = &max9286_slave[2],
		},
	}, 

	{
		.module = THIS_MODULE,
		.name = "max9286",
		.type = v4l2_int_type_slave,
		.u = {
			.slave = &max9286_slave[3],
		},
	}
};

/*!
 * max9286 I2C probe function
 *
 * @param adapter            struct i2c_adapter *
 * @return  Error code indicating success or failure
 */
static int max9286_probe(struct i2c_client *client,
			const struct i2c_device_id *id)
{
	struct device *dev = &client->dev;
	int retval;

	/* Set initial values for the sensor struct. */
	memset(&max9286_data[0], 0, sizeof(max9286_data[0]));
	max9286_data[0].sensor_clk = devm_clk_get(dev, "csi_mclk");
	if (IS_ERR(max9286_data[0].sensor_clk)) {
		/* assuming clock enabled by default */
		max9286_data[0].sensor_clk = NULL;
		dev_err(dev, "clock-frequency missing or invalid\n");
		return PTR_ERR(max9286_data[0].sensor_clk);
	}

	retval = of_property_read_u32(dev->of_node, "mclk",
					&(max9286_data[0].mclk));
	if (retval) {
		dev_err(dev, "mclk missing or invalid\n");
		return retval;
	}

	retval = of_property_read_u32(dev->of_node, "mclk_source",
					(u32 *) &(max9286_data[0].mclk_source));
	if (retval) {
		dev_err(dev, "mclk_source missing or invalid\n");
		return retval;
	}

	retval = of_property_read_u32(dev->of_node, "csi_id",
					&(max9286_data[0].csi));
	if (retval) {
		dev_err(dev, "csi id missing or invalid\n");
		return retval;
	}

	clk_prepare_enable(max9286_data[0].sensor_clk);

	max9286_data[0].i2c_client = client;
	max9286_data[0].pix.pixelformat = V4L2_PIX_FMT_UYVY;
	max9286_data[0].pix.width = g_max9286_width;
	max9286_data[0].pix.height = g_max9286_height;
	max9286_data[0].streamcap.capturemode = 0;
	max9286_data[0].streamcap.timeperframe.denominator = 30;
	max9286_data[0].streamcap.timeperframe.numerator = 1;
	max9286_data[0].is_mipi = 1;

	retval = max9286_read_reg(0x1e);
	if (retval != 0x40) {
		pr_warning("max9286 is not found, chip id reg 0x1e = 0x%x.\n", retval);
		clk_disable_unprepare(max9286_data[0].sensor_clk);
		return -ENODEV;
	}

	memcpy(&max9286_data[1], &max9286_data[0], sizeof(struct sensor_data));
	memcpy(&max9286_data[2], &max9286_data[0], sizeof(struct sensor_data));
	memcpy(&max9286_data[3], &max9286_data[0], sizeof(struct sensor_data));

	max9286_data[1].i2c_client = NULL;
	max9286_data[2].i2c_client = NULL;
	max9286_data[3].i2c_client = NULL;

	max9286_data[0].ipu_id = 0;
	max9286_data[0].csi = 0;
	max9286_data[0].v_channel = 0;

	max9286_data[1].ipu_id = 0;
	max9286_data[1].csi = 1;
	max9286_data[1].v_channel = 1;

	max9286_data[2].ipu_id = 1;
	max9286_data[2].csi = 0;
	max9286_data[2].v_channel = 2;

	max9286_data[3].ipu_id = 1;
	max9286_data[3].csi = 1;
	max9286_data[3].v_channel = 3;

	max9286_int_device[0].priv = &max9286_data[0];
	max9286_int_device[1].priv = &max9286_data[1];
	max9286_int_device[2].priv = &max9286_data[2];
	max9286_int_device[3].priv = &max9286_data[3];
	v4l2_int_device_register(&max9286_int_device[0]);
	v4l2_int_device_register(&max9286_int_device[1]);
	v4l2_int_device_register(&max9286_int_device[2]);
	retval = v4l2_int_device_register(&max9286_int_device[3]);

	clk_disable_unprepare(max9286_data[0].sensor_clk);

	pr_info("max9286_mipi is found\n");
	return retval;
}

/*!
 * max9286 I2C detach function
 *
 * @param client            struct i2c_client *
 * @return  Error code indicating success or failure
 */
static int max9286_remove(struct i2c_client *client)
{
	v4l2_int_device_unregister(&max9286_int_device[3]);
	v4l2_int_device_unregister(&max9286_int_device[2]);
	v4l2_int_device_unregister(&max9286_int_device[1]);
	v4l2_int_device_unregister(&max9286_int_device[0]);

	return 0;
}

/*!
 * max9286 init function
 *
 * @return  Error code indicating success or failure
 */
static __init int max9286_init(void)
{
	u8 err;

	err = i2c_add_driver(&max9286_i2c_driver);
	if (err != 0)
		pr_err("%s:driver registration failed, error=%d\n",
			__func__, err);

	return err;
}

/*!
 * MAX9286 cleanup function
 *
 * @return  Error code indicating success or failure
 */
static void __exit max9286_clean(void)
{
	i2c_del_driver(&max9286_i2c_driver);
}

module_init(max9286_init);
module_exit(max9286_clean);

MODULE_AUTHOR("Freescale Semiconductor, Inc.");
MODULE_DESCRIPTION("MAX9286 GSML Deserializer Driver");
MODULE_LICENSE("GPL");
MODULE_VERSION("1.0");
MODULE_ALIAS("CSI");
